/*---------------------------------------------------------------------------*/
/*!
 * @file Clustering.cpp
 * @brief Gaussian mixture modeling of a multidimensionnal spaces by the MDL principle\n FOR ANY EXTENSION, REQUIRES TO BE RE-IMPLEMENTED COMPLETLY
 *
 *
 * This is the main program for infering by MDL using a data 2-part coding,  a Gaussian mixture model of unknown complexity (paramters and number of Gaussians).\n 
 * Modeling procedures can be chained. For clustering consecutive TL feature spaces, the initialization is done using the Gaussian parameters infered for the previous modeling. Furthermore, the algorithm checks that there is a sufficient number of Gaussians (fixed by the user) after an initialisation, and adds new Gaussian if necessary. \n
 * This is the main program which incorporates the users input/output parameters and launch the optimisation procedure by intializing and using methods of the object Temporal_MDL_classifier.\n
* \n 
 *This algorithm, which makes the hypothesis that the Gaussian in the mixture are sufficiently separatyed to not interfer, may be extended to the general case.\n
 *However, to extend the modeling to the general case, the whole program seriously REQUIRES TO BE RE-CODED (This program was my first try in C++ so the developpment  was chaotic!!)\n
 *
 *\n
  *
 *
 * @author P.Has (IRIT / ENSAE / DLR / CNES)
 * @since January 2005
 *
 */
/*---------------------------------------------------------------------------*/
#include <iostream>
#include <string>  
#include <vector>
#include <fstream>

#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cassert>
#include <ctime>

#include "./TimeGaussMIX_MDL.hh"
 
 
/*! 	@brief  Usage function 	*/ 
void usage(std::string argv0) {
    std::cout << "\nusage: 				" << ">"
              << argv0 
              << "inputImageFileName outputImageFileName nbBands nbLines nbColumns " << std::endl; 
    std::cout << "option for spatial window:	"
    	      << " offsetLines offsetColumns OriginalNbColumns"<< std::endl;
    std::cout << "option for subsampling :	 " 
    	      << "sub_sampleLines sub_sampleColumns "<< std::endl;
    std::cout << "option for initializing :	 "
    	      << "nb of initial Gaussians"<< std::endl;
  }

/*!	@brief Main procedure for Gaussian mixture modeling.
*
*	The main procedure begins by a creating a Temporal_MDL_classifier object. After initializing this object, its method "Cluster_optimiser" is used to search the model which induces the minimum description length of the data. This method produces  also the output classfile.
	@param argv0[1]  Text file name of input multiband-images : the image bands of the different images are listed one after the other; The number of lines of this file must be (numberImagesXnumberBands)   
*    	@param argv0[2]  Text file name of output image classfiles 
*	@param argv0[3]  Number of bands of each image 
*	@param argv0[4]  Number of lines of the images
*	@param argv0[5]  Number of columns of the images 
*	
*	Optional parameters for selecting a spatial window  :
*	@param argv0[6]  Column offset when reading the image file 
*	@param argv0[7]  Line offset when reading the image file
*	@param argv0[8]  Number of columns of the image file 

	Optional parameters for spatial subsampling   :
*	@param argv0[9]   factor to reduce each line (factor 1 = no subsampling)
*	@param argv0[10]  factor to reduce each column (factor 1 = no subsampling)
*
*	Optional parameters for Initialization  :
*	@param argv0[11]  Minimum number of Gaussians in the mixture for initialization
	
*/	
int main(int argc, char *argv []){   
 
 
  
 
std::string ifn1;
std::string ifn2;
unsigned long   int         nb_bands	   = 0;
unsigned long   int         dimx           = 0;
unsigned long   int         dimy	   = 0;
unsigned long   int         ofx		   = 0;
unsigned long   int         ofy		   = 0;
unsigned long   int         Dy	 	   = 0;
unsigned long   int         subx	   = 1;
unsigned long   int         suby	   = 1;
int			    nb_init	   =-1;
switch (argc-1) {
	case 11: nb_init 		= atoi(argv[11]);
    	case 10: suby 			= atoi(argv[10]);
	case 9:  subx  			= atoi(argv[9]);
        case 8:  Dy 			= atoi(argv[8]);
	case 7:  ofy  			= atoi(argv[7]);
        case 6:  ofx   			= atoi(argv[6]);
        case 5:  dimy       		= atoi(argv[5]);
        case 4:  dimx       		= atoi(argv[4]);
	case 3:  nb_bands   		= atoi(argv[3]);
        case 2:  ifn2	 		= std::string(argv[2]);
        case 1:  ifn1	 		= std::string(argv[1]); break;
        case 0:
        default: usage(argv[0]); return -1;
    }
    if ((argc-1) < 5) { usage(argv[0]); return -1;}
    if ((argc-1) == 5) {Dy=dimy;} 
    std::cout << "> " << argv[0] 
              << " " << ifn1 
              << " " << ifn2 
	      << " " << nb_bands 
              << " " << dimx 
              << " " << dimy 
              << " " << ofx
	      << " " << ofy  
              << " " << Dy 
	      << " " << subx  
              << " " << suby
	      << " " << nb_init<< std::endl;
	 
/*******MDL clasiffications for image time series **********************************************************/
 	srand((unsigned)time(NULL));
	time_t tdeb, t1,t2,t3;
	time(&tdeb);
	t1=tdeb;
        Temporal_MDL_classifier test;
	test.Initialise((char *)ifn1.c_str(),(char *)ifn2.c_str(),dimx,dimy,nb_bands,ofx,ofy,Dy,subx,suby,nb_init);
	time(&tdeb);
	t2=tdeb;
	cout<< "Inatializing time duration : "<<t2-t1<<" seconds."<<endl;
	test.Cluster_optimiser();
	time(&t3);
	cout<< "Processing time duration : "<<t3-t2<<" seconds."<<endl;
	return 0;
}
