/*---------------------------------------------------------------------------*/
/*!
 * @file Image_basics.hh
 * @brief  File defining a tool object for dealing with many images of variable format\n
 * FOR ANY EXTENSION, REQUIRES TO BE RE-IMPLEMENTED COMPLETLY*/
/*---------------------------------------------------------------------------*/
#include <iostream>
#include <fstream>
#include <string>  
#include <vector>

#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cassert>
using namespace std;

#define InfINI 1e20

template<class PIX>
/*!@brief tool object for dealing with many images of variable format\n
 * FOR ANY EXTENSION, REQUIRES TO BE RE-IMPLEMENTED COMPLETLY*/
class Image {
 	
 unsigned long int i,j,k,l,n,m;
 public:
 	/*!@brief number of lines in an image*/
 	int nx;
	/*!@brief number of column in an image*/
	int ny;
	/*!@brief number of bands of each multispectral image*/
	int nb_bands;
	/*!@brief number of images in the steack*/
	int nb_images;
	/*!@brief pointer to the first element of the array containing the data*/
	PIX *position;
	/*!@brief constructor*/
	Image()
	{
		nx=0;
		ny=0;
		nb_images=0;
		nb_bands=0;
		nb_dim=0;
		dim=0;
		/*! pointer to first memory location*/
		position=NULL;
		
	}
		
	/*!@brief Allocating method for several multispectral images*/
	void Allocate(int nx_temp, int ny_temp, int nb_bands_temp, int nb_images_temp)
	{
		nx=nx_temp;
		ny=ny_temp;
		nb_bands=nb_bands_temp;
		nb_images=nb_images_temp;
		nb_dim=nb_bands*nb_images;
		dim=nx*ny;
		PIX *Y= new PIX[nb_dim*dim]; 	
		position=&Y[0];
	}
	
	//virtual ~Image()
		//{		
		//delete [] position;	
		//}
		
	/*!@brief method returning a pixel value located in space on a given band */
	PIX Get_pixel_value(char *path,int ligne, int colonne, int ny)// Monoband image in file only
		
	{
		PIX a[1];
		FILE * ifp= NULL;
		assert(ifp=fopen(path,"r"));
		assert(0==fseek(ifp,(sizeof(PIX)*ny*ligne+sizeof(PIX)*colonne), SEEK_SET));
		assert(1==fread(&a[0],sizeof(PIX),1,ifp));		
		fclose(ifp);
		return a[0];	
	}
	/*!@brief Reading the input paths*/ 
	void Read_pathfiles_in (char *pathfile_in,char *** path_images_in, int nb_bands_temp)
	{
		
        string line;
		ifstream inaux;
		inaux.open(pathfile_in,ifstream::in);
		ifstream in;
		in.open(pathfile_in,ifstream::in);
		
		
		n=0;
		while(getline(inaux,line)) {n++;} 
		nb_bands=nb_bands_temp;
	        nb_images=n/nb_bands;  
		nb_dim=nb_images*nb_bands;
		std::cout << "number of images :" << nb_images<< ",  nb_bands : " << nb_bands <<std::endl;
		std::cout<<" input path : "<<std::endl;;
		for (i=0;i<(unsigned long int)nb_images;i++){
		  for (j=0;j<(unsigned long int)nb_bands;j++){
 		   	getline(in,line);
		  	strcpy(path_images_in[i][j],(char *)line.c_str());
		  	std::cout<<path_images_in[i][j]<<std::endl;
 		 }	
		}	
		
	}
	/*!@brief  function returning the number of line in the text file*/
	int NumberImages_pathfiles_in (char *pathfile_in, int nb_bands_temp)
	{
		
		string line;
		ifstream in;
		in.open(pathfile_in,ifstream::in);
		
		int n=0;
		while (getline(in,line)) {n++;}
		nb_bands=nb_bands_temp;
	        nb_images=n/nb_bands;  
		nb_dim=nb_images*nb_bands;
		
		return nb_images;
	}
	/*!@brief  Reading the output paths*/ 
	void Read_pathfiles_out (char *pathfile_out, char ***path_images_out)
	{
		
		ifstream out;
		out.open(pathfile_out,ifstream::in);
			
		string line;
		for (i=0;i<(unsigned long int)nb_images;i++){
  			for (j=0;j<(unsigned long int)nb_bands;j++){
    				getline(out,line);
				strcpy(path_images_out[i][j],(char *)line.c_str());
				std::cout << "save path : " << path_images_out[i][j]<< std::endl;
  			}
		}
	
	}
	/*!@brief  Reading the image spatial window with the given  subsampling factors*/
	void Read_Cut_SubSample(char ***path_images_in,int ofx,int ofy,int Dy,int subx, int suby)
	{
		
		unsigned long  int jump=0;
		int oldny=(int)(ny*suby);
		
		std::cout << "size image output:  " <<nx <<"x"<< ny << "x"<< nb_images  <<std::endl;
		
		

		PIX **a= new PIX *[nx]; //vector clust contains the data
        	for(m=0;m<(unsigned long int)nx;++m)
          	   a[m] =new PIX[ny];	 
	
       	 	char *ifn=new char[128];
         	FILE * ifp;
	 	m=0;n=0;
	 	for (l = 0 ;l <(unsigned long int)nb_dim; l++){
		
	   		strcpy(ifn,path_images_in[m][n]);std::cout<<"Reading image "<<ifn<<std::endl; 
	   		ifp=NULL;
	   		assert (ifp=fopen(ifn,"r"));
	   		assert (0==fseek(ifp,(sizeof(PIX)*Dy*ofx+sizeof(PIX)*ofy), SEEK_SET));
	   		for (jump = 0 ;jump <(unsigned long int)ny; jump++){
           			assert (1==fread(&a[0][jump],sizeof (PIX),1,ifp));    
	   			assert (0==fseek(ifp,(sizeof(PIX)*(suby-1)), SEEK_CUR));
	   		}	   
	   		for (i=1;i<(unsigned long int)nx;i++){
	     			assert (0==fseek(ifp,(sizeof(PIX)*(Dy-oldny)+sizeof(PIX)*Dy*(subx-1)), SEEK_CUR));	     
	     			for (jump = 0 ;jump <(unsigned long int)ny; jump++){
           				assert (1==fread(&a[i][jump],sizeof (PIX),1,ifp));    
	        			assert (0==fseek(ifp,(sizeof(PIX)*(suby-1)), SEEK_CUR));
	     			}
	     
	   		}
	   		fclose(ifp); 
	   		if (n==(unsigned long int)nb_bands-1){n=0;m++;}
	   		else{n++;}
	   		for (i = 0 ;i < (unsigned long int)nx; i++){ k=(ny)*i;          
	   	  		for (j = 0; j <(unsigned long int)ny; j++) { 	
	       				*(position+l*dim+k)=a[i][j];
					k=k+1;
	  	   		}
	   		}
	 	}   
	 	for(m=0;m<(unsigned long int)nx;++m) {delete [] a[m];}
	 	delete [] a;    
		delete [] ifn;
		
	}
	
	/*!@brief Method to write a steack of multi-spectral images */
	void Write(char ***path_images_out)
	{
		PIX *sauv= new PIX [dim];
		FILE *ifp;
		char *ifn;
		  for(m=0;m<(unsigned long int)nb_images;m++){
  			  for(l=0;l<(unsigned long int)nb_bands;l++){ 
  			    	for (i=0;i<dim;i++){sauv[i]=*(position+dim*(m+l)+i); }	
  		            	ifp=NULL; 
  			    	ifn=path_images_out[m][l];
     			    	assert (ifp=fopen(ifn,"wb"));
     			    	assert ( dim==fwrite(&sauv[0], sizeof(PIX),dim,ifp));
     			    	assert (EOF!=fclose(ifp)); 
   			  }
  		   }
		 delete [] sauv;
	}
	/*!@brief Method to write a panchromatic image */
	void Write_singleImage(char *path_images_out)
	{
		PIX *sauv= new PIX[dim];
		FILE *ifp;
		char *ifn;
		  for(m=0;m<(unsigned long int)nb_images;m++){
  			  for(l=0;l<(unsigned long int)nb_bands;l++){ 
  			    	for (i=0;i<dim;i++){sauv[i]=*(position+dim*(m+l)+i);}	
  		            	ifp=NULL; 
  			    	ifn=path_images_out;
     			    	assert (ifp=fopen(ifn,"wb"));
     			   	assert ( dim==fwrite(&sauv[0], sizeof(PIX),dim,ifp));
     			    	assert (EOF!=fclose(ifp)); 
   			  }
  		   }
		   
		 delete [] sauv;
	}
    PIX GetPixel(int band, int x, int y){return *(position+dim*(band)+y*nx +x);}
	void PutPixel(int band, int x, int y, PIX value){ *(position+dim*(band)+y*nx +x)=value;}

	/*!@brief Method returning the maximum value of an image */
	float MAX(int image_nb_m)
	{
		float m=-InfINI,aux;
  		for (i=0;i<dim;i++){aux=*(position+dim*(image_nb_m)+i);if (aux> m){m=aux;}}
		return m;  	
	}
	/*!@brief Method returning the minimum value of an image */
	float MIN(int image_nb_m)
	{
		float m=InfINI,aux;
  		for (i=0;i<dim;i++){aux=*(position+dim*(image_nb_m)+i);if (aux< m){m=aux;}}
		return m;  	
	}
	/*!@brief Method returning the mean value of an image */
	float Mean(int image_nb_m)
	{
		 float sum=0,aux;
  		for (i=0;i<dim;i++){sum=*(position+dim*(image_nb_m)+i)+sum; }
		aux=sum/(float)dim;
		return aux;  	
	}
	/*!@brief Method returning the standard deviation value of an image */
	float Stdev(int image_nb_m)
	{
		 float sum=0,aux,moy;
		 moy=Mean(image_nb_m);
  		for (i=0;i<dim;i++){sum=(*(position+dim*(image_nb_m)+i)-moy)*(*(position+dim*(image_nb_m)+i)-moy)+sum; }	  
		aux=sqrt(sum/(float)dim);
		return aux;  	
	}
	/*!@brief Method returning the covriance value between 2 images */
	float Cov(int image_nb_m1,int image_nb_m2)
	{
		 float sum=0,aux,moy1,moy2;
		 moy1=Mean(image_nb_m1);
		 moy2=Mean(image_nb_m2);
		 
  		for (i=0;i<dim;i++){sum=(*(position+dim*(image_nb_m1)+i)-moy1)*(*(position+dim*(image_nb_m2)+i)-moy2)+sum; }	  
		aux=(sum/(float)dim);
		return aux;  	
	}
	/*!@brief Method nomalizing each image in a steack of images */
	void  Normalize()
	{
		  float moy, dev;
		  PIX aux;
		  for(int m=0;m<nb_images*nb_bands;m++){
  			moy=Mean(m);
			dev=Stdev(m);
			for (i=0;i<dim;i++)
			{
				aux=GetPixel(m,i,0);
				PutPixel(m,i,0,(PIX)(aux-moy)/dev);	
			}
		  }
		  
		
	}


private:
		
		/*!@brief dimensionnality */
		int nb_dim;
		/*!@brief number of samples (pixels) in an image */
		unsigned long int dim;
		
		/*!@brief method returning the minimum of two pixel values*/
		PIX Min(PIX a,PIX b)
		{
			PIX res;
			if (a<b){res=a;}
			else{res=b;}
			return res;
		
		}

	};
	
			
