/*---------------------------------------------------------------------------*/
/*!
 * @file Inliers.cpp
 * @brief File defining the Inliers object methods.\n
 FOR ANY EXTENSION, THIS OBJECT REQUIRES TO BE RE-IMPLEMENTED 
 *
 */
 /*---------------------------------------------------------------------------*/
#include <iostream>
#include <string>  
#include <vector>
#include <fstream>

#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cassert>

#include "./Inliers.hh"

#define Inf 1.0e20
#define TINY 1.0e-20

using namespace std;

Inliers::Inliers(){};
Inliers::Inliers(Image<PIXEL>* data,Outliers * O,int nb)
{        
	M_LOG2=log(2);
	Initialise(data,O,nb);
	nb_inliers=(im_point->nx)*(im_point->ny);
	nb_init=nb;
	Allocate_Variable_StdDev();//memory allocation
	Initialise_IndexInliers();//usefull when the outliers are considered, SHOULD BE REMOVED!!

	Assigne_data_Euclidian();//assignement of the data points according to their euclidian distance to the Gaussian Means.
	Update_effectif();// 
	Remove_Unaffected_RefVector(0);
	Update_Centers();
	Estimation_Stdev_RefVector();
	Adaptation();
	
	int count=0;
        while (A[0].nb_ref<nb_init)//while there are not enough components in the mixture...iterate the procedure
        {
		cout<<A[0].nb_ref<<"  <  "<<nb_init<<endl;
       		
		A[0].AddRefVectors(nb_init);
		Assigne_data_Euclidian();
		Update_effectif();
		Remove_Unaffected_RefVector(0);
		Update_Centers();
		Estimation_Stdev_RefVector();
		Adaptation();
		count++;
		if (count>10){break;}
	}
	MinMax();
	cout<<"End of Reference Vector Initialization"<<endl;			 
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Initialise(Image<PIXEL> * data_point,Outliers * O,int nb)
{
	im_point=data_point;
	O_point=O;
	//A=new Ref_Vector[1];A[0].Initialise(im_point,nb);
    Ref_Vector tmp(im_point,nb);
    A.push_back(tmp);
	nbMeanAdapt=0;
	nbMeanIt=0;
	
	maxi_dim=new float[im_point->nb_bands];
	mini_dim=new float[im_point->nb_bands];
	MinMax();
	
		
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Adaptation()
{	
	ChangingCriterion=Inf;// varaible to check how good the algorithm has converged 
	int i=0;
	float range=0;
	for (int k=0;k<im_point->nb_bands;k++){range+=(maxi_dim[k]-mini_dim[k]);}
	range=range/im_point->nb_bands;
	while(ChangingCriterion>(im_point->nb_bands)*A[0].nb_ref*range/(1e3)) //precision inversely proportional to the number of components
	{
		NearestRefVector_StdDevVar();
		Update_Centers();
		Estimation_Stdev_RefVector();
		i++;
		if (i>10){break;}
	}
	nbMeanIt+=i;
	nbMeanAdapt++;
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Selection()
{
	Update_RefVectors();	
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Outliers_detection()
{	
	//Update_outliers();
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::AddRandomRefVectors()
{
        A[0].AddRefVectors(nb_init);
       	ChangingCriterion=Inf;
	Assigne_data_Euclidian();
	Update_effectif();
	Remove_Unaffected_RefVector(0);
	float range=0;
	for (int k=0;k<im_point->nb_bands;k++){range+=(maxi_dim[k]-mini_dim[k]);}
	range=range/im_point->nb_bands;
	while(ChangingCriterion>(im_point->nb_bands)*A[0].nb_ref*range/(1e3))
	{	
		Update_Centers();
		Estimation_Stdev_RefVector();
	        Reassigne_inliers_StdDevVariable();
		Remove_Unaffected_RefVector(1);
	}
	int count=0;
        while (A[0].nb_ref<nb_init)
        {
		cout<<A[0].nb_ref<<"  <=   "<<nb_init<<endl;
       		ChangingCriterion=Inf;
       		
		A[0].AddRefVectors(nb_init);
		Assigne_data_Euclidian();
		Update_effectif();
		Remove_Unaffected_RefVector(0);
		while(ChangingCriterion>(im_point->nb_bands)*A[0].nb_ref*range/(1e3))
       		{
			Update_Centers();
			Estimation_Stdev_RefVector();
	        	Reassigne_inliers_StdDevVariable();
			Remove_Unaffected_RefVector(1);
       		}
		count++;
		if (count>10){break;}
	}
	Estimation_Stdev_RefVector();
       MinMax();
       cout<<"End of Reference Vector Re-initialization"<<endl;
}
/*--------------------------------------------------------------------------------------------*/		 
void Inliers::Allocate_Variable_StdDev()
{
	int k,l,j;	
	unsigned long int dim=(im_point->nx)*(im_point->ny);
	if (nb_init==-1){nb_init=(int)(log((double)dim))*3.5*log((double)im_point->nb_bands)*2;}
	nb_init=(int)nb_init*2;
	nb_inliers=dim;	 	
	 
	firstNearestA=new int[dim];secondNearestA=new int[dim];
	cross_effectif_cluster=new unsigned long int*[nb_init];
	effectif_cluster=new unsigned long int[nb_init];
	covariance_cluster=new float**[nb_init];
	Inv_covariance_cluster=new float**[nb_init];
	covariance_cluster_PostRemoval=new float***[nb_init];
	Inv_covariance_cluster_PostRemoval=new float***[nb_init];
	center_RefVector_PostRemoval=new float**[nb_init];
	determinant_PostRemoval=new float*[nb_init];
	determinant=new float[nb_init];
	singularity=new int[nb_init];
	Singularity_PostRemoval= new int*[nb_init];
	Index_Inliers=new unsigned long int[dim];
	
	for (k=0;k<nb_init;k++)
	{	
		cross_effectif_cluster[k]=new unsigned long int[nb_init];
		determinant_PostRemoval[k]=new float[nb_init];
		covariance_cluster[k]=new float*[im_point->nb_bands];
		Inv_covariance_cluster[k]=new float*[im_point->nb_bands];
		covariance_cluster_PostRemoval[k]=new float**[nb_init];
		Inv_covariance_cluster_PostRemoval[k]=new float**[nb_init];
		center_RefVector_PostRemoval[k]=new float*[nb_init];
		Singularity_PostRemoval[k]=new int[nb_init];
	}
	for (k=0;k<nb_init;k++)
	{
		for (l=0;l<nb_init;l++)
		{
			covariance_cluster_PostRemoval[k][l]=new float*[im_point->nb_bands];
			Inv_covariance_cluster_PostRemoval[k][l]=new float*[im_point->nb_bands];
			center_RefVector_PostRemoval[k][l]=new float[im_point->nb_bands];

		}
		for(j=0;j<im_point->nb_bands;j++)
		{
			covariance_cluster[k][j]=new float[im_point->nb_bands];
			Inv_covariance_cluster[k][j]=new float[im_point->nb_bands];
		}
	}
	for (k=0;k<nb_init;k++)
	{
		for (l=0;l<nb_init;l++)
		{
			for(j=0;j<im_point->nb_bands;j++)
			{
			covariance_cluster_PostRemoval[l][k][j]=new float[im_point->nb_bands];
			Inv_covariance_cluster_PostRemoval[l][k][j]=new float[im_point->nb_bands];
			}
		}
	}
	nb_init=(int)((float)nb_init/2.);
			
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::MinMax()
{
	unsigned long int i;
	int j;
	for (j=0;j<im_point->nb_bands;j++){maxi_dim[j]=0;mini_dim[j]=Inf;}
	unsigned long int dim=(unsigned long int)(im_point->nx)*(unsigned long int)(im_point->ny);
	for (i=0;i<dim;i++)
	{
		for (j=0;j<im_point->nb_bands;j++){
			if (maxi_dim[j]<(float)*(im_point->position+i+dim*j))
				{maxi_dim[j]=(float)*(im_point->position+i+dim*j);}
			if (mini_dim[j]>(float)*(im_point->position+i+dim*j))
				{mini_dim[j]=(float)*(im_point->position+i+dim*j);}
		}
	}
	
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Initialise_IndexInliers()
{	
	unsigned long int dim=(unsigned long int)(im_point->nx)*(unsigned long int)(im_point->ny);
	for (unsigned long int i=0; i<dim;i++){Index_Inliers[i]=i;}
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Assigne_data_Euclidian()
{
	int k;
	unsigned long int i;
	float max,aux;
			
	for (i=0;i<nb_inliers;i++){
		max=-Inf;
		for (k=0;k<A[0].nb_ref;k++){ 
			aux=Proba_Assigne_Euclidian(i,k);
			if (aux>max){
				max=aux;
				firstNearestA[i]=k;
			}
		}
	}
	
		
}
		
/*--------------------------------------------------------------------------------------------*/
float Inliers::Proba_Assigne_Euclidian(unsigned long int i, int k)
{
	float aux=0;
	unsigned long int dim=(im_point->nx)*(im_point->ny);
	for (int j=0;j<(im_point->nb_bands);j++){
		aux=(float)(*(A[0][k]+j)-(float)*(im_point->position+Index_Inliers[i]+dim*j))*(float)(*(A[0][k]+j)-*(im_point->position+Index_Inliers[i]+dim*j))+aux;	
	}
			
	return 1.0/aux;

}/*--------------------------------------------------------------------------------------------*/
float Inliers::	Proba_Assigne_Gauss_PostRemoval( unsigned long int i, int k, int l)
{

    float aux=1.;
    int j,j1,j2;    
    unsigned long int dim=(im_point->nx)*(im_point->ny);
    float auxbis=0;
    float *auxligne = new float[im_point->nb_bands];
    
    // no singularity in the covariance matrix
    if (Singularity_PostRemoval[k][l]!=1) 
    {
    	aux=sqrt(determinant_PostRemoval[k][l])/pow(2.0*M_PI,(double)(im_point->nb_bands/2.));
	for (j1=0;j1<(im_point->nb_bands);j1++){
		auxligne[j1]=0;
		for (j2=0;j2<(im_point->nb_bands);j2++){	
		auxligne[j1]=((float)*(im_point->position+Index_Inliers[i]+dim*j2)-(center_RefVector_PostRemoval[k][l][j2]))*Inv_covariance_cluster_PostRemoval[k][l][j1][j2]+auxligne[j1];								}
	}
	for (j1=0;j1<(im_point->nb_bands);j1++){
		auxbis=auxbis+auxligne[j1]*((float)*(im_point->position+Index_Inliers[i]+dim*j1)-(center_RefVector_PostRemoval[k][l][j1]));
					}
    aux=aux*exp(-0.5*auxbis);
    }
    
    //singularity : independence intra bande simplification
    else
    {
    	aux=1;
    	for (j=0;j<(im_point->nb_bands);j++)
   	{
		if (covariance_cluster_PostRemoval[k][l][j][j]!=0){
				
aux=1.0/(sqrt(covariance_cluster_PostRemoval[k][l][j][j]*2.0*M_PI))*exp(-((center_RefVector_PostRemoval[k][l][j])-(float)*(im_point->position+Index_Inliers[i]+dim*j))*((center_RefVector_PostRemoval[k][l][j])-(float)*(im_point->position+Index_Inliers[i]+dim*j))/(2*covariance_cluster_PostRemoval[k][l][j][j]))*aux;	
			}
		else
		{				
		if((float)*(im_point->position+Index_Inliers[i]+dim*j)!=(center_RefVector_PostRemoval[k][l][j])){aux=0;}
		}
    	}
    
    }
				
    delete [] auxligne;
    return aux;
}

/*--------------------------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------------------------*/
float Inliers::Proba_Assigne_Gauss( unsigned long int i, int k)
{

    float aux=1.;
    int j,j1,j2;    
    unsigned long int dim=(im_point->nx)*(im_point->ny);
    float auxbis=0;
    float *auxligne = new float[im_point->nb_bands];
    
    // no singularity in the covariance matrix
    if (singularity[k]!=1) 
    {
    	aux=sqrt(determinant[k])/pow(2.0*M_PI,(double)(im_point->nb_bands/2.));
	for (j1=0;j1<(im_point->nb_bands);j1++){
		auxligne[j1]=0;
		for (j2=0;j2<(im_point->nb_bands);j2++){	 
		auxligne[j1]=((float)*(im_point->position+Index_Inliers[i]+dim*j2)-*(A[0][k]+j2))*Inv_covariance_cluster[k][j1][j2]+auxligne[j1];								}
	}
	for (j1=0;j1<(im_point->nb_bands);j1++){
		auxbis=auxbis+auxligne[j1]*((float)*(im_point->position+Index_Inliers[i]+dim*j1)-*(A[0][k]+j1));
					}
    aux=aux*exp(-0.5*auxbis);
    }
    
    //singularity : independence intra bande simplification
    else
    {
    	aux=1;
    	for (j=0;j<(im_point->nb_bands);j++)
   	{
		if (covariance_cluster[k][j][j]!=0){
				
aux=1.0/(sqrt(covariance_cluster[k][j][j]*2.0*M_PI))*exp(-(*(A[0][k]+j)-(float)*(im_point->position+Index_Inliers[i]+dim*j))*(*(A[0][k]+j)-(float)*(im_point->position+Index_Inliers[i]+dim*j))/(2*covariance_cluster[k][j][j]))*aux;	
			}
		else
		{				
		if((float)*(im_point->position+Index_Inliers[i]+dim*j)!=*(A[0][k]+j)){aux=0;}
		}
    	}
    
    }
				
    delete [] auxligne;
    return aux;
}

/*--------------------------------------------------------------------------------------------*/
void Inliers::Reassigne_inliers_StdDevVariable()
{			
	int k;
	unsigned long int i;
	float max,aux;
			
	for (i=0; i<nb_inliers;i++){
		max=-Inf;
		for (k=0;k<A[0].nb_ref;k++){ 
			aux=Proba_Assigne_Gauss(i,k);
			if (aux>max){
				max=aux;
				firstNearestA[i]=k;
			}
		}
	}
	Update_effectif();
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::NearestRefVector_StdDevVar()
{
	unsigned long int i;
	int k;
	float firstaux=0, secondaux=0;
	float max;
	float secondmax;
	float *aux=new float[A[0].nb_ref];
	for (i=0;i<nb_inliers;i++)
	{	
			firstNearestA[i]=-1;
			secondNearestA[i]=-1;
			max=-Inf;
			secondmax=-Inf;
			for (k=0;k<A[0].nb_ref;k++)
				aux[k]=Proba_Assigne_Gauss(i,k);
				
			for (k=0;k<A[0].nb_ref;k++)
			{
				firstaux=aux[k];
				if (firstaux> max)
				{
				firstNearestA[i]=k;
				max=firstaux;
				}
			}
			for (k=0;k<A[0].nb_ref;k++)
			{
				secondaux=aux[k];
				if ( (secondaux>secondmax) && firstNearestA[i]!=k )
				{
				secondNearestA[i]=k;
				secondmax=secondaux;
				}
			}
		
	}
	Update_effectif();
	Remove_Unaffected_RefVector(1);
	delete [] aux;
}
/*--------------------------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------------------------*/
void Inliers::NearestRefVector_StdDevVar_PostRemoval()
{
	unsigned long int i;
	int k;
	float firstaux=0, secondaux=0;
	float max;
	float secondmax;
	for (i=0;i<nb_inliers;i++)
	{	
			firstNearestA[i]=-1;
			secondNearestA[i]=-1;
			max=-Inf;
			secondmax=-Inf;
			for (k=0;k<A[0].nb_ref;k++)
			{
				firstaux=Proba_Assigne_Gauss(i,k);
				if (firstaux> max)
				{
				firstNearestA[i]=k;
				max=firstaux;
				}
			}
			for (k=0;k<A[0].nb_ref;k++)
			{
			   if(firstNearestA[i]!=k)
			   {
				secondaux=Proba_Assigne_Gauss_PostRemoval(i,k,firstNearestA[i]);
				if (secondaux>secondmax)
				{
				secondNearestA[i]=k;
				secondmax=secondaux;
				}
			   }
			}
			//max=Proba_Assigne_Gauss(i,firstNearestA[i])-Proba_Assigne_Gauss_PostRemoval(i,secondNearestA[i],firstNearestA[i]);
			//if (max<0){
			//cout<<"ecart "<<max<<endl;
			//}
	}
	Estimation_CrossEffectif();
	Remove_Unaffected_RefVector(1);
}
/*--------------------------------------------------------------------------------------------*/

void Inliers::Update_effectif()
{
	for (int k=0;k<A[0].nb_ref;k++){effectif_cluster[k]=0;}
	for (unsigned long int i=0;i<nb_inliers;i++)
	{
	 	effectif_cluster[firstNearestA[i]]++;
	}

}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Remove_Unaffected_RefVector(int AllCov)
{
	for (int k=0;k<A[0].nb_ref;k++)
	{
		if(effectif_cluster[k]==0)
		{
			Remove_RefVector(k,AllCov);
			k--;
		}
	}
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Remove_RefVector(int k, int AllCov)
{
	unsigned long int i;
	int j,j1,j2,kk;
 	A[0].nb_ref=A[0].nb_ref-1;
 	for (i=0;i<nb_inliers;i++){
 	        if (firstNearestA[i]>k){firstNearestA[i]--;}
 	}
 	for (kk=k;kk<A[0].nb_ref;kk++){
 	effectif_cluster[kk]=effectif_cluster[kk+1];
 	        for (j=0;j<im_point->nb_bands;j++){
 	        	A[0][kk][j]=A[0][kk+1][j];
 	        }
 	}
 	

 	 vector<float*>::iterator it1;
	 vector<float*>::iterator it2;

	 it1=A[0].begin()+A[0].size()-2;
	 it2=A[0].begin()+A[0].size()-1;
	 A[0].erase(it1,it2);
	 nb_RefVector=A[0].nb_ref;
	 if (AllCov==1)
	 {
	 	for (kk=k;kk<A[0].nb_ref;kk++){
			singularity[kk]=singularity[kk+1];
			determinant[kk]=determinant[kk+1];
 	        	for (j1=0;j1<im_point->nb_bands;j1++){
				for (j2=0;j2<im_point->nb_bands;j2++){
 	        			covariance_cluster[kk][j1][j2]=covariance_cluster[kk+1][j1][j2];
					Inv_covariance_cluster[kk][j1][j2]=Inv_covariance_cluster[kk+1][j1][j2];
				}
 	        	}
 	 	}
		for (i=0;i<nb_inliers;i++){
 	        	if (secondNearestA[i]>k){secondNearestA[i]--;}
 		}
	 
	 }
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Update_Centers()
{
	unsigned long int i;
	unsigned long int dim=(im_point->nx)*(im_point->ny);
	int j,k;
	float auxi;
	float **aux=new float*[A[0].nb_ref];
	for (k=0;k<A[0].nb_ref;k++){aux[k]=new float[im_point->nb_bands];}
	for (k=0;k<A[0].nb_ref;k++){for (j=0;j<im_point->nb_bands;j++){aux[k][j]=0;}}
	
	ChangingCriterion=0;
	for (i=0;i<nb_inliers;i++)
	{
		for (j=0;j<im_point->nb_bands;j++)
		{
			aux[firstNearestA[i]][j]= (float)*(im_point->position+Index_Inliers[i]+dim*j)+aux[firstNearestA[i]][j];			
		}
	}
	for (k=0;k<A[0].nb_ref;k++)
	{
		auxi=0;
		for (j=0;j<im_point->nb_bands;j++)
		{
			auxi=(A[0][k][j]-aux[k][j]/(float)effectif_cluster[k])*(A[0][k][j]-aux[k][j]/(float)effectif_cluster[k])+auxi;
		}
		ChangingCriterion+=auxi;
	}
	Change=1;
	ChangingCriterion=sqrt(ChangingCriterion);
	float range=0;
	for (int k=0;k<im_point->nb_bands;k++){range+=(maxi_dim[k]-mini_dim[k]);}
	range=range/im_point->nb_bands;
	if (ChangingCriterion<(im_point->nb_bands)*A[0].nb_ref*range/(1e4)){Change=0;}//convergence finale!!!
	
	for (k=0;k<A[0].nb_ref;k++){
		for (j=0;j<im_point->nb_bands;j++)
		{
		A[0][k][j]=aux[k][j]/(float)effectif_cluster[k];
		}
	}	
	
	delete [] aux;
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Estimation_CovarianceMatrix()
{
	unsigned long int i;
	unsigned long int dim=(unsigned long int)(im_point->nx*im_point->ny);
	int l,j,k;
	
	for(k=0;k<A[0].nb_ref;k++){
		for(l=0;l<im_point->nb_bands;l++){
			for(j=0;j<im_point->nb_bands;j++){ covariance_cluster[k][l][j]=0;}}}

	//Covariance Matrix
	for (i=0;i<nb_inliers;i++)
	{     
		for (l=0;l<im_point->nb_bands;l++)
		{
			for (j=l;j<im_point->nb_bands;j++)
			{
	    		
covariance_cluster[firstNearestA[i]][l][j]=((float)*(im_point->position+Index_Inliers[i]+dim*j)-A[0][firstNearestA[i]][j])*((float)*(im_point->position+Index_Inliers[i]+dim*l)-A[0][firstNearestA[i]][l])+covariance_cluster[firstNearestA[i]][l][j];
  			 }
	 	}
	} 
	for(k=0;k<A[0].nb_ref;k++)
	{
	   if (effectif_cluster[k]!=0)
	   {
		for (l=0;l<im_point->nb_bands;l++)
		{
			for(j=l;j<im_point->nb_bands;j++)
			{
				covariance_cluster[k][l][j]=covariance_cluster[k][l][j]/effectif_cluster[k];
				covariance_cluster[k][j][l]=covariance_cluster[k][l][j];
			}	
		}
	   }
	}
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Singularity_Detection()
{
	int j;
	for (int k=0;k<A[0].nb_ref;k++)
	{
		singularity[k]=0;
		for (j=0;j<im_point->nb_bands;j++)
		{
			if (covariance_cluster[k][j][j]==0)
			{
				singularity[k]=1;
			}
		}
	}

}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Estimation_Stdev_RefVector()
{
	Estimation_CovarianceMatrix();
	Singularity_Detection();
	Invert_CovarianceMatrix();
		
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Invert_CovarianceMatrix()
{
	int l,j,k;
	float **auxCOV=new float*[im_point->nb_bands+1];
	for (k=0;k<im_point->nb_bands+1;k++){auxCOV[k]=new float[im_point->nb_bands+1];}
	float *col=new float[im_point->nb_bands+1];
	auxCOV[0][0]=0;
	float d;
	int *indx=new int[im_point->nb_bands+1], n=(int)im_point->nb_bands;
	void ludcmp(float **a,int n,int *indx,float *d);
	void lubksb(float **a,int n,int *indx, float b[]);
	
	for (k=0;k<A[0].nb_ref;k++)
	{
		for (l=0;l<im_point->nb_bands;l++)
		{
			for(j=l;j<im_point->nb_bands;j++)
			{
				auxCOV[l+1][j+1]=covariance_cluster[k][l][j];
				auxCOV[j+1][l+1]=auxCOV[l+1][j+1];
			}
	
		}
	   	if (singularity[k]!=1)
	   	{
			//INVERSE of COV MATRIX
			ludcmp(auxCOV,n,indx,&d);
			for (j=1;j<im_point->nb_bands+1;j++){
				for (l=1;l<im_point->nb_bands+1;l++){col[l]=0.0;}
				col[j]=1.0;
				lubksb(auxCOV,n,indx,col);
				for (l=1;l<im_point->nb_bands+1;l++) 
				{
					Inv_covariance_cluster[k][l-1][j-1]=col[l];
				}		
			}
			for (j=1;j<im_point->nb_bands+1;j++)
			{
				for (l=1;l<im_point->nb_bands+1;l++)
				{
					auxCOV[j][l]=Inv_covariance_cluster[k][j-1][l-1];
				}
			}
			
			//DETERMINANT OF THE INVERSE COV MATRIX
			ludcmp(auxCOV,n,indx,&d);
			for (j=1;j<im_point->nb_bands+1;j++){d=auxCOV[j][j]*d;}
			determinant[k]=d;
	   
		   }
		   else {determinant[k]=0;}
	   
	}
 	for(k=0;k<im_point->nb_bands+1;k++){delete [] auxCOV[k];}
 	delete [] auxCOV;delete [] col;
       


}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Estimation_CrossEffectif()
{
	int k;
	for(int l=0;l<A[0].nb_ref;l++)
	{
		effectif_cluster[l]=0;
		for (k=0;k<A[0].nb_ref;k++)
		{
			cross_effectif_cluster[l][k]=0;
		}
	}
	for(unsigned long int i=0;i<nb_inliers;i++)
	{
		effectif_cluster[firstNearestA[i]]++;
		cross_effectif_cluster[firstNearestA[i]][secondNearestA[i]]++;
						
	}

}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Estimation_Center_RefVector_PostRemoval()
{
	unsigned long int i;
	unsigned long int dim=(im_point->nx)*(im_point->ny);
	int l,k,j;	
	Estimation_CrossEffectif();
	for(l=0;l<A[0].nb_ref;l++)
	{
		for (k=0;k<A[0].nb_ref;k++)
		{
			for(j=0;j<im_point->nb_bands;j++)
			{
				
				center_RefVector_PostRemoval[l][k][j]=0;
			}
		}
	}
	for(i=0;i<nb_inliers;i++)
	{
		for (j=0;j<im_point->nb_bands;j++) 
		{
		
center_RefVector_PostRemoval[secondNearestA[i]][firstNearestA[i]][j]=center_RefVector_PostRemoval[secondNearestA[i]][firstNearestA[i]][j]+((float)*(im_point->position+Index_Inliers[i]+dim*j));
		}	
	}
	
	for(l=0;l<A[0].nb_ref;l++)
	{
		for (k=0;k<A[0].nb_ref;k++)
		{
			for(j=0;j<im_point->nb_bands;j++)
			{	center_RefVector_PostRemoval[l][k][j]=(center_RefVector_PostRemoval[l][k][j]+A[0][l][j]*(float)effectif_cluster[l])/((float)effectif_cluster[l]+(float)cross_effectif_cluster[k][l]);
			
			}
		}
	}
		

}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Estimation_CovarianceMatrix_PostRemoval()
{
	unsigned long int i;
	unsigned long int dim=(im_point->nx)*(im_point->ny);
	float aux1,aux2;
	int l,k,j1,j2;
	for(l=0;l<A[0].nb_ref;l++)
	{
		for (k=0;k<A[0].nb_ref;k++)
			{
			for(j1=0;j1<im_point->nb_bands;j1++)
			{
				for(j2=0;j2<im_point->nb_bands;j2++)
				{
					covariance_cluster_PostRemoval[l][k][j1][j2]=0;
				}
			}
		}
	}
			
	//COVARIANCE POSTREMOVAL ESTIMATION : 1- New inliers contribution
	for(i=0;i<nb_inliers;i++)
	{		
		for (j1=0;j1<im_point->nb_bands;j1++) 
		{
			for (j2=j1;j2<im_point->nb_bands;j2++) 
			{					aux1=((float)*(im_point->position+Index_Inliers[i]+dim*j1)-center_RefVector_PostRemoval[secondNearestA[i]][firstNearestA[i]][j1]);	
			aux2=((float)*(im_point->position+Index_Inliers[i]+dim*j2)-center_RefVector_PostRemoval[secondNearestA[i]][firstNearestA[i]][j2]);
	
		covariance_cluster_PostRemoval[secondNearestA[i]][firstNearestA[i]][j1][j2]=covariance_cluster_PostRemoval[secondNearestA[i]][firstNearestA[i]][j1][j2]+aux1*aux2;
			}
		}
	}
	
	//COVARIANCE POSTREMOVAL ESTIMATION : 2- Old inliers contribution
	for(l=0;l<A[0].nb_ref;l++)
	{
		for (k=0;k<A[0].nb_ref;k++)
		{
			for(j1=0;j1<im_point->nb_bands;j1++)
			{
				for(j2=j1;j2<im_point->nb_bands;j2++)
				{
				//if (((float)effectif_cluster[l]+(float)cross_effectif_cluster[k][l])==0){cout<<"putain\n"<<effectif_cluster[l];}
					covariance_cluster_PostRemoval[l][k][j1][j2]=(covariance_cluster_PostRemoval[l][k][j1][j2]+covariance_cluster[l][j1][j2]*(float)effectif_cluster[l])/((float)effectif_cluster[l]+(float)cross_effectif_cluster[k][l]);
				covariance_cluster_PostRemoval[l][k][j2][j1]=covariance_cluster_PostRemoval[l][k][j1][j2];						
				}
			
			}
		}
	}
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Singularity_Detection_Postremoval()
{
	int j1,j2,j,nb1=0,nb2=0;;
	for (j1=0;j1<A[0].nb_ref;j1++){	singularity[j1]=0;
		for (j2=0;j2<A[0].nb_ref;j2++){Singularity_PostRemoval[j1][j2]=0;
		}
	}
	
	for (j=0;j<im_point->nb_bands;j++){
	   for (j1=0;j1<A[0].nb_ref;j1++){	
		if (covariance_cluster[j1][j][j]==0){singularity[j1]=1;nb1++;}
		for (j2=0;j2<A[0].nb_ref;j2++){
			if (covariance_cluster_PostRemoval[j1][j2][j][j]==0){Singularity_PostRemoval[j1][j2]=1;nb2++;}
		}
	   }
	}
	//cout<<"Number singularities : "<<nb1<<endl;
	//cout<<"Number singularities PostRemoval : "<<nb2<<endl;
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Invert_PostCovarianceMatrix()
{
	int l,k,k1,k2,j;
	float **auxCOV=new float*[im_point->nb_bands+1];
	for (k=0;k<im_point->nb_bands+1;k++){auxCOV[k]=new float[im_point->nb_bands+1];}
	float *col=new float[im_point->nb_bands+1];
	auxCOV[0][0]=0;
	float d;
	int *indx=new int[im_point->nb_bands+1], n=(int)im_point->nb_bands;
	
	void ludcmp(float **a,int n,int *indx,float *d);
	void lubksb(float **a,int n,int *indx, float b[]);
	
	//INVERSE COVARIANCE POSTREMOVAL ESTIMATION
	
	for (k1=0;k1<A[0].nb_ref;k1++)
	{
		for (k2=0;k2<A[0].nb_ref;k2++)
		{			
			Singularity_PostRemoval[k1][k2]=0;		
			for (l=0;l<im_point->nb_bands;l++)
			{
				if (covariance_cluster_PostRemoval[k1][k2][l][l]==0)
				{
					Singularity_PostRemoval[k1][k2]=1;
				}
				for(j=l;j<im_point->nb_bands;j++)
				{
					auxCOV[l+1][j+1]=covariance_cluster_PostRemoval[k1][k2][l][j];
					auxCOV[j+1][l+1]=auxCOV[l+1][j+1];
				}		
			}
			
		   	if (Singularity_PostRemoval[k1][k2]!=1)
		  	{
				//INVERSE of COV MATRIX
				 ludcmp(auxCOV,n,indx,&d);
				for (j=1;j<im_point->nb_bands+1;j++){
					for (l=1;l<im_point->nb_bands+1;l++){col[l]=0.0;}
					col[j]=1.0;
					lubksb(auxCOV,n,indx,col);
					for (l=1;l<im_point->nb_bands+1;l++) 
					{
	Inv_covariance_cluster_PostRemoval[k1][k2][l-1][j-1]=col[l];
					}		
				}
				for (j=1;j<im_point->nb_bands+1;j++){
					for (l=1;l<im_point->nb_bands+1;l++){
						auxCOV[l][j]=Inv_covariance_cluster_PostRemoval[k1][k2][l-1][j-1];
					}
				}
				//DETERMINANT OF THE INVERSE COV MATRIX
				ludcmp(auxCOV,n,indx,&d);
				for (j=1;j<im_point->nb_bands+1;j++){d*=auxCOV[j][j];}
				determinant_PostRemoval[k1][k2]=d;
			}
			else {determinant_PostRemoval[k1][k2]=0;}
		        
		}
	}
 	for(k=0;k<im_point->nb_bands+1;k++){delete [] auxCOV[k];}
 	delete [] auxCOV;delete [] col;	
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Estimation_Stdev_RefVector_PostRemoval()
{

	Estimation_Center_RefVector_PostRemoval();
	Estimation_CovarianceMatrix_PostRemoval();
	Singularity_Detection_Postremoval();
	Invert_PostCovarianceMatrix();
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Inliers_Variation_Index_term_cost2(float* aux)
{
	int l,k;
	Estimation_CrossEffectif();
	for (k=0;k<A[0].nb_ref;k++)
	{			
		aux[k]=(float)effectif_cluster[k]*log((float)effectif_cluster[k]/(float)nb_inliers)/M_LOG2;
		for (l=0;l<A[0].nb_ref;l++)
		{
			if (l!=k)
			{
				aux[k]=aux[k]-(float)((effectif_cluster[l]+cross_effectif_cluster[k][l])*log((float)(effectif_cluster[l]+cross_effectif_cluster[k][l])/(float)nb_inliers)/M_LOG2-effectif_cluster[l]*log((float)effectif_cluster[l]/(float)nb_inliers)/M_LOG2);
			}
		}
	}
}
/*--------------------------------------------------------------------------------------------*/
float Inliers::Error_FreeInliers(unsigned long int i)
{
       unsigned long int dim=(unsigned long int)(im_point->nx*im_point->ny);
       int j1,j2;
       float aux1,aux2;
       float *auxligne = new float[im_point->nb_bands];
       float ratio=0;
       
       aux2=0;
       for (j1=0;j1<(im_point->nb_bands);j1++)
       {
               auxligne[j1]=0;
               for (j2=0;j2<(im_point->nb_bands);j2++)
               {	       auxligne[j1]=((float)*(im_point->position+Index_Inliers[i]+dim*j2)-(A[0][firstNearestA[i]][j2]))*Inv_covariance_cluster[firstNearestA[i]][j1][j2]+auxligne[j1];			       
               }
       }

       for (j1=0;j1<(im_point->nb_bands);j1++){
               aux2=aux2+auxligne[j1]*((float)*(im_point->position+Index_Inliers[i]+dim*j1)-(A[0][firstNearestA[i]][j1]));
       }

       aux1=0;
       for (j1=0;j1<(im_point->nb_bands);j1++){
               auxligne[j1]=0;
               for (j2=0;j2<(im_point->nb_bands);j2++)
               {	       auxligne[j1]=((float)*(im_point->position+Index_Inliers[i]+dim*j2)-center_RefVector_PostRemoval[secondNearestA[i]][firstNearestA[i]][j2])*Inv_covariance_cluster_PostRemoval[secondNearestA[i]][firstNearestA[i]][j1][j2]+auxligne[j1];  		       
               }
       }

       for (j1=0;j1<(im_point->nb_bands);j1++){
               aux1=aux1+auxligne[j1]*((float)*(im_point->position+Index_Inliers[i]+dim*j1)-center_RefVector_PostRemoval[secondNearestA[i]][firstNearestA[i]][j1]);
       }
       ratio=(aux1-aux2)/(2.0*M_LOG2)+0.5/M_LOG2*log(determinant[firstNearestA[i]]/determinant_PostRemoval[secondNearestA[i]][firstNearestA[i]]);
  	 //if (ratio<0){
	 //cout<<Proba_Assigne_Gauss(i,firstNearestA[i])-Proba_Assigne_Gauss_PostRemoval(i,secondNearestA[i],firstNearestA[i])<<endl;
	 //}
	 delete [] auxligne;
	 return ratio;
}
/*--------------------------------------------------------------------------------------------*/
float Inliers::Error_FreeInliers_Singularity(unsigned long int i)
{
	//independence between bands assumption for simplication!!!
	
	unsigned long int dim=(unsigned long int)(im_point->nx*im_point->ny);
	int j;
	float aux1,aux2,ratio=0;
	if (singularity[firstNearestA[i]]==1 || Singularity_PostRemoval[secondNearestA[i]][firstNearestA[i]]==1)
	{
		for (j=0;j<im_point->nb_bands;j++)
		{
	      		//bands without a singularity
	      	      if 	(covariance_cluster[firstNearestA[i]][j][j]!=0&&covariance_cluster_PostRemoval[secondNearestA[i]][firstNearestA[i]][j][j]!=0)
		      {             	aux1=((float)*(im_point->position+Index_Inliers[i]+dim*j)-center_RefVector_PostRemoval[secondNearestA[i]][firstNearestA[i]][j]);
		      		aux2=((float)*(im_point->position+Index_Inliers[i]+dim*j)-A[0][firstNearestA[i]][j]);       
						      ratio=aux1*aux1/(2.*M_LOG2*covariance_cluster_PostRemoval[secondNearestA[i]][firstNearestA[i]][j][j])-aux2*aux2/(2.*M_LOG2*covariance_cluster[firstNearestA[i]][j][j])+0.5*log(covariance_cluster_PostRemoval[secondNearestA[i]][firstNearestA[i]][j][j]/covariance_cluster[firstNearestA[i]][j][j])/M_LOG2+ratio;
	      	      }
		      //bands with a singularity (except case "singularity->singularity" : zero cost exclusion)
		      else
	      	      {
	              	if (covariance_cluster_PostRemoval[secondNearestA[i]][firstNearestA[i]][j][j]!=0)//singularity->no singularity 
	              	{    	aux1=((float)*(im_point->position+Index_Inliers[i]+dim*j)-center_RefVector_PostRemoval[secondNearestA[i]][firstNearestA[i]][j]);
	         	aux1=aux1*aux1/(2.*M_LOG2*covariance_cluster_PostRemoval[secondNearestA[i]][firstNearestA[i]][j][j])+0.5*log(covariance_cluster_PostRemoval[secondNearestA[i]][firstNearestA[i]][j][j])/M_LOG2;
		          	ratio=aux1;	              
	              	}
			if (covariance_cluster[firstNearestA[i]][j][j]!=0)// no singularity->singularity 
	              	{    	aux1=((float)*(im_point->position+Index_Inliers[i]+dim*j)-A[0][firstNearestA[i]][j]);
	         	aux1=aux1*aux1/(2.*M_LOG2*covariance_cluster[firstNearestA[i]][j][j])+0.5*log(covariance_cluster[firstNearestA[i]][j][j])/M_LOG2;
		          	ratio=-aux1;	              
	              	}
	      	      }
		
		}
	}
	return ratio;
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Error_OtherInliers(float * SumErrorRatio)
{
	float ratio,aux1,aux2;
	float *auxligne = new float[im_point->nb_bands];
	unsigned long int dim=(unsigned long int)(im_point->nx*im_point->ny);
	unsigned long int i;
	int k,j1,j2;
	for (k=0;k<A[0].nb_ref;k++)  
	{	
	   for (i=0;i<nb_inliers;i++)
	   {
		if (singularity[firstNearestA[i]]==0&&Singularity_PostRemoval[firstNearestA[i]][k]==0)
	  	 {
		   if (firstNearestA[i]!=k)
		   {
			aux2=0;
			for (j1=0;j1<(im_point->nb_bands);j1++){
				auxligne[j1]=0;
				for (j2=0;j2<(im_point->nb_bands);j2++)
				{		auxligne[j1]=((float)*(im_point->position+Index_Inliers[i]+dim*j2)-*(A[0][firstNearestA[i]]+j2))*Inv_covariance_cluster[firstNearestA[i]][j1][j2]+auxligne[j1];				
				}
			}
			for (j1=0;j1<(im_point->nb_bands);j1++){
				aux2=aux2+auxligne[j1]*((float)*(im_point->position+Index_Inliers[i]+dim*j1)-*(A[0][firstNearestA[i]]+j1));
			}
		
			aux1=0;
			for (j1=0;j1<(im_point->nb_bands);j1++){
				auxligne[j1]=0;
				for (j2=0;j2<(im_point->nb_bands);j2++)
				{		auxligne[j1]=((float)*(im_point->position+Index_Inliers[i]+dim*j2)-center_RefVector_PostRemoval[firstNearestA[i]][k][j2])*Inv_covariance_cluster_PostRemoval[firstNearestA[i]][k][j1][j2]+auxligne[j1];				
				}
			}
			
			for (j1=0;j1<(im_point->nb_bands);j1++){
			aux1=aux1+auxligne[j1]*((float)*(im_point->position+Index_Inliers[i]+dim*j1)-center_RefVector_PostRemoval[firstNearestA[i]][k][j1]);
			}
			ratio=(aux1-aux2)/(2.0*M_LOG2)+0.5/M_LOG2*log(determinant[firstNearestA[i]]/determinant_PostRemoval[firstNearestA[i]][k]);
			SumErrorRatio[k]=SumErrorRatio[k]+ratio;		
		   }
		}
	   }
	}
	delete [] auxligne;
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Inliers_Variation_Error_term_cost4(float * SumErrorRatio)
{
	unsigned long int i;
	int k;
	for (k=0;k<A[0].nb_ref;k++) {SumErrorRatio[k]=0;}
	float ratio=0;
	Estimation_Stdev_RefVector();
	Estimation_Stdev_RefVector_PostRemoval();
	NearestRefVector_StdDevVar_PostRemoval();
	
	//variation error brougth by the new free inliers
	for (i=0;i<nb_inliers;i++)
	{
		ratio=0;
		
		//cost error free inlier when no singularity
		if ((singularity[firstNearestA[i]]==0) && (Singularity_PostRemoval[secondNearestA[i]][firstNearestA[i]]==0))
	   	{	
	   		ratio=Error_FreeInliers(i);								
	    	}
		// cost  error free inlier when singularities
	    	else 
	    	{	//cout<<singularity[firstNearestA[i]]<<" && "<<Singularity_PostRemoval[secondNearestA[i]][firstNearestA[i]]<<endl;
	    		ratio=Error_FreeInliers_Singularity(i);
		
	    	}
		
	    	SumErrorRatio[firstNearestA[i]]=SumErrorRatio[firstNearestA[i]]+ratio;
		
	}
	
	//variation error brougth by the others inliers du to the change of their variance
	Error_OtherInliers(SumErrorRatio);
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Update_RefVectors()
{

	unsigned long int i;
	int k;
	unsigned long int dim=(unsigned long int)(im_point->nx*im_point->ny);
	float ChangeCodeLength;
	float* SumErrorRatio=new float[dim];
	float* Indexcost2=new float[dim];
	for (i=0;i<dim;i++){SumErrorRatio[i]=0;}
	int nb_ref_previous=0;
	float FactorOverCostModel3=0;
	float maxidecrease;
	int indexmaxidecrease;
	cout<<"number of reference Vector after selection step : "<<A[0].nb_ref<<endl; 
	while ((nb_ref_previous!=A[0].nb_ref) && (A[0].nb_ref>1))
	{
	     	nb_ref_previous=A[0].nb_ref;
		Inliers_Variation_Error_term_cost4(SumErrorRatio);
		FactorOverCostModel3=(im_point->nb_bands)+1;// factor for calculating the variation of the encoding length ofteh Gaussian paramters. THIS IS A VERY ROUGH APPROXIMATION AND SHOULD BE REPLACED BY THE TERM DEFINED IN THEORITICAL NOTE  
		Inliers_Variation_Index_term_cost2(Indexcost2);

		//search of the greatest decrease
		maxidecrease=Inf;
		indexmaxidecrease=0;//default
		for (k=0;k<A[0].nb_ref;k++)
		{
	     		ChangeCodeLength=(-1.)*(im_point->nb_bands)*sizeof(PIXEL)*8*FactorOverCostModel3+Indexcost2[k]+SumErrorRatio[k];
			if (maxidecrease>ChangeCodeLength){maxidecrease=ChangeCodeLength;indexmaxidecrease=k;}
			
		}
		k=indexmaxidecrease;
		
		//suppression of the greatest decrease     			     		
		ChangeCodeLength=(-1.)*(im_point->nb_bands)*sizeof(PIXEL)*8*FactorOverCostModel3+Indexcost2[k]+SumErrorRatio[k];
		
		 if (ChangeCodeLength<0)
		{
			Remove_RefVector(k,1);
			Adaptation();
		}
	cout<<"number of reference Vector intermediate : "<<A[0].nb_ref<<endl; 
		
     	}
	nb_RefVector=A[0].nb_ref;
	
	delete [] SumErrorRatio;
	delete [] Indexcost2;	
     	
}
/*--------------------------------------------------------------------------------------------*/




















/*--------------------------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------------------------*/

//////////////////////////DISPLAY FUNCTIONS/////////////////////////////////////////////////////

/*--------------------------------------------------------------------------------------------*/
void Inliers::Print_Mean_Covariance(int l)
{
	int j2,j1;
	if (l<0)
	{
	 for(l=0;l<A[0].nb_ref;l++)
	 {	cout<<"\nReference Vector "<<l<<", determinant: "<<determinant[l]<<", effectif: "<<effectif_cluster[l]<<", Singularity : "<<singularity[l]<<"\n";
		 cout<<"Mean : 		 	Covariance : "<<endl;
		 for(j2=0;j2<im_point->nb_bands;j2++)
		 {

			 cout<<A[0][l][j2]<<"	";
			 for (j1=0;j1<im_point->nb_bands;j1++)
			 {
				 cout<<"		"<<(covariance_cluster[l][j2][j1]);
			 }
			 cout<<"\n";
		 }
	 }
	 cout<<""<<endl;
	}
	else
	{
		cout<<"\nReference Vector "<<l<<", determinant: "<<determinant[l]<<", effectif: "<<effectif_cluster[l]<<", Singularity : "<<singularity[l]<<"\n";
		 cout<<"Mean : 		 	Covariance : "<<endl;
		 for(j2=0;j2<im_point->nb_bands;j2++)
		 {

			 cout<<A[0][l][j2]<<"	";
			 for (j1=0;j1<im_point->nb_bands;j1++)
			 {
				 cout<<"		"<<(covariance_cluster[l][j2][j1]);
			 }
			 cout<<"\n";
		 }
	}
}
/*--------------------------------------------------------------------------------------------*/
void Inliers::Print_Mean_Covariance_PostRemoval(int l)
{
   int j2,j1,k;
   if (l<0)
   {
   
   
	for(l=0;l<A[0].nb_ref;l++)
	{	cout<<"\nReference Vector "<<l<<", determinant: "<<determinant[l]<<", effectif: "<<effectif_cluster[l]<<", Singularity : "<<singularity[l]<<"\n";
		cout<<"Mean : 		 	Inv_covariance_cluster : "<<endl;
		for(j2=0;j2<im_point->nb_bands;j2++)
		{
			
			cout<<A[0][l][j2]<<"	";
			for (j1=0;j1<im_point->nb_bands;j1++)
			{
				cout<<"		"<<(covariance_cluster[l][j2][j1]);
			}
			cout<<"\n";
		}
		for(k=0;k<A[0].nb_ref;k++)
		{
			cout<<"\n\t If Removal Vector "<<k<<", determinant: "<<determinant_PostRemoval[l][k]<<", effectif: "<<cross_effectif_cluster[k][l]+effectif_cluster[l]<<", Singularity : "<<Singularity_PostRemoval[l][k]<<"\n";
			cout<<"\tMean : 		 	Inv_covariance_cluster : "<<endl;
			for(j2=0;j2<im_point->nb_bands;j2++)
			{
				
				cout<<"\t"<<center_RefVector_PostRemoval[l][k][j2]<<"	";
				for (j1=0;j1<im_point->nb_bands;j1++)
				{
					cout<<"		"<<(covariance_cluster_PostRemoval[l][k][j2][j1]);
				}
				cout<<"\n";
			}
		}
	
	}
	cout<<""<<endl;
    }
    else
    {
    		cout<<"\nReference Vector "<<l<<", determinant: "<<determinant[l]<<", effectif: "<<effectif_cluster[l]<<", Singularity : "<<singularity[l]<<"\n";
		cout<<"Mean : 		 	Inv_covariance_cluster : "<<endl;
		for(j2=0;j2<im_point->nb_bands;j2++)
		{
			
			cout<<A[0][l][j2]<<"	";
			for (j1=0;j1<im_point->nb_bands;j1++)
			{
				cout<<"		"<<(covariance_cluster[l][j2][j1]);
			}
			cout<<"\n";
		}
		for(k=0;k<A[0].nb_ref;k++)
		{
			cout<<"\n\t If Removal Vector "<<k<<", determinant: "<<determinant_PostRemoval[l][k]<<", effectif: "<<cross_effectif_cluster[k][l]+effectif_cluster[l]<<", Singularity : "<<Singularity_PostRemoval[l][k]<<"\n";
			cout<<"\tMean : 		 	Inv_covariance_cluster : "<<endl;
			for(j2=0;j2<im_point->nb_bands;j2++)
			{
				
				cout<<"\t"<<center_RefVector_PostRemoval[l][k][j2]<<"	";
				for (j1=0;j1<im_point->nb_bands;j1++)
				{
					cout<<"		"<<(covariance_cluster_PostRemoval[l][k][j2][j1]);
				}
				cout<<"\n";
			}
		}
    }
}				 
