/*---------------------------------------------------------------------------*/
/*!
 * @file ReferenceVector.hh
 * @brief  File defining a object for managing the collection of component of the mixture\n
 FOR ANY EXTENSION, THIS LIBRARY OF OBJECT REQUIRES TO BE RE-IMPLEMENTED COMPLETLY*/
/*---------------------------------------------------------------------------*/

#include <iostream>
#include <string>  
#include <vector>
#include <fstream>

#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cassert>

#include "./Image_basics.hh"
/*! format of the data */
#define PIXEL float
using namespace std;

/*!@brief   the inheritance is not any more useful : OBJECT TO BE REMOVED*/
class Generic_Vector :  public vector<float*>
{
	public: 
		
		Generic_Vector()
		{	
			Initialise();
			
		}
	
	//private : 
		/*! @brief fixing the pointer to the image data*/
		virtual void Initialise()
		{
			im_point=NULL; 
		}	
		
    protected:
        /*! @brief pointer to the image data*/
        Image<PIXEL>* im_point;
    
	
};
/**********************************************************************************************/

/*!@brief Object for managing the collection of component of the mixture\n
 FOR ANY EXTENSION, THIS LIBRARY OF OBJECT REQUIRES TO BE RE-IMPLEMENTED COMPLETLY */
class Ref_Vector : public Generic_Vector 	
{	
	public:
		/*!@brief Number of component */
		int nb_ref;
		/*! @brief Constructor : points to the image and initializes a Gaussian mixture */
        Ref_Vector(): Generic_Vector(){}
		Ref_Vector(Image<PIXEL> *data_point,int nb_init) : Generic_Vector()
		{		
		im_point=data_point;
		Initialise(im_point,nb_init);

		}
		/*--------------------------------------------------------------------------------------------*/
		/*!@brief  method to add other components in the mixture if initialisation has failed */
		void AddRefVectors(int nb_init)
		{
		int i,j;
			unsigned long int dim;
			dim=(*im_point).nx*(*im_point).ny;
			int nb_bands=(*im_point).nb_bands;
			int Nb_Ref_Vector_Init=(int)((nb_init-nb_ref)*1.6);
			float *moy=new float[nb_bands];
			float *dev=new float[nb_bands];
			float gasdev(long *idum); 
			long int *auxI=new long int[1];
			float ** tmp = new float * [Nb_Ref_Vector_Init];
			for (i=0;i<Nb_Ref_Vector_Init;i++)
				 {
				 tmp[i] = new float[nb_bands];
				 }
			for (j=0;j<nb_bands;j++){
				moy[j]=(*im_point).Mean(j);
				dev[j]=(*im_point).Stdev(j);
			}
			
			
			for (i=0;i<Nb_Ref_Vector_Init;i++){	
				
				auxI[0]=(unsigned long)((float)rand()*1);
				for (j=0;j<nb_bands;j++)
					{tmp[i][j]=gasdev(auxI)*i/Nb_Ref_Vector_Init*dev[j]+moy[j];}
				push_back(tmp[i]);
			}
			
			
			nb_ref=Nb_Ref_Vector_Init+nb_ref;
			cout <<Nb_Ref_Vector_Init<<" added Reference Vectors"<<endl;		
			
			 delete [] moy; delete [] dev; delete [] auxI;
			
		}
		/*---------------------------	-----------------------------------------------------------------*/

	//private:	
		/*--------------------------------------------------------------------------------------------*/
			/*!@brief  method to initialise component in the mixture according to the data stdev and mean */
			void Initialise(Image<PIXEL> *data_point,int nb_init)
		{
			int i,j;
			unsigned long int dim;
			dim=(*data_point).nx*(*data_point).ny;
			
			if(nb_init==-1){ nb_init=(int)((log((double)dim)))*3.5*log((double)im_point->nb_bands)*2.0;}
		    
			int nb_bands=(*data_point).nb_bands;
			int Nb_Ref_Vector_Init=(nb_init);
			float ** tmp = new float * [Nb_Ref_Vector_Init];
			float *moy=new float[nb_bands];
			float *dev=new float[nb_bands];
			float gasdev(long *idum); 
			long int *auxI=new long int[1];
			//auxI[0]=-(int)(((float)rand())/RAND_MAX);
			for (i=0;i<Nb_Ref_Vector_Init;i++)
				 {
				 tmp[i] = new float[nb_bands];
				 }
			for (j=0;j<nb_bands;j++){
				moy[j]=(*data_point).Mean(j);
				dev[j]=(*data_point).Stdev(j);
			}
			
			for (i=0;i<Nb_Ref_Vector_Init;i++){	
				
				auxI[0]=(unsigned long)((float)rand());
				for (j=0;j<nb_bands;j++)
					{tmp[i][j]=gasdev(auxI)*dev[j]+moy[j];}
				push_back(tmp[i]);
			}
			
			nb_ref=Nb_Ref_Vector_Init;
			cout << "Initialisation of " <<Nb_Ref_Vector_Init<<" Reference Vectors done"<<endl;		
			 delete [] moy; delete [] dev; delete [] auxI;
			
			
		}
		/*--------------------------------------------------------------------------------------------*/

		

		
};
/**********************************************************************************************/
/*!@brief Outliers object, which are not considered here\n
 OBJECT HAS TO BE REMOVED*/
class Outliers : public Generic_Vector
{

	public:
		/*!@brief number of outliers: always equal 0*/
		int nb_outliers;
		
		Outliers() : Generic_Vector()
		{
		Initialise();
		}
	private:
		/*!@brief fixing the number of outliers to 0*/
		void Initialise()
		{
		nb_outliers=0;
		}



};
/**********************************************************************************************/
