/*!@file MI.cpp 
*@brief File containing the MI object source */
#include <iostream>
#include <string>  
#include <vector>
#include <fstream>

#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cassert>

using namespace std;

#include "MI.hh"

MI::MI(){}
/*****************************************************************************************************/
MI::~MI(){}
/*****************************************************************************************************/
void MI::Allocate(int nb_dimensions){
	
	nb_dim=nb_dimensions;
	
		float* AuxMeanT1=new float[nb_dim];					
    		float** AuxCovT1=new float*[nb_dim];			   			 	
		for (int i=0;i<nb_dim;i++){
			AuxCovT1[i]=new float[nb_dim];			
		}	   	
    		float* AuxdeterminantT1=new float[1];
		MeanT1.push_back(AuxMeanT1);
		CovT1.push_back(AuxCovT1);
		determinantT1.push_back(AuxdeterminantT1[0]);  
	
	
		float* AuxMeanT2=new float[nb_dim];
		float** AuxCovT2=new float*[nb_dim];
		for (int i=0;i<nb_dim;i++){
			AuxCovT2[i]=new float[nb_dim];
		}
		float* AuxdeterminantT2=new float[1];
		MeanT2.push_back(AuxMeanT2);
		CovT2.push_back(AuxCovT2);
		determinantT2.push_back(AuxdeterminantT2[0]);
	
		float* AuxMeanT12=new float[nb_dim*2];					
    		float** AuxCovT12=new float*[nb_dim*2];			   			 	
		for (int i=0;i<nb_dim*2;i++){
			AuxCovT12[i]=new float[nb_dim*2];			
		}	   	
    		float* AuxdeterminantT12=new float[1];
		MeanT12.push_back(AuxMeanT12);
		CovT12.push_back(AuxCovT12);
		determinantT12.push_back(AuxdeterminantT12[0]); 
	

} 
/*****************************************************************************************************/
void MI::Initialise1(vector<float**> *C1,vector<float*> *M1)
{
	CovT1=*C1;
	MeanT1=*M1;
	for (int j1=0;j1<nb_dim;j1++){CovT1[0][0][j1]=0;CovT1[0][j1][0]=0;}
	DetCovMatrixT1();

}
/*****************************************************************************************************/
void MI::Initialise2(vector<float**> *C2,vector<float*> *M2)
{
	CovT2=*C2;
	MeanT2=*M2;
	
	DetCovMatrixT2();

}
/*****************************************************************************************************/
void MI::Initialise12(vector<float**> *C12,vector<float*> *M12)
{
	CovT12=*C12;
	MeanT12=*M12;
	
	DetCovMatrixT12();
}
/*****************************************************************************************************/
void MI::DetCovMatrixT1(){

   int j1,j2,j;
   
/*
**  Putting the zero components as the last dimsensions in the covariance matrix
*/
   	
   int nb_Gdim=nb_dim;
   float aux;
   for (j1=0;j1<nb_dim;j1++){
		if (CovT1[0][j1][j1]==0){
		    	nb_Gdim--;
		}
   }
   bool again=true;
   while(again){
      for (j1=0;j1<nb_dim;j1++)
      {
   	   if (CovT1[0][j1][j1]==0){
   	       if (j1<nb_dim-1){
   		   for (j2=0;j2<nb_dim;j2++){
   			   aux=CovT1[0][j1][j2];
   			   CovT1[0][j1][j2]=CovT1[0][j1+1][j2];
   			   CovT1[0][j1+1][j2]=aux;
   			   
   			   
   		   }
   		   for (j2=0;j2<nb_dim;j2++){	   
   			   aux=CovT1[0][j2][j1];
   			   CovT1[0][j2][j1]=CovT1[0][j2][j1+1];
   			   CovT1[0][j2][j1+1]=aux;
   			  
   		   }
   	       }
   	   }
      }
      for (j1=0;j1<nb_Gdim;j1++){
   	   if (CovT1[0][j1][j1]==0){again=true;}
   	   else {again=false;}
      }
      if (nb_Gdim==0){again=false;}
   }
   
/*
** Determinant considering only the non singular dimensions
*/
      
   
   float **auxCov=new float*[nb_Gdim+1];
   for (j=0;j<nb_Gdim+1;j++)
		auxCov[j]=new float[nb_Gdim+1]; 
   float d;
   int *indx=new int[nb_Gdim+1]; 
   int n=nb_Gdim;
   void ludcmp(float **a,int n,int *indx,float *d);
  		
   for (j1=0;j1<nb_Gdim;j1++)
   {
   	   for (j2=0;j2<nb_Gdim;j2++){auxCov[j1+1][j2+1]=CovT1[0][j1][j2];}
   }

   ludcmp(auxCov,n,indx,&d);//LU decomposition of COV MATRIX T1
   for (j=1;j<nb_Gdim+1;j++){d=auxCov[j][j]*d;}
   determinantT1[0]=d;//DETERMINANT OF THE COV MATRIX 
    
	
   for (j=0;j<nb_Gdim+1;j++){delete [] auxCov[j];}
   delete [] auxCov;delete [] indx;
	
}
/*****************************************************************************************************/
void MI::DetCovMatrixT2(){

   int j1,j2,j;
   
      
/*
**  Putting the zero components as the last dimsensions in the covariance matrix
*/
   	
   int nb_Gdim=nb_dim;
   float aux;
   for (j1=0;j1<nb_dim;j1++){
		if (CovT2[0][j1][j1]==0){
		    	nb_Gdim--;
		}
   }
   bool again=true;
   while(again){
      for (j1=0;j1<nb_dim;j1++)
      {
   	   if (CovT2[0][j1][j1]==0){
   	       if (j1<nb_dim-1){
   		   for (j2=0;j2<nb_dim;j2++){
   			   aux=CovT2[0][j1][j2];
   			   CovT2[0][j1][j2]=CovT2[0][j1+1][j2];
   			   CovT2[0][j1+1][j2]=aux;
   			   
   			   
   		   }
   		   for (j2=0;j2<nb_dim;j2++){	   
   			   aux=CovT2[0][j2][j1];
   			   CovT2[0][j2][j1]=CovT2[0][j2][j1+1];
   			   CovT2[0][j2][j1+1]=aux;
   			  
   		   }
   	       }
   	   }
      }
      for (j1=0;j1<nb_Gdim;j1++){
   	   if (CovT2[0][j1][j1]==0){again=true;}
   	   else {again=false;}
      }
      if (nb_Gdim==0){again=false;}
   }
   
/*
** Determinant considering only the non singular dimensions
*/

   float **auxCov=new float*[nb_Gdim+1];
   for (j=0;j<nb_Gdim+1;j++)
	{
		auxCov[j]=new float[nb_Gdim+1];
	}
   float d;
   int *indx=new int[nb_Gdim+1]; 
   int n=nb_Gdim;
   void ludcmp(float **a,int n,int *indx,float *d);
	
   for (j1=0;j1<nb_Gdim;j1++)
   {
   	   for (j2=0;j2<nb_Gdim;j2++){auxCov[j1+1][j2+1]=CovT2[0][j1][j2];}

   }
   ludcmp(auxCov,n,indx,&d);//LU decomposition of COV MATRIX T2
   for (j=1;j<nb_Gdim+1;j++){d=auxCov[j][j]*d;}
   determinantT2[0]=d;//DETERMINANT OF THE COV MATRIX 
	
 	
   for (j=0;j<nb_Gdim+1;j++){delete [] auxCov[j];}
   delete [] auxCov;delete [] indx;
	
	
}
/*****************************************************************************************************/
void MI::DetCovMatrixT12(){
	
   int nb_dimAux=nb_dim*2;
   int j1,j2,j;
   
   /*
**  Putting the zero components as the last dimsensions in the covariance matrix
*/
   	
   int nb_Gdim=nb_dimAux;
   float aux;
   for (j1=0;j1<nb_dimAux;j1++){
		if (CovT12[0][j1][j1]==0){
		    	nb_Gdim--;
		}
   }
   bool again=true;
   while(again){
      for (j1=0;j1<nb_dimAux;j1++)
      {
   	   if (CovT12[0][j1][j1]==0){
   	       if (j1<nb_dimAux-1){
   		   for (j2=0;j2<nb_dimAux;j2++){
   			   aux=CovT12[0][j1][j2];
   			   CovT12[0][j1][j2]=CovT12[0][j1+1][j2];
   			   CovT12[0][j1+1][j2]=aux;
   			   
   			   
   		   }
   		   for (j2=0;j2<nb_dimAux;j2++){	   
   			   aux=CovT12[0][j2][j1];
   			   CovT12[0][j2][j1]=CovT12[0][j2][j1+1];
   			   CovT12[0][j2][j1+1]=aux;
   			  
   		   }
   	       }
   	   }
      }
      for (j1=0;j1<nb_Gdim;j1++){
   	   if (CovT12[0][j1][j1]==0){again=true;}
   	   else {again=false;}
      }
      if (nb_Gdim==0){again=false;}
   }
   
/*
** Determinant considering only the non singular dimensions
*/

   
   float **auxCov=new float*[nb_Gdim+1];
   for (j=0;j<nb_Gdim+1;j++)
	{
		auxCov[j]=new float[nb_Gdim+1];
	}
   float d;
   int *indx=new int[nb_Gdim+1]; 
   int n=nb_Gdim;
   void ludcmp(float **a,int n,int *indx,float *d);
	   	
   for (j1=0;j1<nb_Gdim;j1++)
   {
   	   for (j2=0;j2<nb_Gdim;j2++){auxCov[j1+1][j2+1]=CovT12[0][j1][j2];} 
   	    
   }
   ludcmp(auxCov,n,indx,&d);//LU decomposition of COV MATRIX T12
   for (j=1;j<nb_Gdim+1;j++){d=auxCov[j][j]*d;}
   determinantT12[0]=d;//DETERMINANT OF THE COV MATRIX 
		
   for (j=0;j<nb_Gdim+1;j++){delete [] auxCov[j];}
   delete [] auxCov;delete [] indx;
	
	
	
}

/******************************************************************************************/

float MI::MICalculus()
{	
	float res=0.5*log((determinantT1[0]*determinantT2[0])/determinantT12[0])/log(2);		
	return res;
}

