/*!@file graphCharac.cpp 
*@brief File containing the GraphDynaCluster object source */
#include <iostream>
#include <string>  
#include <vector>
#include <fstream>

#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cassert>


#include "graphCharac.hh"


using namespace std;

/*****************************************************************************************************/
GraphDynaCluster::GraphDynaCluster()
{

}
/*****************************************************************************************************/
GraphDynaCluster::~GraphDynaCluster()
{
	
}
/*****************************************************************************************************/
void GraphDynaCluster::Initialise(char * path_in_Temporal, char * path_in_Timeless, int dimx_tmp, int dimy_tmp, int nb_bands)
{
	nb_dim=nb_bands;
	dim=dimx_tmp*dimy_tmp;
	nx=dimx_tmp;
	ny=dimy_tmp;
	
	Acquire_path(path_in_Temporal,nb_dim,path_in_Timeless);	
	Allocate_variables();
	Aquire_data();
} 
/*****************************************************************************************************/ 
void GraphDynaCluster::Acquire_path(char * path_in_Temporal, int nb_bands, char* path_in_Timeless)  
{
	
	int m,k;
	Image<unsigned char> pippo;
	nb_images=pippo.NumberImages_pathfiles_in(path_in_Temporal,2);
	
	//auxiliary paths
	char ***PathInTemp= new char **[nb_images];
	for(m=0;m<nb_images;++m){PathInTemp[m] =new char*[2];} 
	for(m=0;m<nb_images;++m){for(k=0;k<2;++k){PathInTemp[m][k]=new char[128];}}
	//read 
	pippo.Read_pathfiles_in(path_in_Temporal,PathInTemp,2);
	
	char ***PathInTimeless= new char **[1];
	for(m=0;m<1;++m){PathInTimeless[m] =new char*[2];} 
	for(m=0;m<1;++m){for(k=0;k<2;++k){PathInTimeless[m][k]=new char[128];}}
	//read
	pippo.Read_pathfiles_in(path_in_Timeless,PathInTimeless,2);
	
	//allocation path
	PathInTemp_Classfile= new char **[nb_images];
	for(m=0;m<nb_images;++m){PathInTemp_Classfile[m] =new char*[1];} 
	for(m=0;m<nb_images;++m){for(k=0;k<1;++k){PathInTemp_Classfile[m][k]=new char[128];}}
	
	PathInTimeless_Classfile= new char **[1];
	for(m=0;m<1;++m){PathInTimeless_Classfile[m] =new char*[1];} 
	for(m=0;m<1;++m){for(k=0;k<1;++k){PathInTimeless_Classfile[m][k]=new char[128];}}
	
	PathInTemp_MeanCov= new char **[nb_images];
	for(m=0;m<nb_images;++m){PathInTemp_MeanCov[m] =new char*[1];} 
	for(m=0;m<nb_images;++m){for(k=0;k<1;++k){PathInTemp_MeanCov[m][k]=new char[128];}}
	
	PathInTimeless_MeanCov= new char **[1];
	for(m=0;m<1;++m){PathInTimeless_MeanCov[m] =new char*[1];} 
	for(m=0;m<1;++m){for(k=0;k<1;++k){PathInTimeless_MeanCov[m][k]=new char[128];}}

	
	//copy auxiliary paths in paths
	for (m=0;m<nb_images;m++)
	{
		strcpy(PathInTemp_Classfile[m][0],PathInTemp[m][0]);
		strcpy(PathInTemp_MeanCov[m][0],PathInTemp[m][1]);
	}
	strcpy(PathInTimeless_Classfile[0][0],PathInTimeless[0][0]);
	strcpy(PathInTimeless_MeanCov[0][0],PathInTimeless[0][1]);
	
	
	
	//delete auxiliary paths
	for(m=0;m<nb_images;++m){for(k=0;k<2;++k){delete [] PathInTemp[m][k];}}
	for(m=0;m<nb_images;++m){delete [] PathInTemp[m];}
	for(k=0;k<2;++k){delete [] PathInTimeless[0][k];}
	

	
}
/****************************************************************************************************/
void GraphDynaCluster::Allocate_variables()
{
	//aux_variables allocation
	Current_C_A=new float*[nb_dim]; Current_C_B=new float*[nb_dim];Current_C_AB=new float*[nb_dim*2]; 
 	Current_M_A=new float[nb_dim]; Current_M_B=new float[nb_dim]; Current_M_AB=new float[nb_dim*2];
	
	for (int j=0;j<nb_dim;j++)
	{
	Current_C_A[j]=new float[nb_dim];
	Current_C_B[j]=new float[nb_dim];
	}
	for (int j=0;j<nb_dim*2;j++){Current_C_AB[j]=new float[nb_dim*2];}
	
	//classfiles allocation
	Temp_classfile.Allocate(nx,ny,1,nb_images);
	Timeless_classfile.Allocate(nx,ny,1,1);
	
	//attributes allocation
	Temp_MeanCov.Allocate(255,(nb_dim+1)*nb_dim,1,nb_images);
	Timeless_MeanCov.Allocate(255,(nb_dim*nb_images+1)*nb_dim*nb_images,1,1);
	
	
	//SuperposeClass alllocation
	supClass=new SuperposeClass[nb_images];
		
	
}
/*****************************************************************************************************/
void GraphDynaCluster::Aquire_data()
{
	//reading attributes 
	Temp_MeanCov.Read_Cut_SubSample(PathInTemp_MeanCov,0,0,Temp_MeanCov.ny,1,1);
	Timeless_MeanCov.Read_Cut_SubSample(PathInTimeless_MeanCov,0,0,Timeless_MeanCov.ny,1,1);
	
	//reading classfiles
	Temp_classfile.Read_Cut_SubSample(PathInTemp_Classfile,0,0,ny,1,1);
	Timeless_classfile.Read_Cut_SubSample(PathInTimeless_Classfile,0,0,ny,1,1);
}
/*****************************************************************************************************/
void GraphDynaCluster::Load_Current(int j1,int j2)
{
	int k,l;
	unsigned long int dim_MeanCov=Temp_MeanCov.nx*Temp_MeanCov.ny;
	for(k=0;k<nb_dim;k++)
	{
		Current_M_A[k]=(*(Temp_MeanCov.position+dim_MeanCov*Index_CurrentImage+(nb_dim+1)*nb_dim*(j1+1)+k));
		
		Current_M_B[k]=(*(Timeless_MeanCov.position+(nb_dim*nb_images+1)*nb_dim*nb_images*(j2+1)+Index_CurrentImage*nb_dim+k));
		if (k==0){Current_M_A[k]/=alpha0;Current_M_B[k]/=alpha0;}
		if (k==1){Current_M_A[k]/=alpha1;Current_M_B[k]/=alpha1;}
		if (k==2){Current_M_A[k]/=alpha2;Current_M_B[k]/=alpha2;}
		for (l=0;l<nb_dim;l++)
		{
			Current_C_A[k][l]=(*(Temp_MeanCov.position+dim_MeanCov*Index_CurrentImage+(nb_dim+1)*nb_dim*(j1+1)+(k+1)*nb_dim+l));
			Current_C_B[k][l]=(*(Timeless_MeanCov.position+(nb_dim*nb_images+1)*nb_dim*nb_images*(j2+1)+(k+1+Index_CurrentImage*nb_dim)*nb_dim*nb_images+l+Index_CurrentImage*nb_dim));
			if (k==0){Current_C_A[k][l]/=alpha0;Current_C_B[k][l]/=alpha0;}
			if (k==1){Current_C_A[k][l]/=alpha1;Current_C_B[k][l]/=alpha1;}
			if (k==2){Current_C_A[k][l]/=alpha2;Current_C_B[k][l]/=alpha2;}
			if (l==0){Current_C_A[k][l]/=alpha0;Current_C_B[k][l]/=alpha0;}
			if (l==1){Current_C_A[k][l]/=alpha1;Current_C_B[k][l]/=alpha1;}	
			if (l==2){Current_C_A[k][l]/=alpha2;Current_C_B[k][l]/=alpha2;}	
		}
		
	}

}
/*****************************************************************************************************/
void GraphDynaCluster::Build(float Factor)
{
	int j1,j2,j,k;
	int nb_CurrentClasses;
	PrecisionFactor=Factor;
	
	Divergence DivObject(nb_dim);
	float *min_bound=new float[nb_dim];float *max_bound=new float[nb_dim];
	float *resolution=new float[nb_dim];
	float DivResult,MinDivResult;
	int IndexMinDiv;
	
	
	nb_TimelessClasses=(int)*(Timeless_MeanCov.position);
	nb_CurrentClasses=(int)*(Temp_MeanCov.position);
	
	//Dynamic classe objects allocation (only for nearest TL class)
	 TmpClass=new DynaClass[nb_TimelessClasses];
	for (j1=0;j1<nb_TimelessClasses;j1++)
	{	
		TmpClass[j1].Allocate(nb_images,nb_dim);
	}
	
	
	//Divergence Calculation
	for (Index_CurrentImage=0;Index_CurrentImage<nb_images;Index_CurrentImage++)
	{
	        supClass[Index_CurrentImage].AcquirePara(Index_CurrentImage,nb_dim,nb_images,nb_TimelessClasses,&Temp_classfile,&Timeless_classfile);
		
		nb_CurrentClasses=(int)*(Temp_MeanCov.position+Temp_MeanCov.nx*Temp_MeanCov.ny*Index_CurrentImage);
		cout<<"Number of classes at time "<<Index_CurrentImage+1<<" : "<<nb_CurrentClasses<<endl;
		float *VectorDiv=new float[255];  //max nb of classes: 255!!
		int *VectorDivIndex=new int[255];
		float boucle=0;
		float auxINF=2.*INF;
		int CalculateDiv;
		float DivWeight;
		for (j2=0;j2<nb_TimelessClasses;j2++)
		{	
		
			for (j1=0;j1<nb_CurrentClasses;j1++){
				VectorDiv[j1]=auxINF;
			}
		   	//Searching the divergence 
			MinDivResult=INF;
			IndexMinDiv=-1;
			for (j1=0;j1<nb_CurrentClasses;j1++){
				Load_Current(j1,j2);
				DivObject.Initialise (Current_C_A, Current_C_B, Current_M_A, Current_M_B );
				DivObject.InvertCovMatrix();
				CalculateDiv=DivObject.BoundResolution_Calculation(min_bound,max_bound,resolution,PrecisionFactor);					DivWeight=(float)supClass[Index_CurrentImage].DivWeight(j1,j2)	;
				if (CalculateDiv<1||DivWeight!=0){
				   DivResult=-1;
				   boucle=0;
				   while (DivResult<0 && boucle<3){
				     boucle++; 
				     DivResult=DivObject.Divergence2GaussModel(min_bound,max_bound,resolution,DivWeight);// spatial constrain introduce via DivWeight.
			              for (int jaux=0;jaux<nb_dim;jaux++){resolution[jaux]=resolution[jaux]*0.5;}
				   }
				   if (DivResult<=0){
				   	DivResult=DivObject.MeanDifference();
				  }
				}
				else{DivResult=INF;}
				if (DivWeight==0){DivResult=INF;}
				if (DivResult<MinDivResult){MinDivResult=DivResult;IndexMinDiv=j1;}
				if (DivResult>=auxINF){DivResult=INF;}
				for (int jj=0;jj<nb_CurrentClasses;jj++){
					if (DivResult<VectorDiv[jj]){
						for (int jjj=nb_CurrentClasses;jjj>jj;jjj--){ 
							VectorDiv[jjj]=VectorDiv[jjj-1];
							VectorDivIndex[jjj]=VectorDivIndex[jjj-1];
							
						}
						VectorDiv[jj]=DivResult;
						VectorDivIndex[jj]=j1;
						break;
					}	
				}
				
					
			}
			
			cout<<"multitemporal class : "<<j2<<" assigned to timelocalized class "<<IndexMinDiv<<"; measurement : "<<MinDivResult<<endl;
			

			//Dynamic classe objects allocation (exept for nearest TL class)
			for (int jj=1;jj<nb_CurrentClasses;jj++){
				int *AuxAssignedTimelessClass=new int[1];
				unsigned long int *AuxWeight=new unsigned long int[1];
				float  *AuxDivergenceValue=new float[1];
				float  **AuxCentroids=new float*[1];
				float ***AuxCovariances=new float**[1];
				AuxCentroids[0]=new float[nb_dim];
				AuxCovariances[0]=new float*[ nb_dim];
				for (j=0;j<nb_dim;j++)
				{
					AuxCovariances[0][j]=new float[nb_dim];
				}
				
			        TmpClass[j2].AssignedTimelessClass[Index_CurrentImage].push_back(AuxAssignedTimelessClass[0]);
				TmpClass[j2].DivergenceValue[Index_CurrentImage].push_back(AuxDivergenceValue[0]);
				TmpClass[j2].Centroids[Index_CurrentImage].push_back(AuxCentroids[0]);
				TmpClass[j2].Covariances[Index_CurrentImage].push_back(AuxCovariances[0]);
				TmpClass[j2].Weight[Index_CurrentImage].push_back(AuxWeight[0]);
				
			}
			
			for (int jj=0;jj<nb_CurrentClasses;jj++){
			
				TmpClass[j2].AssignedTimelessClass[Index_CurrentImage][jj]=VectorDivIndex[jj];
				TmpClass[j2].Weight[Index_CurrentImage][jj]=(unsigned long int)supClass[Index_CurrentImage].DivWeight(VectorDivIndex[jj],j2);
				//cout<<"timelocalized class "<<VectorDivIndex[jj]<<"; Divergence : "<<VectorDiv[jj]<<" "<<TmpClass[j2].Weight[Index_CurrentImage][jj]<<"";"";
				
				TmpClass[j2].DivergenceValue[Index_CurrentImage][jj]=VectorDiv[jj];
				Load_Current(VectorDivIndex[jj],j2);
				for (j=0;j<nb_dim;j++)
				{
					TmpClass[j2].Centroids[Index_CurrentImage][jj][j]=Current_M_A[j];
					//cout<<" "<<Current_M_A[j]<<"/"<<Current_M_B[j];
					for (k=0;k<nb_dim;k++)
					{
					   TmpClass[j2].Covariances[Index_CurrentImage][jj][j][k]=Current_C_A[j][k];
					}
					//cout<<","<<Current_C_A[j][j]<<"/"<<Current_C_B[j][j];
				}
				//cout<<""<<endl;
				
			}
			
			
		}
	
	}
}
/*****************************************************************************************************/	
void GraphDynaCluster::Out_Evolutions(int format, bool intensiveCalculus)
{
    
     if (format==2){//format graph where the mean and covariance are stored many times (for the actual implementation of the learning procedure and  of the GUI) 
        Create_TxtGraphOutPutVersion2(intensiveCalculus);
	ReadWriteGraphVersion2JAVA("./grapheV2",1);
     } 
     if (format==3){//format graph where the mean and covariance are stored many times (for the actual implementation of the learning procedure and creation of a format without edge for the GUI) 
        Create_TxtGraphOutPutVersion2(intensiveCalculus);
	ReadWriteGraphVersion2JAVA("./grapheV2",0);
     }
     else{//Standard graph format output
        if (format!=2){
		Create_TxtGraphOutPutVersion1(intensiveCalculus);
	}
     }

}
/*****************************************************************************************************/	
void GraphDynaCluster::Create_TxtGraphOutPutVersion1(bool intensiveCalculus )
{

	char *path=new char[128];
	FILE *ifp;
	strcpy(path,"./grapheV1");
	ifp=NULL; 
     	assert (ifp=fopen(path,"w"));
	int length;
	float flow;
	float weight;
	//float auxInf=INF;
	Divergence divObject(nb_dim);
	float *min_bound=new float[nb_dim];float *max_bound=new float[nb_dim];
	float *resolution=new float[nb_dim];float *resolutionAux=new float[nb_dim];
	int calculateDiv=0;float DivResult=0;
	fprintf(ifp,"%d %s",(int) nb_TimelessClasses," ");//nb MT clusters
	fprintf(ifp,"%d %s",(int) nb_dim," ");//nb dimension FS
	fprintf(ifp,"%d %s",(int) nb_images," ");//nb images samples
	//for each MT class
	for (int kMT=0;kMT<nb_TimelessClasses;kMT++){
	    cout<<"MT class "<<kMT<<endl;
	    fprintf(ifp,"%d %s",kMT," ");//MT index
	    //for each TL window
	    for(int t=0;t<nb_images;t++){
	    	fprintf(ifp,"%d %s",t," ");//time sample
	        Index_CurrentImage=t;
		Load_MonoCurrentPMT(kMT);
	    	length=(int)TmpClass[kMT].AssignedTimelessClass[t].size();
		fprintf(ifp,"%d %s",length," ");//number of assigned  TLs
		//for each TL_t class
	    	for (int kTL=0;kTL<(int)length;kTL++){ 
			fprintf(ifp,"%d %s", (int)TmpClass[kMT].AssignedTimelessClass[t][kTL]," ");//TL_t cluster index
			fprintf(ifp,"%u %s", (unsigned int)TmpClass[kMT].Weight[t][kTL]," ");//pixel quantity belonging to classes kTL_t & kMT
			fprintf(ifp,"%e %s", (float)TmpClass[kMT].DivergenceValue[t][kTL]*weight," ");//divergence between classes kTL_t & kMT
							
			if (t>0){
			    //for each assigne TL_(t-1)class
			    for (int kTLpre=0;kTLpre<(int)TmpClass[kMT].AssignedTimelessClass[t-1].size();kTLpre++){
				fprintf(ifp,"%d %s",(int)TmpClass[kMT].AssignedTimelessClass[t-1][kTLpre]," ");//TL_(t-1) cluster index
				flow=(unsigned long int)supClass[t].FlowWeight(TmpClass[kMT].AssignedTimelessClass[t][kTL],TmpClass[kMT].AssignedTimelessClass[t-1][kTLpre],kMT);
				fprintf(ifp,"%u %s",(unsigned int) flow," ");//pixel quantity belonging to classes kTL_t&kTL_(t-1)&kMT : flow
	  		 	LoadpreTLTL(TmpClass[kMT].AssignedTimelessClass[t-1][kTLpre],TmpClass[kMT].AssignedTimelessClass[t][kTL]);
				
			 	divObject.Initialise(Current_C_A, Current_C_B, Current_M_A, Current_M_B);
				divObject.InvertCovMatrix();
			 	calculateDiv=divObject.BoundResolution_Calculation(min_bound,max_bound,resolution,PrecisionFactor);
				if (intensiveCalculus==true && calculateDiv<1){ //Divergence between consecutive TL classes (intensive calculation)				
					DivResult=-1;
					for (int jaux=0;jaux<nb_dim;jaux++){resolutionAux[jaux]=resolution[jaux];}
				  	while (DivResult<0&&(float)resolution[0]/resolutionAux[0]<=(float)1){
				      	  DivResult=divObject.Divergence2GaussModel(min_bound,max_bound,resolutionAux,1.);//div TL(t-1) TLt
			                  for (int jaux=0;jaux<nb_dim;jaux++){resolutionAux[jaux]=resolutionAux[jaux]*0.5;}
				   	}
					if (DivResult<=0){DivResult=divObject.MeanDifference();}
					//DivResult=sqrt(DivResult*DivResult);
			 		fprintf(ifp,"%e %s",DivResult," ");//div TL(t-1) TLt
				  
				}
				else{//Mean difference between consecutive TL classes (fast calculation)
					DivResult=divObject.MeanDifference();
					fprintf(ifp,"%e %s",DivResult," ");
				}
			    }
			}
		}

		if (t>0){ fprintf(ifp,"%f %s", (float)TmpClass[kMT].MutualInfo[t-1]," ");}// Mutual information of MT class between t(t-1)
	    }
	}
     	assert (EOF!=fclose(ifp)); 
	delete [] min_bound; delete [] max_bound;
	delete [] resolution;
		
}
/*****************************************************************************************************/	
void GraphDynaCluster::Create_TxtGraphOutPutVersion2(bool intensiveCalculus )
{
	char *path=new char[128];
	FILE *ifp;
	strcpy(path,"./grapheV2");
	ifp=NULL; 
     	assert (ifp=fopen(path,"w"));
	int length;
	float flow;
	float weight;
	//float auxInf=INF;
	Divergence divObject(nb_dim);
	float *min_bound=new float[nb_dim];float *max_bound=new float[nb_dim];
	float *resolution=new float[nb_dim];float *resolutionAux=new float[nb_dim];
	int calculateDiv=0;float DivResult=0;
	fprintf(ifp,"%d %s",(int) nb_TimelessClasses," ");//nb MT clusters
	fprintf(ifp,"%d %s",(int) nb_dim," ");//nb dimension FS
	fprintf(ifp,"%d %s",(int) nb_images," ");//nb images samples
	//for each MT class
	for (int kMT=0;kMT<nb_TimelessClasses;kMT++){
	    cout<<"MT class "<<kMT<<endl;
	    fprintf(ifp,"%d %s",kMT," ");//MT index
	    weight=(float)supClass[0].MTPopulation(kMT);
	    fprintf(ifp,"%u %s",(unsigned int)weight," ");//MT weight
	    //for each TL window
	    for(int t=0;t<nb_images;t++){
	    	fprintf(ifp,"%d %s",t," ");//time sample
	        Index_CurrentImage=t;
		Load_MonoCurrentPMT(kMT);
		//mean and cov kMT
	        for (int j=0;j<nb_dim;j++){fprintf(ifp,"%f %s",Current_M_A[j]," ");}
		for (int j=0;j<nb_dim;j++){
			for (int l=0;l<nb_dim;l++){
				fprintf(ifp,"%f %s",Current_C_A[j][l]," ");
			}
		}
	    	length=(int)TmpClass[kMT].AssignedTimelessClass[t].size();
		fprintf(ifp,"%d %s",length," ");//number of assigned  TLs
		//for each TL_t class
	    	for (int kTL=0;kTL<(int)length;kTL++){ 
			fprintf(ifp,"%d %s", (int)TmpClass[kMT].AssignedTimelessClass[t][kTL]," ");//TL_t cluster index
			fprintf(ifp,"%u %s", (unsigned int)TmpClass[kMT].Weight[t][kTL]," ");//pixel quantity belonging to classes kTL_t & kMT
			fprintf(ifp,"%e %s", (float)TmpClass[kMT].DivergenceValue[t][kTL]*weight," ");//divergence between classes kTL_t & kMT
			//mean and cov TL_t
			for (int i=0;i<nb_dim;i++){fprintf(ifp,"%f %s",TmpClass[kMT].Centroids[t][kTL][i]," ");}
			for (int i=0;i<nb_dim;i++){
				for (int j=0;j<nb_dim;j++){fprintf(ifp,"%f %s",TmpClass[kMT].Covariances[t][kTL][i][j]," ");}
			}				
			if (t>0){
			    //for each assigne TL_(t-1)class
			    for (int kTLpre=0;kTLpre<(int)TmpClass[kMT].AssignedTimelessClass[t-1].size();kTLpre++){
				fprintf(ifp,"%d %s",(int)TmpClass[kMT].AssignedTimelessClass[t-1][kTLpre]," ");//TL_(t-1) cluster index
				flow=(unsigned long int)supClass[t].FlowWeight(TmpClass[kMT].AssignedTimelessClass[t][kTL],TmpClass[kMT].AssignedTimelessClass[t-1][kTLpre],kMT);
				fprintf(ifp,"%u %s",(unsigned int) flow," ");//pixel quantity belonging to classes kTL_t&kTL_(t-1)&kMT : flow
	  		 	LoadpreTLTL(TmpClass[kMT].AssignedTimelessClass[t-1][kTLpre],TmpClass[kMT].AssignedTimelessClass[t][kTL]);
				
			 	divObject.Initialise(Current_C_A, Current_C_B, Current_M_A, Current_M_B);
				divObject.InvertCovMatrix();
			 	calculateDiv=divObject.BoundResolution_Calculation(min_bound,max_bound,resolution,PrecisionFactor);
				if (intensiveCalculus==true && calculateDiv<1){
					
					DivResult=-1;
					for (int jaux=0;jaux<nb_dim;jaux++){resolutionAux[jaux]=resolution[jaux];}
				  	while (DivResult<0&&(float)resolution[0]/resolutionAux[0]<=(float)1){
				      	  DivResult=divObject.Divergence2GaussModel(min_bound,max_bound,resolutionAux,1.);//div TL(t-1) TLt
			                  for (int jaux=0;jaux<nb_dim;jaux++){resolutionAux[jaux]=resolutionAux[jaux]*0.5;}
				   	}
					if (DivResult<=0){DivResult=divObject.MeanDifference();}
					//DivResult=sqrt(DivResult*DivResult);
			 		fprintf(ifp,"%e %s",DivResult," ");//div TL(t-1) TLt
				  
				}
				else{
					DivResult=divObject.MeanDifference();
					fprintf(ifp,"%e %s",DivResult," ");
				}
			    }
			}
		}

		if (t>0){ fprintf(ifp,"%f %s", (float)TmpClass[kMT].MutualInfo[t-1]," ");}// Mutual information of MT class between t(t-1)
	    }
	}
     	assert (EOF!=fclose(ifp)); 
	delete [] min_bound; delete [] max_bound;
	delete [] resolution;
		
}
/*****************************************************************************************************/	
void GraphDynaCluster::ReadWriteGraphVersion2JAVA(char * path, int edge)
{
	
	FILE *ifp;
	ifp=NULL; 
     	assert (ifp=fopen(path,"r"));
	int auxRead;int length;float auxReadF;float weightMT; 	
	
	char *pathw=new char[128];
	FILE *ifpw;
	strcpy(pathw,"./grapheV2JAVA");
	ifpw=NULL; 
     	assert (ifpw=fopen(pathw,"w"));
	
	fscanf(ifp,"%d",&auxRead);fprintf(ifpw,"%d %s",(int) auxRead," ");
	nb_TimelessClasses=auxRead;//nb MT clusters
	fscanf(ifp,"%d",&auxRead);fprintf(ifpw,"%d %s",(int) auxRead," ");
	nb_dim=auxRead;//nb dimension FS
	fscanf(ifp,"%d",&auxRead);fprintf(ifpw,"%d %s",(int) auxRead," ");
	nb_images=auxRead;//nb images samples
	
	
	//aux_variables allocation
	Current_C_A=new float*[nb_dim]; Current_C_B=new float*[nb_dim];Current_C_AB=new float*[nb_dim*2]; 
 	Current_M_A=new float[nb_dim]; Current_M_B=new float[nb_dim]; Current_M_AB=new float[nb_dim*2];
	for (int j=0;j<nb_dim;j++)
	{
	Current_C_A[j]=new float[nb_dim];
	Current_C_B[j]=new float[nb_dim];
	}
	for (int j=0;j<nb_dim*2;j++){Current_C_AB[j]=new float[nb_dim*2];}
	
	float *min_bound=new float[nb_dim];float *max_bound=new float[nb_dim];
	float *resolution=new float[nb_dim];
		
	//Dynamic classe objects allocation
	 TmpClass=new DynaClass[nb_TimelessClasses];
	for (int kMT=0;kMT<nb_TimelessClasses;kMT++){TmpClass[kMT].InitRead(nb_images,nb_dim);}
	
	//for each MT class
	for (int kMT=0;kMT<nb_TimelessClasses;kMT++){
	    fscanf(ifp,"%d",&auxRead);fprintf(ifpw,"%d %s",(int) auxRead," ");//MT index
	    fscanf(ifp,"%f",&weightMT);fprintf(ifpw,"%u %s",(unsigned int) weightMT," ");//MT weight   
	    //for each TL window
	    for(int t=0;t<nb_images;t++){
	        fscanf(ifp,"%d",&auxRead);fprintf(ifpw,"%d %s",(int) auxRead," ");//time sample
	        TmpClass[kMT].MTAlloc(t);
		TmpClass[kMT].MTWeight[t][0]=(unsigned long int)weightMT;
		//mean and cov kMT
	        for (int j=0;j<nb_dim;j++){
			fscanf(ifp,"%f",&auxReadF);fprintf(ifpw,"%f %s",(float) auxReadF," ");
			TmpClass[kMT].MTCentroids[t][0][j]=auxReadF;
		}
		for (int j=0;j<nb_dim;j++){
			for (int l=0;l<nb_dim;l++){
				fscanf(ifp,"%f",&auxReadF);fprintf(ifpw,"%f %s",(float) auxReadF," ");
				TmpClass[kMT].MTCovariances[t][0][j][l]=auxReadF;
			}
		} 
		fscanf(ifp,"%d",&auxRead);fprintf(ifpw,"%d %s",(int) auxRead," ");
	    	length=auxRead;//number of assigned  TLs
		TmpClass[kMT].TLAlloc(t,length);
		if (t>0){TmpClass[kMT].flowDivAlloc(t,(int)TmpClass[kMT].AssignedTimelessClass[t-1].size(),length);}
		//for each TL_t class
	    	for (int kTL=0;kTL<(int)length;kTL++){ 
			fscanf(ifp,"%d",&auxRead);fprintf(ifpw,"%d %s",(int) auxRead," ");
			TmpClass[kMT].AssignedTimelessClass[t][kTL]=auxRead;//TL_t cluster index
			fscanf(ifp,"%f",&auxReadF);fprintf(ifpw,"%u %s",(unsigned int) auxReadF," ");
			TmpClass[kMT].Weight[t][kTL]=(unsigned long int)auxReadF;//pixel quantity belonging to classes kTL_t & kMT
			fscanf(ifp,"%e",&auxReadF);fprintf(ifpw,"%e %s",(double) auxReadF," ");
			TmpClass[kMT].DivergenceValue[t][kTL]=(float)auxReadF/(float)TmpClass[kMT].MTWeight[t][0];//divergence between classes kTL_t & kMT
			//mean and cov TL_t
			for (int i=0;i<nb_dim;i++){
				fscanf(ifp,"%f",&auxReadF);fprintf(ifpw,"%f %s",(float) auxReadF," ");
				TmpClass[kMT].Centroids[t][kTL][i]=auxReadF;
			}
			for (int i=0;i<nb_dim;i++){
				for (int j=0;j<nb_dim;j++){
					fscanf(ifp,"%f",&auxReadF);fprintf(ifpw,"%f %s",(float) auxReadF," ");
					TmpClass[kMT].Covariances[t][kTL][i][j]=auxReadF;
				}
			}
			if (t>0){
 			   //for each assigne TL_(t-1)class
			   for (int kTLpre=0;kTLpre<(int)TmpClass[kMT].AssignedTimelessClass[t-1].size();kTLpre++){
			        fscanf(ifp,"%d",&auxRead);
				if (edge==1){fprintf(ifpw,"%d %s",(int) auxRead," ");}//TL_(t-1) cluster index
				fscanf(ifp,"%f",&auxReadF);
				if (edge==1){fprintf(ifpw,"%u %s",(unsigned int) auxReadF," ");}
				TmpClass[kMT].flow[t][kTLpre][kTL]=auxReadF;//pixel quantity belonging to classes kTL_t&kTL_(t-1)&kMT : flow
				fscanf(ifp,"%f",&auxReadF);
				if (edge==1){fprintf(ifpw,"%e %s",(double) auxReadF," ");}
				TmpClass[kMT].div[t][kTLpre][kTL]=auxReadF;//div TL(t-1) TLt
			   }
			   
			}
			fprintf(ifpw,"\n");
		}

		if (t>0){ 	
			fscanf(ifp,"%f",&auxReadF);fprintf(ifpw,"%e %s\n",(double) auxReadF," ");
			TmpClass[kMT].MutualInfo[t-1]=auxReadF;// Mutual information of MT class between t(t-1)
		}
	    }
	}
	dim=0;
	for (int kMT=0;kMT<nb_TimelessClasses;kMT++){dim+=TmpClass[kMT].MTWeight[0][0];}
	
     	assert (EOF!=fclose(ifp)); 
	delete [] min_bound; delete [] max_bound;
	delete [] resolution;
		
}

/*****************************************************************************************************/
void GraphDynaCluster::Load_CurrentPMT(int kMT)
{
	for(int k=0;k<nb_dim;k++)
	{
		Current_M_A[k]=(*(Timeless_MeanCov.position+(nb_dim*nb_images+1)*nb_dim*nb_images*(kMT+1)+(Index_CurrentImage-1)*nb_dim+k));
		Current_M_B[k]=(*(Timeless_MeanCov.position+(nb_dim*nb_images+1)*nb_dim*nb_images*(kMT+1)+Index_CurrentImage*nb_dim+k));
		if (k==0){Current_M_A[k]/=alpha0;Current_M_B[k]/=alpha0;}
		if (k==1){Current_M_A[k]/=alpha1;Current_M_B[k]/=alpha1;}
		if (k==2){Current_M_A[k]/=alpha2;Current_M_B[k]/=alpha2;}
		for (int l=0;l<nb_dim;l++)
		{
			Current_C_A[k][l]=(*(Timeless_MeanCov.position+(nb_dim*nb_images+1)*nb_dim*nb_images*(kMT+1)+(k+1+(Index_CurrentImage-1)*nb_dim)*nb_dim*nb_images+l+(Index_CurrentImage-1)*nb_dim));
			Current_C_B[k][l]=(*(Timeless_MeanCov.position+(nb_dim*nb_images+1)*nb_dim*nb_images*(kMT+1)+(k+1+Index_CurrentImage*nb_dim)*nb_dim*nb_images+l+Index_CurrentImage*nb_dim));
			if (k==0){Current_C_A[k][l]/=alpha0;Current_C_B[k][l]/=alpha0;}
			if (k==1){Current_C_A[k][l]/=alpha1;Current_C_B[k][l]/=alpha1;}
			if (k==2){Current_C_A[k][l]/=alpha2;Current_C_B[k][l]/=alpha2;}
			if (l==0){Current_C_A[k][l]/=alpha0;Current_C_B[k][l]/=alpha0;}	
			if (l==1){Current_C_A[k][l]/=alpha1;Current_C_B[k][l]/=alpha1;}
			if (l==2){Current_C_A[k][l]/=alpha2;Current_C_B[k][l]/=alpha2;}	
			
		}
	}
	for(int k=0;k<nb_dim;k++)
	{
		Current_M_AB[k]=Current_M_A[k];
		Current_M_AB[k+nb_dim]=Current_M_B[k];
	}
	for(int k=0;k<nb_dim*2;k++){
		for (int l=0;l<nb_dim*2;l++){
		
Current_C_AB[k][l]=*(Timeless_MeanCov.position+(nb_dim*nb_images+1)*nb_dim*nb_images*(kMT+1)+(k+1+(Index_CurrentImage-1)*nb_dim)*nb_dim*nb_images+l+(Index_CurrentImage-1)*nb_dim);
			if (k==0||k==3){Current_C_AB[k][l]/=alpha0}
			if (k==1||k==4){Current_C_AB[k][l]/=alpha1}
			if (k==2||k==5){Current_C_AB[k][l]/=alpha2}
			if (l==0||l==3){Current_C_AB[k][l]/=alpha0}	
			if (l==1||l==4){Current_C_AB[k][l]/=alpha1}
			if (l==2||l==5){Current_C_AB[k][l]/=alpha2}	
		}
	}

}
/*****************************************************************************************************/
void GraphDynaCluster::Load_MonoCurrentPMT(int kMT)
{
	for(int k=0;k<nb_dim;k++)
	{
		Current_M_A[k]=(*(Timeless_MeanCov.position+(nb_dim*nb_images+1)*nb_dim*nb_images*(kMT+1)+Index_CurrentImage*nb_dim+k));
		if (k==0){Current_M_A[k]/=alpha0}
		if (k==1){Current_M_A[k]/=alpha1}
		if (k==2){Current_M_A[k]/=alpha2}
		for (int l=0;l<nb_dim;l++)
		{
						Current_C_A[k][l]=(*(Timeless_MeanCov.position+(nb_dim*nb_images+1)*nb_dim*nb_images*(kMT+1)+(k+1+Index_CurrentImage*nb_dim)*nb_dim*nb_images+l+Index_CurrentImage*nb_dim));
			if (k==0){Current_C_A[k][l]/=alpha0}
			if (k==1){Current_C_A[k][l]/=alpha1}
			if (k==2){Current_C_A[k][l]/=alpha2}
			if (l==0){Current_C_A[k][l]/=alpha0}    
			if (l==1){Current_C_A[k][l]/=alpha1}    
			if (l==2){Current_C_A[k][l]/=alpha2}    
			
		}
	}

}
/*****************************************************************************************************/	
void GraphDynaCluster::LoadpreTLTL(int prekTL,int kTL){

	unsigned long int dim_MeanCov=Temp_MeanCov.nx*Temp_MeanCov.ny;
	for(int k=0;k<nb_dim;k++)
	{
		Current_M_A[k]=(*(Temp_MeanCov.position+dim_MeanCov*(Index_CurrentImage-1)+(nb_dim+1)*nb_dim*(prekTL+1)+k));
		Current_M_B[k]=(*(Temp_MeanCov.position+dim_MeanCov*Index_CurrentImage+(nb_dim+1)*nb_dim*(kTL+1)+k));
		if (k==0){Current_M_A[k]/=alpha0;Current_M_B[k]/=alpha0;}
		if (k==1){Current_M_A[k]/=alpha1;Current_M_B[k]/=alpha1;}
		if (k==2){Current_M_A[k]/=alpha2;Current_M_B[k]/=alpha2;}
		for (int l=0;l<nb_dim;l++)
		{
			Current_C_A[k][l]=(*(Temp_MeanCov.position+dim_MeanCov*(Index_CurrentImage-1)+(nb_dim+1)*nb_dim*(prekTL+1)+(k+1)*nb_dim+l));
			Current_C_B[k][l]=(*(Temp_MeanCov.position+dim_MeanCov*Index_CurrentImage+(nb_dim+1)*nb_dim*(kTL+1)+(k+1)*nb_dim+l));
			if (k==0){Current_C_A[k][l]/=alpha0;Current_C_B[k][l]/=alpha0;}
			if (k==1){Current_C_A[k][l]/=alpha1;Current_C_B[k][l]/=alpha1;}
			if (k==2){Current_C_A[k][l]/=alpha2;Current_C_B[k][l]/=alpha2;}
			if (l==0){Current_C_A[k][l]/=alpha0;Current_C_B[k][l]/=alpha0;}	
			if (l==1){Current_C_A[k][l]/=alpha1;Current_C_B[k][l]/=alpha1;}
			if (l==2){Current_C_A[k][l]/=alpha2;Current_C_B[k][l]/=alpha2;}	
		}
	}

}

/*****************************************************************************************************/	
void  GraphDynaCluster::MIEstimation()
{	
	
	for (int kMT=0;kMT<nb_TimelessClasses;kMT++){
		for (int t=1;t<nb_images;t++){  
			Index_CurrentImage=t;
			Load_CurrentPMT(kMT);
			TmpClass[kMT].MTWeight[t-1][0]=1;
			TmpClass[kMT].MTWeight[t][0]=1;
			TmpClass[kMT].JointWeight[t-1][0]=1;
			for (int j=0;j<nb_dim;j++){
				TmpClass[kMT].MTCentroids[t-1][0][j]=Current_M_A[j];
				TmpClass[kMT].MTCentroids[t][0][j]=Current_M_B[j];
				TmpClass[kMT].JointCentroids[t-1][0][j]=Current_M_AB[j];
				TmpClass[kMT].JointCentroids[t-1][0][j+nb_dim]=Current_M_AB[j+nb_dim];
				for (int l=0;l<nb_dim;l++){
					TmpClass[kMT].MTCovariances[t-1][0][j][l]=Current_C_A[j][l];
					TmpClass[kMT].MTCovariances[t][0][j][l]=Current_C_B[j][l];
				}
			}
			for (int j=0;j<nb_dim*2;j++){
				for (int l=0;l<nb_dim*2;l++){
					TmpClass[kMT].JointCovariances[t-1][0][j][l]=Current_C_AB[j][l];
				}
			}
			
		}
	}

cout<<"Calculating the mutual Information for each MT Class between each consecutive times"<<endl;

	
	for (int t=1;t<nb_images;t++){  
	    mi.Allocate(nb_dim);
	    for (int kMT=0;kMT<nb_TimelessClasses;kMT++){	
	        //cout<<"t "<<t-1<<t<<" , kMT "<<kMT<<endl;	    
		vector<float**> *C1=&TmpClass[kMT].MTCovariances[t-1];
		vector<float*> *M1=&TmpClass[kMT].MTCentroids[t-1];	
		vector<float**> *C2=&TmpClass[kMT].MTCovariances[t];
		vector<float*> *M2=&TmpClass[kMT].MTCentroids[t];		
		vector<float**> *C12=&TmpClass[kMT].JointCovariances[t-1];
		vector<float*> *M12=&TmpClass[kMT].JointCentroids[t-1];
		mi.Initialise1(C1,M1);
		mi.Initialise2(C2,M2);
		mi.Initialise12(C12,M12);
	    	TmpClass[kMT].MutualInfo[t-1]=mi.MICalculus();
		
	    }
	    
	}
cout<<"Mutual Information end"<<endl;
	
}
/*****************************************************************************************************/	



