/*!@file graphCharac.hh 
*@brief File containing the GraphDynaCluster object used for ceating the graph of dynamic cluster trajectories*/
#ifndef TREE_BUIDLER_H
#define TREE_BUIDLER_H
#include "./Image_basics.hh"
#include "./DynaClass.hh"
#include "./SuperposeClass.hh"
#include "./divergence.h"
#include "./MI.hh"	

/*!@brief If the feature ranges are very different, this factor is used to  normalize the first band*/
#define alpha0 1; 
/*!@brief If the feature ranges are very different, this factor is used to  normalize the second band*/
#define alpha1 1; 
/*!@brief If the feature ranges are very different, this factor is used to  normalize the third band*/
#define alpha2 1; 

/*!@brief Object to ceate the graph of dynamic cluster trajectories

@todo redesign this object and the GraphManipulator object used for the supervised learning on graphs so that they share attributes and methods from a commun herited parent object*/
class GraphDynaCluster
{
  
  public:

  // Operations
  
     //GraphDynaCluster public Methods
    	GraphDynaCluster();
	virtual ~GraphDynaCluster();
	/*!@brief Initialisation of the graph object : Allocate memory and read the input files*/
	void Initialise(char * path_in_Temporal, char * path_in_Timeless, int dimx_tmp, int dimy_tmp, int nb_bands);
	/*!@brief Method to infer the graph trajectories.
	*
	*This method calculates for each MT cluster, the probabilities of association with the TL clusters at each time sample. Thus, for each time sample,  collections of time-localized clusters, sorted according the assocation probabilities, are attached to each MT cluster.\n
	The calculus of the assocations probabilities are derived using a Divergence object. The probabilities and the collection of TL clusters are stored in the attributes of the GraphDynaCluster object.*/
	void Build(float PrecisionFactor);
	/*!@brief Method to characterize the graph with mutual information.
	*
	* Calculates the mutual information between two consecutive times for each MT cluster by an analytical calculation based on the covariance determinants\n
	The calculus of the mutual information measurements are derived using an object named MI*/
	void MIEstimation();
	/*!@brief Method to characterize  the graph and write the graph output.
	*
	* The characterization of the other graph's edge attributes (flow and the change of distributions of TL clusters) is done while writing the graph output on the disk.
	*@param format a value of 1 is for the standard graph output, a value of 2 or 3 are for formats compatible with the actual learning programm and GUI (graphe used by the GUI with (2) or without (3) edges).
	@param intensiveCalculus This variable is true if the the change of distributions of TL clusters is quantified with Kullback-Leibler divergence, and false if a simple mean difference is used.*/
	void Out_Evolutions(int format, bool intensiveCalculus);
	

	
  // Attributes
  	/*!@brief  Number of MT clusters or Classes*/
  	 int nb_TimelessClasses;
	 
  private:

  // Attributes
  	/*!@brief  number of dimension of the feature space*/
	int nb_dim;
	/*!@brief number of  images time sample*/
	int nb_images;
	/*!@brief number of samples in each image*/
	unsigned long int dim;
	/*!@brief size of a image (in pixels)*/
	int nx,ny;
	//@brief index of processed current image
	int Index_CurrentImage;
	

	
	
	/*!@brief path of the TL classfiles */
	char ***PathInTemp_Classfile;
	/*!@brief path of the TL  attributes*/
	char ***PathInTemp_MeanCov;
	/*!@brief path of the MT classfile*/
	char ***PathInTimeless_Classfile;
	/*!@brief path of the MT attribute file*/
	char ***PathInTimeless_MeanCov;
	/*!@brief TL Classfile images */
	Image<unsigned char> Temp_classfile;
	/*!@brief TL  Attribute variables*/
	Image<float> Temp_MeanCov;
	/*!@brief MT Classfile image */
	Image<unsigned char> Timeless_classfile;
	/*!@brief MT  Attribute variable*/
	Image<float> Timeless_MeanCov;
	
	//Auxiliary variables 
	/*!@brief Current covariance matrice considered for divergence (or mutual information) calculation    */
  	float **Current_C_A;
	/*!@brief Current covariance matrice considered for divergence (or mutual information) calculation  */
	float **Current_C_B;
	/*!@brief Current mean vector considered for divergence (or mutual information) calculation  */
	float *Current_M_A;
	/*!@brief Current mean vector considered for divergence (or mutual information) calculation   */
	float *Current_M_B;
	/*!@brief Current joint MT covariance matrice considered for mutual information calculation  */
	float **Current_C_AB;
	/*!@brief Current joint MT mean vector considered for mutual information calculation  */
	float *Current_M_AB;
	/*!@brief Resolution of the Gaussian-Approximated divergence Integration   */
	float PrecisionFactor;
	
	/*!@brief DynamicCluster : an array of DynamicCluster objects is used to manage the MT cluster trajectories*/
	DynaClass *TmpClass;
	/*!@brief Object for classfiles intersection calculations*/
	SuperposeClass *supClass;
	/*!@brief Object for mutual infroamtion calculation*/	
	MI mi;

  // Operations
      //GraphDynaCluster private methods
      	/*!@brief Memory alocation method */ 
  	void Allocate_variables();
	/*!@brief Method to read the input output file names */ 
	void Acquire_path(char * path_in_Temporal, int nb_bands, char* path_in_Timeless);
	/*!@brief Method to read the input data */ 
	void Aquire_data();
	/*!@brief Method to load  a TL and a MT cluster Gaussian attributes for divergence  calculation
	*
	@param j1 index of the TL cluster 
	@param j2 index of the MT cluster*/ 
	void Load_Current(int j1,int j2);
	/*!@brief Method to load  2 consecutive  TL  cluster Gaussian attributes for divergence calculation 
	@param prekTL index of the TL cluster at t-1 
	@param kTL index of the TL cluster at t*/ 
	void LoadpreTLTL(int prekTL,int kTL);
	/*!@brief Method to load  MT  cluster Gaussian attributes for mutual information calculation. The time sample is given by the private attribute "Index_CurrentImage" */ 
	void Load_CurrentPMT(int kMT);
	/*!@brief Method to load  a single MT  cluster Gaussian attribute at a given time.The time sample is given by the private attribute "Index_CurrentImage" */ 
	void Load_MonoCurrentPMT(int kMT);
	/*!@brief method to characterize and write the graph file in standard format (format 1)\n
	*
	*This format is the right one to be adopted in future : the learning and visualization algorithms using the graph must be adpated. However, for the moment, this format is useless\n
	*output file name "./grapheV1"\n
	*@param intensiveCalculus Characterize the edge by calculating the TL cluster distrubution change in consecutive times using Kullback Leibler divergence (true) or with a simple mean difference (false) */ 
	void Create_TxtGraphOutPutVersion1(bool intensiveCalculus);
	/*!@brief method to characterize and write the graph file (format 2)\n output file name "./grapheV2"\n
	*
	*@param intensiveCalculus Characterize the edge by calculating the TL cluster distrubution change in consecutive times using Kullback Leibler divergence (true) or with a simple mean difference (false) */ 
	void Create_TxtGraphOutPutVersion2(bool intensiveCalculus);
	/*!@brief Method to read a graphe at the format 2 and create a graphe at at format adapted to the JAVA GUI.\n output file name"./grapheV2JAVA"\n
	*@param path input graph format 2 to read 
	*@param edge set this variable to "0" or "1" for writing a graph format for the GUI, which displayd ("1") or not ("0")  the edges of graph
	*/
	void ReadWriteGraphVersion2JAVA(char * path, int edge);
	
	
};
#endif
