/*---------------------------------------------------------------------------*/
/*!
 * @file /TreeNode.hh
 * @brief  File containing the TreeNode object.
 
  @author P.Has (IRIT / ENSAE / DLR / CNES)
 * @since January 2005
 *
 */
/*---------------------------------------------------------------------------*/
#ifndef TREENODE_H
#define TREENODE_H
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cassert>

/**********************************************************/
/*******************TreeNode class*************************/
/**********************************************************/
/*! @brief node Object used for the constitution of a SearchTree Object.*/ 
class TreeNode
{
 public: 
 	/*! @brief hight level in the search tree*/ 
	int timeLevel;
	/*! @brief node index of the candidate sub-graph (in the matching procedure) */
 	int g2VertexIndex;
	 /*! @brief Class MT Index associated to the node of the candidate sub-graph (in the matching procedure)*/
	int g2MTIndex;
	/*! @brief node index of the reference sub-graph (in the matching procedure) */
 	int g1VertexIndex;
	/*! @brief Class MT Index associated to the node of the reference sub-graph (in the matching procedure)*/
	int g1MTIndex;
	/*! @brief Node index in the previous search tree level which is linked to this node*/ 
	int branch;
	/*! @brief global node cost*/ 
	float nodeCost;
	/*! @brief partial node cost : contribution of the  population of each node. */
	float weightMTinTLDiffCost;
	/*! @brief partial node cost :  contribution of the nodes' distribution. */
	float TL1TL2DivCost; 
	/*! @brief partial node cost :  contribution of the flow of pixels exchanged between nodes.*/
	float flowDiffCost;
	/*! @brief partial node cost :  contribution of the change in time of the nodes' distribution. */
	float divDiffCost;
	/*! @brief partial node cost :  contribution of the intra-node change in time. */
	float MIDiffCost;
	/*! @brief partial node cost :  contribution of the interaction of the 2 MT classes (only if 2 MT class are considered). */ 
	float divMTaMTbCost;
	/*! @brief partial node cost :  contribution of the time lattice on which the sub-graph is defined. */ 
	float timeCost;
	

	TreeNode(){
		
		timeLevel=-1;
		g2VertexIndex=-1;
		g2MTIndex=-1;
		g1VertexIndex=-1;
		g1MTIndex=-1;
		branch=-1;
		nodeCost=-1;
	
	}

	virtual~TreeNode(){
		
	}
	
};
#endif

