/*!@file divergence.cpp 
*@brief File containing the Divergence object source */
#include <iostream>
#include <string>  
#include <vector>
#include <fstream>

#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cassert>


#include "divergence.h"
using namespace std;

Divergence::Divergence (int nb_dimensions)
{
	
	nb_dim=nb_dimensions;
	MeanA=new float[nb_dim];
	MeanB=new float[nb_dim];
	CovA=new float*[nb_dim];
	CovB=new float*[nb_dim];
	InvCovA=new float*[nb_dim];
	InvCovB=new float*[nb_dim];
	a=new float[nb_dim];
	b=new float[nb_dim];
	for (int j=0;j<nb_dim;j++)
	{
		CovA[j]=new float[nb_dim];
		CovB[j]=new float[nb_dim];
		InvCovA[j]=new float[nb_dim];
		InvCovB[j]=new float[nb_dim];
	}
}
Divergence::~Divergence()

{
    //cout<<"desctruction of Divergence object"<<endl;
	/*delete [] MeanA; delete [] MeanB;
	for (int j=0;j<nb_dim;j++)
	{
		delete [] CovA[j]; delete [] CovB[j];
		delete [] InvCovA[j]; delete [] InvCovB[j];
	}
	delete [] CovA; delete [] InvCovA; 
	delete [] CovB; delete [] InvCovB; */
}
void Divergence::Initialise ( float** C_A, float** C_B, float *M_A, float* M_B )
{
	int j1,j2;
	nb_Gdim=nb_dim;
	float aux;
	float il=IntLimit;
	for (j1=0;j1<nb_dim;j1++)
	{
		MeanA[j1]=M_A[j1];
		MeanB[j1]=M_B[j1];
		for (j2=0;j2<nb_dim;j2++)
		{
			CovA[j1][j2]=C_A[j1][j2];
			CovB[j1][j2]=C_B[j1][j2];
		}
	
	}
	
/*
**  Putting the zero components as the last dimsensions in the covariance matrix
*/
	for (j1=0;j1<nb_dim;j1++){
		if (CovA[j1][j1]<il||CovB[j1][j1]<il){
		    	nb_Gdim--;
		}
	}
	bool again=true;
	while(again){
	   for (j1=0;j1<nb_dim;j1++)
	   {
		if (CovA[j1][j1]<il||CovB[j1][j1]<il){
		    if (j1<nb_dim-1){
			for (j2=0;j2<nb_dim;j2++){
				aux=CovA[j1][j2];
			  	CovA[j1][j2]=CovA[j1+1][j2];
				CovA[j1+1][j2]=aux;
				aux=CovB[j1][j2];
			  	CovB[j1][j2]=CovB[j1+1][j2];
				CovB[j1+1][j2]=aux;
				
			}
			for (j2=0;j2<nb_dim;j2++){	
				aux=CovA[j2][j1];
			  	CovA[j2][j1]=CovA[j2][j1+1];
				CovA[j2][j1+1]=aux;
				aux=CovB[j2][j1];
			  	CovB[j2][j1]=CovB[j2][j1+1];
				CovB[j2][j1+1]=aux;
			}
		    }
		}
	   }
	   for (j1=0;j1<nb_Gdim;j1++){
		if (CovA[j1][j1]<il||CovB[j1][j1]<il){again=true;}
		else {again=false;}
	   }
	   if (nb_Gdim==0){again=false;}
	}
	
}
void Divergence::InvertCovMatrix()
{
	int j1,j2,j,l;
	float **auxCov=new float*[nb_Gdim+1];
	for (j=0;j<nb_Gdim+1;j++)
	{
		auxCov[j]=new float[nb_Gdim+1];
	}
	float *col=new float[nb_Gdim+1];
	float d;
	int *indx=new int[nb_Gdim+1]; 
	int n=nb_Gdim;// number of non-zero dim in cov matrix 
	void ludcmp(float **a,int n,int *indx,float *d);
	void lubksb(float **a,int n,int *indx, float b[]);
	for (j1=0;j1<nb_Gdim;j1++)
	{
		for (j2=0;j2<nb_Gdim;j2++){auxCov[j1+1][j2+1]=CovA[j1][j2];}
	
	}
	ludcmp(auxCov,n,indx,&d);//INVERSE of COV MATRIX A
	for (j=1;j<nb_Gdim+1;j++){
	        for (l=1;l<nb_Gdim+1;l++){col[l]=0.0;}
	        col[j]=1.0;
	        lubksb(auxCov,n,indx,col);
	        for (l=1;l<nb_Gdim+1;l++) 
	        {
	        	InvCovA[l-1][j-1]=col[l];
	        }		
	}
	for (j=1;j<nb_Gdim+1;j++){
	        for (l=1;l<nb_Gdim+1;l++){auxCov[j][l]=InvCovA[j-1][l-1];}}
	ludcmp(auxCov,n,indx,&d);//DETERMINANT OF THE INVERSE COV MATRIX A
	for (j=1;j<nb_Gdim+1;j++){d=auxCov[j][j]*d;}
	determinantA=d;
	for (j1=0;j1<nb_Gdim;j1++)
	{
		for (j2=0;j2<nb_Gdim;j2++){auxCov[j1+1][j2+1]=CovB[j1][j2];}
	
	}
	ludcmp(auxCov,n,indx,&d);//INVERSE of COV MATRIX B
	for (j=1;j<nb_Gdim+1;j++){
	        for (l=1;l<nb_Gdim+1;l++){col[l]=0.0;}
	        col[j]=1.0;
	        lubksb(auxCov,n,indx,col);
	        for (l=1;l<nb_Gdim+1;l++) 
	        {
	        	InvCovB[l-1][j-1]=col[l];
	        }		
	}
	for (j=1;j<nb_Gdim+1;j++){
	        for (l=1;l<nb_Gdim+1;l++){auxCov[j][l]=InvCovB[j-1][l-1];}}
	
	ludcmp(auxCov,n,indx,&d);//DETERMINANT OF THE INVERSE COV MATRIX B
	for (j=1;j<nb_Gdim+1;j++){d=auxCov[j][j]*d;}
	determinantB=d;
	
	for (j=0;j<nb_Gdim+1;j++){delete [] auxCov[j];}
	delete [] auxCov;delete [] col;delete [] indx;
	
}

float Divergence::Div_MultiDimFunc(float x,float y, float z)//extended for different nb_dim
{
	int j , j1 , j2;
	float res=0,res1=0,res2=0,res3=1;
	float *xx=new float[nb_dim];
	xx[0]=x;xx[1]=y;xx[2]=z;//cout<<xx[0]<<"	"<<xx[1]<<"	"<<xx[2]<<endl;
	float *auxligne = new float[nb_Gdim];
	
	for (j=0;j<nb_Gdim;j++){auxligne[j]=0;}//first term
	for (j1=0;j1<nb_Gdim;j1++)
	{
		for (j2=0;j2<nb_Gdim;j2++)
		{
			auxligne[j1]=(xx[j2]-MeanB[j2])*(InvCovB[j1][j2])+auxligne[j1];
		}
	}
	for (j=0;j<nb_Gdim;j++){res1=auxligne[j]*(xx[j]-MeanB[j])+res1;}
	
	for (j=0;j<nb_Gdim;j++){auxligne[j]=0;}//second term
	for (j1=0;j1<nb_Gdim;j1++)
	{
		for (j2=0;j2<nb_Gdim;j2++)
		{
			auxligne[j1]=(xx[j2]-MeanA[j2])*(InvCovA[j1][j2])+auxligne[j1];
		}
	}
	for (j=0;j<nb_Gdim;j++){res2=auxligne[j]*(xx[j]-MeanA[j])+res2;}

	res3=sqrt(determinantA)/(pow(2.0*M_PI,(double)(nb_Gdim/2.)))*exp(-0.5*res2);
	
	res=(0.5*(log(determinantA/determinantB)+(res1-res2)))*res3;//all terms
	
	delete [] xx;
	delete [] auxligne;
 	return res;
}

float Divergence::Qgaus(float (Divergence::*func)(float),float a, float b)
{
	int j;
	float xr,xm,dx,s;
	static float x[]={0.0,0.1488743389,0.4333953941,
		0.6794095682,0.8650633666,0.9739065285};
	static float w[]={0.0,0.2955242247,0.2692667193,
		0.2190863625,0.1494513491,0.0666713443};

	xm=0.5*(b+a);
	xr=0.5*(b-a);
	s=0;
	for (j=1;j<=5;j++) {
		dx=xr*x[j];
		s += w[j]*((*this.*func)(xm+dx)+(*this.*func)(xm-dx));
	}
	return s *= xr;


}

float Divergence::Integration()		//defined only for 3D integration
{
	float res=0;
	res=Qgaus(&Divergence::f1,a[0],b[0]);
	return res;
}

/***************auxiliary recursive functions for integration*******************/
float  Divergence::f1(float x)
	{
		xsav=x;	
		return Qgaus(&Divergence::f2,a[1],b[1]);
	}
float  Divergence::f2(float y)
	{
		ysav=y;
		return Qgaus(&Divergence::f3,a[2],b[2]);
	}
float  Divergence::f3(float z)
	{
		return Div_MultiDimFunc(xsav,ysav,z);
	}
/*****************************************************************************************************/
float Divergence::MeanDifference()
{
	float res=0;
	for (int i=0;i<nb_dim;i++){res+=(MeanA[i]-MeanB[i])*(MeanA[i]-MeanB[i]);}
	return sqrt(res);
}
/*****************************************************************************************************/
int Divergence::BoundResolution_Calculation(float *min_bound,float *max_bound,float *resolution,float PrecisionFactor)
{
	
	int indexMax;
	int indexMin;	
	int res=0;
	float il=IntLimit;
	//bound and resolution of integration calculation
	for (int j=0;j<nb_dim;j++)
	{
		if (MeanB[j]+sqrt(CovB[j][j])*3.>MeanA[j]+sqrt(CovA[j][j])*3.){	
			max_bound[j]=MeanB[j]+sqrt(CovB[j][j])*3.;
			indexMax=2;	
		}
		else{
			max_bound[j]=MeanA[j]+sqrt(CovA[j][j])*3.;
			indexMax=1;
		}
		if (MeanB[j]-sqrt(CovB[j][j])*3.<MeanA[j]-sqrt(CovA[j][j])*3.){	
			min_bound[j]=MeanB[j]-sqrt(CovB[j][j])*3.;
			indexMin=2;
		}
		else{
			min_bound[j]=MeanA[j]-sqrt(CovA[j][j])*3.;
			indexMin=1;
		}
		if (indexMax!=indexMin){// checking if the PDF are not too far from each other to avoid an useless calculation
	   		if (indexMax==2){
			   if ((MeanB[j]-sqrt(CovB[j][j])*3.)>(MeanA[j]+sqrt(CovA[j][j])*3.)){res++;}  
	   		}
			if (indexMax==1){
			   if ((MeanA[j]-sqrt(CovA[j][j])*3.)>(MeanB[j]+sqrt(CovB[j][j])*3.)){res++;}  
	   		}
	   	}
		resolution[j]=-1;
		if (CovB[j][j]<il||CovA[j][j]<il){ // non integration for Diracs
			min_bound[j]=0;
			max_bound[j]=1;
			resolution[j]=0;
		}
	}	
	for (int j=0;j<nb_dim;j++)
	{
	   if (resolution[j]!=0){
		if(CovA[j][j]<CovB[j][j])
			resolution[j]=sqrt(CovA[j][j])*PrecisionFactor;
		
		else
			resolution[j]=sqrt(CovB[j][j])*PrecisionFactor;
		
		if (resolution[j]<=sqrt(il)*PrecisionFactor)
			resolution[j]=0;
			
		if ((float)(max_bound[j]-min_bound[j])/resolution[j]>3.){resolution[j]=(float)(max_bound[j]-min_bound[j])/3.;}
		if ((float)(max_bound[j]-min_bound[j])/resolution[j]<1.){resolution[j]=(float)(max_bound[j]-min_bound[j])/1.;}
		
	   }
	}
	//if (res>=1){cout<<"avoided useless calculation : "<<res<<endl;for (int j=0;j<nb_dim;j++){cout<<CovA[j][j]<<"    "<<CovB[j][j]<<endl;}}
	return res;
	
}

/*******************************************************************************/

void Divergence::BoundCalculus(float *pas, float *resolution)
{
	int j;
	for (j=0;j<nb_dim;j++)
	{
		a[j]=pas[j];
		b[j]=pas[j]+resolution[j];
		if (resolution[j]==0){b[j]=a[j]+1;}//For diracs Integration on a unit in order to be = to 1
	}

}
/*******************************************************************************/

float Divergence::Divergence2GaussModel(float *min_bound,float *max_bound,float *resolution, float weight)
{
	//only 3D : to be extend!!!
	double res=0;
	float *pas=new float[nb_dim];
	float *resolutionaux=new float[nb_dim];
	float il=IntLimit;
	int maxIteration=0;
	resolutionaux[0]=resolution[0];
	resolutionaux[1]=resolution[1];
	resolutionaux[2]=resolution[2];
	float j1=min_bound[0],j2=min_bound[1],j3=min_bound[2];
	while (j1<max_bound[0])
	{   		
		if (j1+resolutionaux[0]>max_bound[0]){resolutionaux[0]=max_bound[0]-j1;}
		pas[0]=j1;
		while (j2<max_bound[1])
		{			
			pas[1]=j2;
			if (j2+resolutionaux[1]>max_bound[1]){resolutionaux[1]=max_bound[1]-j2;}
			while (j3<max_bound[2])
			{
				pas[2]=j3;
				if (j3+resolutionaux[2]>max_bound[2]){resolutionaux[2]=max_bound[2]-j3;}
				BoundCalculus(pas,resolutionaux);
				res=res+Integration();
				maxIteration++;
				j3=j3+resolution[2];
				if (resolutionaux[2]==0){break;}
				//if (maxIteration>5000){break;}
			}
			resolutionaux[2]=resolution[2];
			j2=j2+resolution[1];
			j3=min_bound[2];
			if (resolutionaux[1]==0){break;}
			//if (maxIteration>5000){break;}
		}
		resolutionaux[1]=resolution[1];
		j1=j1+resolution[0];
		j2=min_bound[1];
		if (resolutionaux[0]==0){break;}
		//if (maxIteration>5000){cout<<"iteration>10000 : Exiting div calculation loop..."<<endl;break;}
	}
	// adding the divergence of the assumed independent components with the diracs
	if ((nb_dim-nb_Gdim)>0){
		for (int j=0;j<nb_dim;j++){
			//cout<<resolution[j]<<endl;
			if (resolution[j]==0){
			    //cout<<res<<endl;
			    if (CovA[j][j]<il&&CovB[j][j]>=il){
				res+=-log(1./(sqrt(CovB[j][j]*2.*M_PI))*exp(-(MeanA[j]-MeanB[j])*(MeanA[j]-MeanB[j])/(2.*CovB[j][j])));
			    }
			    if (CovB[j][j]<il&&CovA[j][j]>=il){
				res+=-log(1./(sqrt(CovA[j][j]*2.*M_PI))*exp(-(MeanB[j]-MeanA[j])*(MeanB[j]-MeanA[j])/(2.*CovA[j][j])));
			    }
			    if (CovB[j][j]<il&&CovA[j][j]<il){
			    	if (sqrt((MeanB[j]-MeanA[j])*(MeanB[j]-MeanA[j]))>4.*sqrt(il)){res=INF;}
				//else {cout<<"Identical singularity on band "<<j<<endl;}
			    }
			    //cout<<MeanA[j]-MeanB[j]<<"  "<<res<<endl;
			}
		}
	}
	res=(float)((double)res/(double)weight);
	
	
	delete [] pas;
	delete [] resolutionaux;
	return res;
}
	
