/*---------------------------------------------------------------------------*/
/*!
 * @file fillsubgraph+.cpp
 * @brief  file containing methods of the GraphManipulator object.
 
  @author P.Has (IRIT / ENSAE / DLR / CNES)
 * @since January 2005
 *
 */
/*---------------------------------------------------------------------------*/
#include <iostream>
#include <string>  
#include <vector>
#include <fstream>

#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cassert>

#include "graphCharac+.hh"


using namespace std;

/*****************************************************************************************************/
GraphManipulator::GraphManipulator()
{
	noTree=true;
	noTreeNeg=true;
}
/*****************************************************************************************************/
GraphManipulator::~GraphManipulator()
{
	//cout<<"desctruction du GraphManipulator"<<endl;
}


/*****************************************************************************************************/	
void GraphManipulator::InitGraphforMatch(int nb_samples,int firstSample1,float seuilDiv,int kMT1a,int kMT1b){
	
	  seuilDivergence=seuilDiv;
	  nbSamples=nb_samples;	  
	  float *t_delay=new float[nb_samples-1];
	  for (int i=0;i<nb_samples-1;i++){t_delay[i]=TimeDelay[i+1+firstSample1];}
    if (graph1.size()>0){graph1.erase(graph1.begin());}
    if (graph2.size()>0){graph2.erase(graph2.begin());}
    SubGraph tmp1(nb_dim,nb_samples,t_delay);
    SubGraph tmp2(nb_dim,nb_samples,t_delay);
    graph1.push_back(tmp1);
    graph2.push_back(tmp2);
	  delete [] t_delay;
	  AddMTtoSubGraph(1,kMT1a,seuilDiv,firstSample1,nb_samples);
	  AddMTtoSubGraph(2,kMT1a,seuilDiv,firstSample1,nb_samples);
	  if (kMT1b!=-1){
	  	AddMTtoSubGraph(1,kMT1b,seuilDiv,firstSample1,nb_samples);
		AddMTtoSubGraph(2,kMT1b,seuilDiv,firstSample1,nb_samples);
	  }
	
}
/*****************************************************************************************************/	
void GraphManipulator::InitGraphforMatchNeg(int nb_samples,float seuilDiv, int b){

	  seuilDivergence=seuilDiv;
	  nbSamples=nb_samples;	  
	  float *t_delay=new float[nb_samples-1];
	  int endList=kMT2aSorted.size()-1;
	  int firstSample1;int kMT1a;int kMT1b;
	  kMT1a=kMT2aSorted[endList];
	  if(b!=-1){kMT1b=kMT2bSorted[endList];}
	  else{kMT1b=-1;}
	  firstSample1=firstSample2Sorted[endList];
	  for (int i=0;i<nb_samples-1;i++){t_delay[i]=TimeDelay[i+1+firstSample1];}
      if (graph1.size()>0){graph1.erase(graph1.begin());}
      if (graph2.size()>0){graph2.erase(graph2.begin());}
	  //graph1=new SubGraph[1](nb_dim,nb_samples,t_delay);
	  //graph2=new SubGraph[1](nb_dim,nb_samples,t_delay);
      SubGraph tmp1(nb_dim,nb_samples,t_delay);
      SubGraph tmp2(nb_dim,nb_samples,t_delay);
      graph1.push_back(tmp1);
      graph2.push_back(tmp2);
      delete [] t_delay;
	  AddMTtoSubGraph(1,kMT1a,seuilDiv,firstSample1,nb_samples);
	  AddMTtoSubGraph(2,kMT1a,seuilDiv,firstSample1,nb_samples);
	  if (kMT1b!=-1){
	  	AddMTtoSubGraph(1,kMT1b,seuilDiv,firstSample1,nb_samples);
		AddMTtoSubGraph(2,kMT1b,seuilDiv,firstSample1,nb_samples);
	  }
	  
}
/*****************************************************************************************************/	
void GraphManipulator::FixWeights(float weightMTinTLDiff,float TL1TL2Div,float flowDiff,float divDiff,float MIDiff,float divMTaMTbCost,float timeDiff,bool meanCalcul){
	 	  	  
	  //TL1TL2Div/=10.;divDiff/=10.;
	  matchCostObject[0].FixCostFuncWeigths(weightMTinTLDiff,TL1TL2Div,flowDiff,divDiff,MIDiff,divMTaMTbCost,timeDiff,meanCalcul);
	  predefinedList=false;
	  factorReducList=1.;

}
/*****************************************************************************************************/	
void GraphManipulator::FixLearnedWeights(int posOrNeg, bool meanCalcul,float factorReduc){
	 	
	  float weightMTinTLDiff;float TL1TL2Div;float flowDiff;float divDiff;float MIDiff;float divMTaMTbCost;float timeDiff; 	  
	  if (posOrNeg==1){
        weightMTinTLDiff=paraSearchObject.estimPara[0];
	  	TL1TL2Div=paraSearchObject.estimPara[1];
	  	flowDiff=paraSearchObject.estimPara[2];
	  	divDiff=paraSearchObject.estimPara[3];
	  	MIDiff=paraSearchObject.estimPara[4];
	 	divMTaMTbCost=paraSearchObject.estimPara[5];
	 	timeDiff=paraSearchObject.estimPara[6];
	   }
	   if (posOrNeg==-1){
	  	weightMTinTLDiff=paraSearchObjectNeg.estimPara[0];
	  	TL1TL2Div=paraSearchObjectNeg.estimPara[1];
	  	flowDiff=paraSearchObjectNeg.estimPara[2];
	  	divDiff=paraSearchObjectNeg.estimPara[3];
	  	MIDiff=paraSearchObjectNeg.estimPara[4];
	 	divMTaMTbCost=paraSearchObjectNeg.estimPara[5];
	 	timeDiff=paraSearchObjectNeg.estimPara[6];
	   }
	  matchCostObject[0].FixCostFuncWeigths(weightMTinTLDiff,TL1TL2Div,flowDiff,divDiff,MIDiff,divMTaMTbCost,timeDiff,meanCalcul);
	  predefinedList=true;
	  factorReducList=factorReduc;

}
/*****************************************************************************************************/	
void GraphManipulator::CreateMatchCostObject(){
    
    MatchCost tmp1(&(graph1[0]),&(graph2[0]));
    matchCostObject.push_back(tmp1);
    //matchCostObject=new MatchCost[1](graph1,graph2);
	  	  
}
/*****************************************************************************************************/	
float GraphManipulator::Match(int posOrNeg, int min_Sample, int max_Sample,int nbMiniBranches){
	  
	
	float cost,miniCost=Inf;
	bool searchEnd=false;bool calculate=true;
	int nb_samples=graph1[0].nbTimeSamp;
	float *t_delay=new float[nb_samples-1];
	for (int i=0;i<nb_samples-1;i++){t_delay[i]=1;}
	minSample=min_Sample; maxSample=max_Sample;
	maxSample+=1;
	int kMT2a=0;//current graph index used for mathcing
	int kMT2b=1;//current other graph index used for mathcing (only if 2 MT considered simultaneously)
	int firstSample2=minSample;//current graph first time smaple used for mathcing
	unsigned long int auxIndex;
	
	int firstSample2Min=-1;
	int kMT2aMin=-1;
	int kMT2bMin=-1;
	
	if (posOrNeg==1){//positive semantic 
	   if (!noTree){// if a search tree is already formed, it is desctruct (memory desallocation)
	   	for (int i=0;i<nb_TimelessClasses;i++){
		   for (int j=0;j<nb_TimelessClasses;j++){
		      for (int k=0;k<maxSample;k++){
		         delete [] searchTreeCollec[i][j][k];
			 delete [] costs[i][j][k];
		      }
		      delete []  searchTreeCollec[i][j];
		      delete [] costs[i][j];
		   }
		   delete []  searchTreeCollec[i];
		   delete [] costs[i];
		}
		delete [] searchTreeCollec;
		delete [] costs;
		
		costSorted.clear();
		kMT2aSorted.clear();
		kMT2bSorted.clear();
		firstSample2Sorted.clear();
	   }
	
	   // Memory allocation for the search tree
	   searchTreeCollec= new SearchTree***[nb_TimelessClasses];
	   costs=new float***[nb_TimelessClasses];
	   for (int i=0;i<nb_TimelessClasses;i++){
		searchTreeCollec[i]=new SearchTree**[nb_TimelessClasses];
		costs[i]=new float**[nb_TimelessClasses];
		for (int j=0;j<nb_TimelessClasses;j++){
			searchTreeCollec[i][j]=new SearchTree*[maxSample];
			costs[i][j]=new float*[maxSample];
			for (int k=0;k<maxSample;k++){
				costs[i][j][k]=new float[8];
				searchTreeCollec[i][j][k]=new SearchTree[1];
				for (int l=0;l<8;l++){costs[i][j][k][l]=-1;}
			}
		}
	   }
	   noTree=false;
	}
	
	if (posOrNeg==-1){//negative semantic 
	   predefinedList=true;//as an negative example is always following a positive example, there is always a collection of sub-graphs (with a size maybe reduced by the factor "factorReducList") when the program reaches this point
	   if (!noTreeNeg){// if a search tree is already formed, it is desctruct (memory desallocation)
	   	for (int i=0;i<nb_TimelessClasses;i++){
		   for (int j=0;j<nb_TimelessClasses;j++){
		      for (int k=0;k<maxSample;k++){
		         delete [] searchTreeCollecNeg[i][j][k];
			 delete [] costsNeg[i][j][k];
		      }
		      delete []  searchTreeCollecNeg[i][j];
		      delete [] costsNeg[i][j];
		   }
		   delete []  searchTreeCollecNeg[i];
		   delete [] costsNeg[i];
		}
		delete [] searchTreeCollecNeg;
		delete [] costsNeg;
		//(no negative semantic list used)
	   }
	
	   searchTreeCollecNeg= new SearchTree***[nb_TimelessClasses];
	   costsNeg=new float***[nb_TimelessClasses];
	   for (int i=0;i<nb_TimelessClasses;i++){
		searchTreeCollecNeg[i]=new SearchTree**[nb_TimelessClasses];
		costsNeg[i]=new float**[nb_TimelessClasses];
		for (int j=0;j<nb_TimelessClasses;j++){
			searchTreeCollecNeg[i][j]=new SearchTree*[maxSample];
			costsNeg[i][j]=new float*[maxSample];
			for (int k=0;k<maxSample;k++){
				costsNeg[i][j][k]=new float[8];
				searchTreeCollecNeg[i][j][k]=new SearchTree[1];
				for (int l=0;l<8;l++){costsNeg[i][j][k][l]=-1;}
			}
		}
	   }
	   noTreeNeg=false;
	}
	
	while (!searchEnd){ //matching of a each element of a collection of sub-graphs with the reference graph
		
		for (int i=0;i<nb_samples-1;i++){t_delay[i]=TimeDelay[i+1+firstSample2];}
		//delete [] graph2;
        graph2.erase(graph2.begin());
		//graph2=new SubGraph[1](nb_dim,nb_samples,t_delay);
		SubGraph tmp2(nb_dim,nb_samples,t_delay);
        graph2.push_back(tmp2);
        //initializing the mathcing procedure with the current sub-graph candidate 
	    AddMTtoSubGraph(2,kMT2a,seuilDivergence,firstSample2,nb_samples);
        if (graph1[0].nbMTClusters==2){AddMTtoSubGraph(2,kMT2b,seuilDivergence,firstSample2,nb_samples);}
	    else{kMT2b=kMT2a;}
	    matchCostObject[0].UpdateGraph2(&graph2[0]);
	    calculate=true;
	    // case when first matching procedure and 2 MT graph considered simultaneously
	    //testing if the current sub-graph belongs to  the most likely graph MT associations
	    if (graph1[0].nbMTClusters==2&&predefinedList==false){
	       calculate=false;factorReducList=4.;
	       for (int i=0;i<sizeList;i++){
	          //selecting the corresponding element in the cost list
	          if(kMT2aSortedAux[i]==kMT2a&&firstSample2aSortedAux[i]==firstSample2){
	              if (kMT2aCostAux[i]<Inf&& i<=(int)((float)sizeList/factorReducList)){
	        	  for (int j=0;j<sizeList;j++){
	        	     if(kMT2bSortedAux[j]==kMT2b&&firstSample2bSortedAux[j]==firstSample2){
	        		  if (kMT2bCostAux[j]<Inf&& j<=(int)((float)sizeList/factorReducList)){calculate=true;}
	        	     }
	        	  }
	              }
	          }
	       }
	       for (int i=0;i<sizeList;i++){
	          if(kMT2aSortedAux[i]==kMT2b&&firstSample2aSortedAux[i]==firstSample2){
	              if (kMT2aCostAux[i]<Inf&& i<=(int)((float)sizeList/factorReducList)){
	        	  for (int j=0;j<sizeList;j++){
	        	     if(kMT2bSortedAux[j]==kMT2a&&firstSample2bSortedAux[j]==firstSample2){
	        		  if (kMT2bCostAux[j]<Inf&& j<=(int)((float)sizeList/factorReducList)){calculate=true;}
	        	     }
	        	  }
	              }
	          }
	       } 
	       factorReducList=1.;    
	    } 
	    //testing if the current sub-graph belongs to the previously learned list of subgraphs
	    if (predefinedList){
	    	calculate=false;
	    	for (int i=0;i<sizeList;i++){
			if(kMT2aSortedAux[i]==kMT2a&&kMT2bSortedAux[i]==kMT2b&&firstSample2aSortedAux[i]==firstSample2){
				if (kMT2aCostAux[i]<Inf && i<=(int)((float)sizeList/factorReducList)){calculate=true;}
				if (i==(sizeList-1)){calculate=true;}
			}
		}
	    }
	    
	    //excluding same MT classes when 2 MT present
	    if (graph1[0].nbMTClusters==2&&kMT2a==kMT2b){calculate=false;}
	    
	    //matching with the current sub-graph candidate 
	    if (calculate){
	       if (posOrNeg==1){	cost=matchCostObject[0].InexactMatchCost(nbMiniBranches,searchTreeCollec[kMT2a][kMT2b][firstSample2],costs[kMT2a][kMT2b][firstSample2]);
	       }
	       if (posOrNeg==-1){
	          cost=matchCostObject[0].InexactMatchCost(nbMiniBranches,searchTreeCollecNeg[kMT2a][kMT2b][firstSample2],costsNeg[kMT2a][kMT2b][firstSample2]);
		
	       }
	    }
	    else{cost=Inf;}
	    	    
	    //testing if current sub-graph candidate is the best match (NOT USEFULL!!)
		if (cost<miniCost){
			kMT2aMin=kMT2a;
			kMT2bMin=kMT2b;
			firstSample2Min=firstSample2;
			miniCost=cost;
		}
	    //including the current sub-graph candidate in the sorted collection. This step is done only  when no a posteriori list is derived that is to say only for the first matching procedure with the first positive graph example. The created collection is used only to make a preselection of MT associations when 2 MT class are considered simultaneously.  It is not any more usefull afterwords. Thus this step SHOULD BE MODIFIED!!! 
		if (calculate){		  
		  if (posOrNeg==1){
		     int *auxkMT2aSorted=new int[1];
		     int *auxkMT2bSorted=new int[1];
		     int *auxfirstSample2Sorted=new int[1];
		     float *auxcostSorted=new float[1];		
		     auxIndex=kMT2aSorted.size();
		     for (unsigned long int k=0;k<kMT2aSorted.size();k++){
			  if (cost<costSorted[k]){
			   auxIndex=k;
			   break;
	  		  }
      		     }
		     auxkMT2aSorted[0]=kMT2a;auxkMT2bSorted[0]=kMT2b;auxfirstSample2Sorted[0]=firstSample2;auxcostSorted[0]=cost;
		     if (kMT2aSorted.size()!=0){
			vector<int>::iterator theIterator1 = kMT2aSorted.begin()+auxIndex;
			kMT2aSorted.insert(theIterator1,auxkMT2aSorted[0]);
			vector<int>::iterator theIterator2 = kMT2bSorted.begin()+auxIndex;
			kMT2bSorted.insert(theIterator2,auxkMT2bSorted[0]);
			vector<int>::iterator theIterator3 = firstSample2Sorted.begin()+auxIndex;
			firstSample2Sorted.insert(theIterator3,auxfirstSample2Sorted[0]);
			vector<float>::iterator theIterator4 = costSorted.begin()+auxIndex;
			costSorted.insert(theIterator4,auxcostSorted[0]);
		     }
		     else{
			kMT2aSorted.push_back(auxkMT2aSorted[0]);
			kMT2bSorted.push_back(auxkMT2bSorted[0]);
			firstSample2Sorted.push_back(auxfirstSample2Sorted[0]);
			costSorted.push_back(auxcostSorted[0]);
		     }
		  }
		}
		
	        //moving to the next subgraph candidate
		firstSample2++;
		if (kMT2a==(nb_TimelessClasses-1)&&(firstSample2+nb_samples)>(maxSample)){
			searchEnd=true;
		}
		else{
			if ((firstSample2+nb_samples)>(maxSample)){
				firstSample2=minSample;
				kMT2b++;
				if (graph1[0].nbMTClusters!=2){kMT2b=(nb_TimelessClasses-1);}
				if (kMT2b==(nb_TimelessClasses-1)){kMT2b=kMT2a+1;kMT2a++;}
			}
		}
		
			
	}
	
	
 	delete [] t_delay;
	factorReducList=1;
	return cost;
}
/*****************************************************************************************************/	
void GraphManipulator::SaveSortedList(int a,int b){
	
	if (a==1&&b==0){//1st MT class in subgraph
	   sizeList=(int)kMT2aSorted.size();
	   kMT2aSortedAux=new int[kMT2aSorted.size()];
	   firstSample2aSortedAux=new int[kMT2aSorted.size()];
	   kMT2aCostAux=new float[kMT2aSorted.size()];
	   for (unsigned long int l=0;l<kMT2aSorted.size();l++){
		kMT2aSortedAux[l]=kMT2aSorted[l];
		firstSample2aSortedAux[l]=firstSample2Sorted[l];
		kMT2aCostAux[l]=costSorted[l];
	   }
	}
	if (a==0&&b==1){//2nd MT class in subgraph
	   sizeList=(int)kMT2aSorted.size();
	   kMT2bSortedAux=new int[kMT2aSorted.size()];
	   firstSample2bSortedAux=new int[kMT2aSorted.size()];
	   kMT2bCostAux=new float[kMT2aSorted.size()];
	   for (unsigned long int l=0;l<kMT2aSorted.size();l++){
	   	kMT2bSortedAux[l]=kMT2aSorted[l];
		firstSample2bSortedAux[l]=firstSample2Sorted[l];
		kMT2bCostAux[l]=costSorted[l];
	   }
	}
	if (a==1&&b==1){
 	   sizeList=(int)kMT2aSorted.size();
	   kMT2aSortedAux=new int[kMT2aSorted.size()];
	   firstSample2aSortedAux=new int[kMT2aSorted.size()];
	   kMT2aCostAux=new float[kMT2aSorted.size()];
	   kMT2bSortedAux=new int[kMT2bSorted.size()];
	   firstSample2bSortedAux=new int[kMT2bSorted.size()];
	   kMT2bCostAux=new float[kMT2bSorted.size()];
	   for (unsigned long int l=0;l<kMT2aSorted.size();l++){
		kMT2aSortedAux[l]=kMT2aSorted[l];
		kMT2bSortedAux[l]=kMT2bSorted[l];
		firstSample2aSortedAux[l]=firstSample2Sorted[l];
		kMT2aCostAux[l]=costSorted[l];
	   }
	   
	}
	//delete [] graph1;
	//delete [] graph2; 
	//delete [] matchCostObject;
}
/*****************************************************************************************************/	
void GraphManipulator::InitParaLearn(int a, int b){
	
	int nbLevel=100;
	
	//positive semantique
	if (a==1){
		paraSearchObject.Allocate(7,nbLevel,nb_TimelessClasses,graph1[0].nbMTClusters,minSample,maxSample,graph1[0].nbTimeSamp);
		paraSearchObject.Init(searchTreeCollec,costs,&kMT2aSorted,&kMT2bSorted,&firstSample2Sorted);
		paraSearchObject.MaxCostalues();
	}
	//negative semantique
	if (b==1){		paraSearchObjectNeg.Allocate(7,nbLevel,nb_TimelessClasses,graph1[0].nbMTClusters,minSample,maxSample,graph1[0].nbTimeSamp);
		paraSearchObjectNeg.Init(searchTreeCollecNeg,costsNeg,&kMT2aSorted,&kMT2bSorted,&firstSample2Sorted);
		paraSearchObjectNeg.MaxCostalues();
	}

	divergences=new float[7];
	for (int i=0;i<7;i++){divergences[i]=0;}
	sizeList=(int)kMT2aSorted.size();
	
}

/*****************************************************************************************************/	
void GraphManipulator::ScaleParaLearnForSG(int a, int b,bool scale){
	
	
	
	//positive semantique
	if (a==1){
		paraSearchObject.Init(searchTreeCollec,costs,&kMT2aSorted,&kMT2bSorted,&firstSample2Sorted);
		if (scale) {paraSearchObject.MaxCostalues();}
	}
	//negative semantique
	if (b==1){		
		paraSearchObjectNeg.Init(searchTreeCollecNeg,costsNeg,&kMT2aSorted,&kMT2bSorted,&firstSample2Sorted);
		if (scale){paraSearchObjectNeg.MaxCostalues();}
	}
	sizeList=(int)kMT2aSorted.size();
	
}

/*****************************************************************************************************/	
void GraphManipulator::LearnRefGraph(int posOrNeg){
	
	if (posOrNeg==-1){
		int indexEnd=kMT2aSorted.size()-1;
		Learn(-1,kMT2aSorted[indexEnd],kMT2bSorted[indexEnd],firstSample2Sorted[indexEnd]);
				//cout<<"- "<<kMT2aSorted[indexEnd]<<" "<<kMT2bSorted[indexEnd]<<" "<<firstSample2Sorted[indexEnd]<<endl;
	}
	if (posOrNeg==1){
		Learn(1,kMT2aSorted[0],kMT2bSorted[0],firstSample2Sorted[0]);
        //cout<<"+ "<<kMT2aSorted[0]<<" "<<kMT2bSorted[0]<<" "<<firstSample2Sorted[0]<<endl;
	}
}
/*****************************************************************************************************/	
float GraphManipulator::Learn(int posOrNeg,int interestkMT2a,int interestkMT2b,int interestfirstSample2){
	  
	//new estimation paramters
	if (posOrNeg>0){paraSearchObject.MMSEestim(interestkMT2a,interestkMT2b,interestfirstSample2);}
	if (posOrNeg<0){paraSearchObjectNeg.MMSEestim(interestkMT2a,interestkMT2b,interestfirstSample2);}
	
	// creation of the sorted cost list after reevaluation of each lowest costs 
	kMT2aSorted.clear();
	kMT2bSorted.clear();
	firstSample2Sorted.clear();
	costSorted.clear();
	float cost,miniCost=Inf;
	
	float negProb=0;float posProb=0;
	int firstSample2Min=-1;
	int kMT2aMin=-1;
	int kMT2bMin=-1;
	unsigned long int auxIndex;
	
	
	// checking that there already exists an estimation of the parameter vector 
	float priorParaPos=0;
	for (int kPara=0;kPara<paraSearchObject.nbPara;kPara++){priorParaPos+=paraSearchObject.estimPara[kPara];}
	float priorParaNeg=0;
	for (int kPara=0;kPara<paraSearchObjectNeg.nbPara;kPara++){priorParaNeg+=paraSearchObjectNeg.estimPara[kPara];}
	//calculating constants for normalization
	
	
	float maxPosCost=-1;
	float maxNegCost=-1;
	
	vector<float> costList;
	vector<float> costList2;
	
	//update of the similarity  with the new para vector and max of similiraty function (calculating the range on which the similarity function takes its values)
	for (int i=0;i<nb_TimelessClasses;i++){
	   for (int j=0;j<nb_TimelessClasses;j++){		   
	       for (int k=0;k<maxSample;k++){
	   	    if (!paraSearchObject.treeCollec[i][j][k][0].emptyTree||!paraSearchObjectNeg.treeCollec[i][j][k][0].emptyTree){ 
		      if (priorParaPos!=0){
		    	 float *auxCostList=new float[1];
			 auxCostList[0]=paraSearchObject.ReevaluateMatch(i,j,k);//update of the similarity  with the new para vector
		    	 //max value (excluding non matched sub-graphs with a cost value > 1e5) 
			 if (auxCostList[0]>maxPosCost&&auxCostList[0]<1e5){maxPosCost=auxCostList[0]*1.001;}
			 costList.push_back(auxCostList[0]);
			 
		      }
		      if (priorParaNeg!=0){
			 float *auxCostList2=new float[1];
			 auxCostList2[0]=paraSearchObjectNeg.ReevaluateMatch(i,j,k);//update of the similarity  with the new para vector   
		         //max value (excluding non matched sub-graphs with a cost value > 1e5) 
			 if (auxCostList2[0]>maxNegCost&&auxCostList2[0]<1e5){maxNegCost=auxCostList2[0]*1.001;}
			 costList2.push_back(auxCostList2[0]);
		      }
		    }
	       }
	   }
	}
	//sum of for normalization and conversion of costs into probabilities
	float sumCostPos=0;float sumCostNeg=0;
	int costListIndex=0;int costListIndex2=0;
	for (int i=0;i<nb_TimelessClasses;i++){
	   for (int j=0;j<nb_TimelessClasses;j++){		   
	       for (int k=0;k<maxSample;k++){
	   	    if (!paraSearchObject.treeCollec[i][j][k][0].emptyTree||!paraSearchObjectNeg.treeCollec[i][j][k][0].emptyTree){    	
			 	
			if (priorParaPos!=0){
				//operator similarity2proba
				posProb=(float)(maxPosCost-costList[costListIndex])/(float)maxPosCost;
				if (posProb<0){posProb=0;}
				sumCostPos+=posProb;
				costListIndex++;
			}
			
			if (priorParaNeg!=0){
				//operator similarity2proba
				negProb=(float)(maxNegCost-costList2[costListIndex2])/(float)maxNegCost;
				if (negProb<0){negProb=0;}
				sumCostNeg+=negProb;
				costListIndex2++;
			}
		    }
	       }
	   }
	}
	
	costListIndex=0;
	costListIndex2=0;
	//reevaluating graph posterior probabilities
	for (int i=0;i<nb_TimelessClasses;i++){
	   for (int j=0;j<nb_TimelessClasses;j++){		   
	       for (int k=0;k<maxSample;k++){
	   	    if (!paraSearchObject.treeCollec[i][j][k][0].emptyTree||!paraSearchObjectNeg.treeCollec[i][j][k][0].emptyTree){    	
			
			//POSITIVE SEMANTIC GRAPH LIKELYHOOD		    	
			if (priorParaPos!=0){
				//operator similarity2proba
				posProb=(float)(maxPosCost-costList[costListIndex])/(float)maxPosCost;
				if (posProb<0){posProb=0;}
				posProb/=(float)sumCostPos;
				costListIndex++;
			}
			else{posProb=1./(float)sizeList;}
			
			//NEGATIVE SEMANTIC GRAPH LIKELYHOOD	
			if (priorParaNeg!=0){
				//operator similarity2proba
				negProb=(float)(maxNegCost-costList2[costListIndex2])/(float)maxNegCost;
				if (negProb<0){negProb=0;}
				negProb/=(float)sumCostNeg;
				costListIndex2++;
			}
			else{negProb=1./(float)sizeList;}

			//GRAPH PROBA A POSTERIORI 
	   		cost=posProb/(float)(negProb+posProb);
	   		
			
			//finding the best match
	   		if (cost<miniCost){
	   		     	kMT2aMin=i;
	   		     	kMT2bMin=j;
	   		     	firstSample2Min=k;
	   		     	miniCost=cost;
	   		}
	   		//creating the sorted collection
	   		int *auxkMT2aSorted=new int[1];
	   		int *auxkMT2bSorted=new int[1];
	   		int *auxfirstSample2Sorted=new int[1];
	   		float *auxcostSorted=new float[1];
	   		auxIndex=kMT2aSorted.size();
	   		for (unsigned long int l=0;l<kMT2aSorted.size();l++){
	   		     	if (cost>costSorted[l]){
	   		     	 auxIndex=l;
	   		     	 break;
	   		     	}
	   		}
	   		auxkMT2aSorted[0]=i;auxkMT2bSorted[0]=j;auxfirstSample2Sorted[0]=k;auxcostSorted[0]=cost;
			if (kMT2aSorted.size()!=0){
				vector<int>::iterator theIterator1 = kMT2aSorted.begin()+auxIndex;
				kMT2aSorted.insert(theIterator1,auxkMT2aSorted[0]);
				vector<int>::iterator theIterator2 = kMT2bSorted.begin()+auxIndex;
				kMT2bSorted.insert(theIterator2,auxkMT2bSorted[0]);
				vector<int>::iterator theIterator3 = firstSample2Sorted.begin()+auxIndex;
				firstSample2Sorted.insert(theIterator3,auxfirstSample2Sorted[0]);
				vector<float>::iterator theIterator4 = costSorted.begin()+auxIndex;
				costSorted.insert(theIterator4,auxcostSorted[0]);
			}
			else{
				kMT2aSorted.push_back(auxkMT2aSorted[0]);
				kMT2bSorted.push_back(auxkMT2bSorted[0]);
				firstSample2Sorted.push_back(auxfirstSample2Sorted[0]);
				costSorted.push_back(auxcostSorted[0]);
			}
	   	    }
	       }
	   }
	}
	StrengthProbLink();
	VariancePosterior();
	ConvergenceQuality();
	paraSearchObject.Init(&kMT2aSorted,&kMT2bSorted,&firstSample2Sorted);
	paraSearchObjectNeg.Init(&kMT2aSorted,&kMT2bSorted,&firstSample2Sorted);
	
	costList2.clear();
	costList.clear();
	
	return miniCost;
	
}
/*****************************************************************************************************/	
void GraphManipulator::EraseMemLearn(int posNeg){
	
	if (posNeg==1||posNeg==2){paraSearchObject.ErasePdf();}
	if (posNeg==-1||posNeg==2){paraSearchObjectNeg.ErasePdf();}
}
/*****************************************************************************************************/	
void GraphManipulator::PriorMemLearn(float weightMTinTLDiff,float TL1TL2Div,float flowDiff,float divDiff,float MIDiff,float divMTaMTbCost,float timeDiff){
    
    int aux=(paraSearchObject.nbQuantifLevel-1);
        paraSearchObject.pdf[0][(int)((float)(1-weightMTinTLDiff)*aux)]++;
        paraSearchObject.pdf[1][(int)((float)(1-TL1TL2Div)*aux)]++;
        paraSearchObject.pdf[2][(int)((float)(1-flowDiff)*aux)]++;
        paraSearchObject.pdf[3][(int)((float)(1-divDiff)*aux)]++;
        paraSearchObject.pdf[4][(int)((float)(1-MIDiff)*aux)]++;
        paraSearchObject.pdf[5][(int)((float)(1-divMTaMTbCost)*aux)]++;
        paraSearchObject.pdf[6][(int)((float)(1-timeDiff)*aux)]++;
    
}
/*****************************************************************************************************/	
void GraphManipulator::CopyPos2NegMemLearn(){
    
    for (int j=0;j<paraSearchObject.nbQuantifLevel;j++){for (int i=0;i<7;i++){paraSearchObjectNeg.pdf[i][j]=paraSearchObject.pdf[i][j];}}

 }


/*****************************************************************************************************/	
void GraphManipulator::PrintAPostList(){

	
	for (int l=(int)(kMT2aSorted.size()-1);l>=0;l--){
		cout<<"graph posterior probability : "<< costSorted[l]<<" for MT class : "
		<<kMT2aSorted[l]<<"&"<<kMT2bSorted[l]<<" and  first sample : " <<firstSample2Sorted[l]<<endl;
	}
				
}

/*****************************************************************************************************/	
void GraphManipulator::WriteAPostList(){

	char *path=new char[128];
	FILE *ifp;
	strcpy(path,"./APostList");
	ifp=NULL; 
     	assert (ifp=fopen(path,"w"));

	if (paraSearchObject.estimPara[0]!=0||paraSearchObjectNeg.estimPara[0]!=0){fprintf(ifp,"%f %s",paraSearchObject.estimPara[0]+paraSearchObjectNeg.estimPara[0]," ");}
	else{fprintf(ifp,"%f %s",-1.," ");}
	if (paraSearchObject.estimPara[1]!=0||paraSearchObjectNeg.estimPara[1]!=0){fprintf(ifp,"%f %s",paraSearchObject.estimPara[1]+paraSearchObjectNeg.estimPara[1]," ");}
	else{fprintf(ifp,"%f %s",-1.," ");}
	if (paraSearchObject.estimPara[2]!=0||paraSearchObjectNeg.estimPara[2]!=0){fprintf(ifp,"%f %s",paraSearchObject.estimPara[2]+paraSearchObjectNeg.estimPara[2]," ");}
	else{fprintf(ifp,"%f %s",-1.," ");}
	if (paraSearchObject.estimPara[3]!=0||paraSearchObjectNeg.estimPara[3]!=0){fprintf(ifp,"%f %s",paraSearchObject.estimPara[3]+paraSearchObjectNeg.estimPara[3]," ");}
	else{fprintf(ifp,"%f %s",-1.," ");}
	if (paraSearchObject.estimPara[4]!=0||paraSearchObjectNeg.estimPara[4]!=0){fprintf(ifp,"%f %s",paraSearchObject.estimPara[4]+paraSearchObjectNeg.estimPara[4]," ");}
	else{fprintf(ifp,"%f %s",-1.," ");}
	if (paraSearchObject.estimPara[5]!=0||paraSearchObjectNeg.estimPara[5]!=0){fprintf(ifp,"%f %s",paraSearchObject.estimPara[5]+paraSearchObjectNeg.estimPara[5]," ");}
	else{fprintf(ifp,"%f %s",-1.," ");}
	if (paraSearchObject.estimPara[6]!=0||paraSearchObjectNeg.estimPara[6]!=0){fprintf(ifp,"%f\n",paraSearchObject.estimPara[6]+paraSearchObjectNeg.estimPara[6]);}
	else{fprintf(ifp,"%f\n",-1.);}
	VariancePosterior();
	fprintf(ifp,"%f %s %f\n",meanPosterior," ",sqrt(varPosterior));
	fprintf(ifp,"%f %s %f\n",convQualityPos," ",convQualityNeg);
	fprintf(ifp,"%d\n",nbSamples);
	for (int l=0;l<(int)kMT2aSorted.size();l++){
		fprintf(ifp,"%d %s %d %s %d %s %f\n",kMT2aSorted[l]," ",kMT2bSorted[l]," ",firstSample2Sorted[l]," ", costSorted[l]);
	}
	assert (EOF!=fclose(ifp)); 
			
}
	
/*****************************************************************************************************/	
void GraphManipulator::PrintProbaLink(){

	cout<<"\n PARAMETER MMSE FOR (+,-) SEMANTICS & DIVERGENCES"<<endl;
	cout<<"_____________________________"<<endl;
	
	if (paraSearchObject.estimPara[0]!=0||paraSearchObjectNeg.estimPara[0]!=0){cout<<"Cluster weights			: ("<<paraSearchObject.estimPara[0]<<", "<<paraSearchObjectNeg.estimPara[0]<<") & "<<divergences[0]<<endl;}
	else{cout<<"Cluster weights		 	: "<<"------"<<endl;}
	if (paraSearchObject.estimPara[1]!=0||paraSearchObjectNeg.estimPara[1]!=0){cout<<"Cluster Absolute Position	: ("<<paraSearchObject.estimPara[1]<<", "<<paraSearchObjectNeg.estimPara[1]<<") & "<<divergences[1]<<endl;}
	else{cout<<"Cluster Absolute Position	: "<<"------"<<endl;}
	if (paraSearchObject.estimPara[2]!=0||paraSearchObjectNeg.estimPara[2]!=0){cout<<"Cluster Flow			: ("<<paraSearchObject.estimPara[2]<<", "<<paraSearchObjectNeg.estimPara[2]<<") & "<<divergences[2]<<endl;}
	else{cout<<"Cluster Flow		 	: "<<"------"<<endl;}
	if (paraSearchObject.estimPara[3]!=0||paraSearchObjectNeg.estimPara[3]!=0){cout<<"Cluster change			: ("<<paraSearchObject.estimPara[3]<<", "<<paraSearchObjectNeg.estimPara[3]<<") & "<<divergences[3]<<endl;}
	else{cout<<"Cluster change		 	: "<<"------"<<endl;}
	if (paraSearchObject.estimPara[4]!=0||paraSearchObjectNeg.estimPara[4]!=0){cout<<"Cluster Mutual Information	: ("<<paraSearchObject.estimPara[4]<<", "<<paraSearchObjectNeg.estimPara[4]<<") & "<<divergences[4]<<endl;}
	else{cout<<"Cluster Mutual Information	: "<<"------"<<endl;}
	if (paraSearchObject.estimPara[5]!=0||paraSearchObjectNeg.estimPara[5]!=0){cout<<"MT Clusters Interaction		: ("<<paraSearchObject.estimPara[5]<<", "<<paraSearchObjectNeg.estimPara[5]<<") & "<<divergences[5]<<endl;}
	else{cout<<"MT Clusters Interaction		: "<<"------"<<endl;}
	if (paraSearchObject.estimPara[6]!=0||paraSearchObjectNeg.estimPara[6]!=0){cout<<"Time Delay			: "<<paraSearchObject.estimPara[6]<<", "<<paraSearchObjectNeg.estimPara[6]<<" - "<<divergences[6]<<endl;}
	else{cout<<"Time Delay		 	: "<<"------"<<endl;}
	cout<<"\nPOSTERIOR DISTRIBUTION		    "<<endl;
	cout<<"_____________________________"<<endl;
	cout<<"Posterior mean			: "<<meanPosterior<<endl;
	cout<<"Posterior Standard deviation	: "<<sqrt(varPosterior)<<endl;
	cout<<"\nCONVERGENCE 	    "<<endl;	
	cout<<"_____________________________"<<endl;
	cout<<"Norm parameter vector+ difference: "<<convQualityPos<<endl;
	cout<<"Norm parameter vector- difference: "<<convQualityNeg<<endl;
	cout<<""<<endl;		
}
	
/*****************************************************************************************************/	
void GraphManipulator::StrengthProbLink(){//problem quand debut apprentissage negatif
	
	double sumPos=0;double meanPos;double varPos;double hPos;
	double sumNeg=0;double meanNeg;double varNeg;double hNeg;
	double sumDiv=0;
	float p,q;
	float piCst=sqrt(2*M_PI);
	for (int kPara=0;kPara<paraSearchObject.nbPara;kPara++){
	
		//Gaussian Kernel parameters calculation
		sumPos=0;sumNeg=0;meanPos=0;meanNeg=0;varPos=0;varNeg=0;
        cout<<"\n"<<kPara<<endl;
		for (int i=0;i<paraSearchObject.nbQuantifLevel;i++){
            cout<<paraSearchObject.pdf[kPara][i]<<"-"<<paraSearchObjectNeg.pdf[kPara][i]<<" ";
			sumPos+=paraSearchObject.pdf[kPara][i];
			sumNeg+=paraSearchObjectNeg.pdf[kPara][i];
		}
		for (int i=0;i<paraSearchObject.nbQuantifLevel;i++){
			meanPos+=paraSearchObject.pdf[kPara][i]*i;
			meanNeg+=paraSearchObjectNeg.pdf[kPara][i]*i;
		}
		meanPos/=sumPos;
		meanNeg/=sumNeg;
		for (int i=0;i<paraSearchObject.nbQuantifLevel;i++){
			varPos+=(i-meanPos)*(i-meanPos)*paraSearchObject.pdf[kPara][i];
			varNeg+=(i-meanNeg)*(i-meanNeg)*paraSearchObjectNeg.pdf[kPara][i];
		}
		varPos/=sumPos;
		varNeg/=sumNeg;
		sumDiv=0;
         
		
		//Optimal Gaussian Kernel variance calculation
		hPos=1.059*sqrt(varPos)/pow(sumPos,0.2);if (hPos==0){hPos=1.059/pow(sumPos,0.2);}
		hNeg=1.059*sqrt(varNeg)/pow(sumNeg,0.2);if (hNeg==0){hNeg=1.059/pow(sumNeg,0.2);}
		
		
		float auxSUMPos=0;float auxSUMNeg=0;
		for (int i=0;i<paraSearchObject.nbQuantifLevel;i++){
			p=0;q=0;
			//gaussian kernel application
			for (int k=0;k<paraSearchObject.nbQuantifLevel;k++){
				if (paraSearchObject.pdf[kPara][k]>0){p+=exp(-(i-k)*(i-k)/(2.*hPos*hPos))*paraSearchObject.pdf[kPara][k]+1./(float)paraSearchObject.nbQuantifLevel;}
				if (paraSearchObjectNeg.pdf[kPara][k]>0){q+=exp(-(i-k)*(i-k)/(2.*hNeg*hNeg))*paraSearchObjectNeg.pdf[kPara][k]+1./(float)paraSearchObject.nbQuantifLevel;}
			}		
			p/=(piCst*hPos);p/=sumPos;
			q/=(piCst*hNeg);q/=sumNeg;
			if (sumPos==0){p=1./(float)paraSearchObject.nbQuantifLevel;}//uniform distribution
			if (sumNeg==0){q=1./(float)paraSearchObject.nbQuantifLevel;}//uniform distribution
			
			auxSUMPos+=p;
			auxSUMNeg+=q;

		}
		//Gaussian kernel application and divergence calculation
		for (int i=0;i<paraSearchObject.nbQuantifLevel;i++){
			p=0;q=0;
			//gaussian kernel application
			for (int k=0;k<paraSearchObject.nbQuantifLevel;k++){
				if (paraSearchObject.pdf[kPara][k]>0){p+=exp(-(i-k)*(i-k)/(2.*hPos*hPos))*paraSearchObject.pdf[kPara][k]+1./(float)paraSearchObject.nbQuantifLevel;;}
				if (paraSearchObjectNeg.pdf[kPara][k]>0){q+=exp(-(i-k)*(i-k)/(2.*hNeg*hNeg))*paraSearchObjectNeg.pdf[kPara][k]+1./(float)paraSearchObject.nbQuantifLevel;;}
			}		
			p/=(piCst*hPos);p/=sumPos;p/=auxSUMPos;
			q/=(piCst*hNeg);q/=sumNeg;q/=auxSUMNeg;
			if (sumPos==0){p=1./(float)paraSearchObject.nbQuantifLevel;}//uniform distribution
			if (sumNeg==0){q=1./(float)paraSearchObject.nbQuantifLevel;}//uniform distribution
			//divergence
			sumDiv+=(p-q)*log(p/q);	
		}
		divergences[kPara]=sumDiv;
        
		
	}
	if (graph1[0].nbMTClusters==1){divergences[5]=0;}
    if (nbSamples==(maxSample-minSample)){divergences[6]=0;}
}
	
/*****************************************************************************************************/	
void GraphManipulator::VariancePosterior(){
	
	meanPosterior=0;varPosterior=0;
	for (unsigned long int l=0;l<kMT2aSorted.size();l++){meanPosterior+=costSorted[l];}
	meanPosterior/=kMT2aSorted.size();
	for (unsigned long int l=0;l<kMT2aSorted.size();l++){varPosterior+=(costSorted[l]-meanPosterior)*(costSorted[l]-meanPosterior);} 
	varPosterior/=kMT2aSorted.size();
}
/*****************************************************************************************************/	
void GraphManipulator::ConvergenceQuality(){
	
	convQualityPos=0;
	convQualityNeg=0;
	float aux=0;
	for (int k=0;k<paraSearchObject.nbPara;k++){
		aux=(paraSearchObject.estimParaPrevious[k]-paraSearchObject.estimPara[k]);
		convQualityPos+=aux*aux;
		aux=(paraSearchObjectNeg.estimParaPrevious[k]-paraSearchObjectNeg.estimPara[k]);
		convQualityNeg+=aux*aux;
	}
	convQualityPos=sqrt(convQualityPos);
	convQualityNeg=sqrt(convQualityNeg);
	
}
/*****************************************************************************************************/	
void GraphManipulator::AddMTtoSubGraph(int subGraph,int kMT,float seuilDiv,int firstSample,int nb_samples){
	
	//Allocation of aux variables
	float **mean; float ***cov;unsigned long int weight;
	mean=new float*[nb_samples];cov=new float**[nb_samples];
	for (int i=0;i<nb_samples;i++){		
		mean[i]=new float[nb_dim];
		cov[i]=new float*[nb_dim];
		for (int j=0;j<nb_dim;j++){cov[i][j]=new float[nb_dim];}
	}
	int nbClusters;
	int auxIndex,auxIndex1,auxIndex2;
	float minDiv;
	int *IndexkTLminDiv=new int[nb_samples+firstSample];
	//Adding MT cluster attributes
	
	for (int t=firstSample;t<nb_samples+firstSample;t++){ 
		weight=TmpClass[kMT].MTWeight[t][0];
		Index_CurrentImage=t;
		for (int j=0;j<nb_dim;j++){
			mean[t-firstSample][j]=TmpClass[kMT].MTCentroids[t][0][j];
			for (int l=0;l<nb_dim;l++){
				cov[t-firstSample][j][l]=TmpClass[kMT].MTCovariances[t][0][j][l];
			}
		}
	} 
	if (subGraph==1)
		graph1[0].AddAttMT(mean,cov,weight);
	if (subGraph==2)
		graph2[0].AddAttMT(mean,cov,weight);
	
	for (int i=0;i<nb_samples;i++){		
		for (int j=0;j<nb_dim;j++){delete [] cov[i][j];}
		delete [] mean[i];delete [] cov[i];
	}
	delete [] mean;delete [] cov;
	
	//Adding TL related cluster attributes
	for (int t=firstSample;t<nb_samples+firstSample;t++){
		nbClusters=0;
		auxIndex=0;
		minDiv=Inf;
		IndexkTLminDiv[t]=-1;	
		int length=TmpClass[kMT].AssignedTimelessClass[t].size();
		for (int kTL=0;kTL<(int)length;kTL++){
			if ((float)TmpClass[kMT].DivergenceValue[t][kTL]<minDiv){
				minDiv=(float)TmpClass[kMT].DivergenceValue[t][kTL];
				IndexkTLminDiv[t]=kTL;
			}
		}
		for (int kTL=0;kTL<(int)length;kTL++){
			if ((float)TmpClass[kMT].DivergenceValue[t][kTL]*weight<seuilDiv||IndexkTLminDiv[t]==kTL){
				nbClusters++;
			}
		}
		//cout<<" t "<<t<<", nbClusters "<<nbClusters<<" (weight) "<<weight<<endl;
		float **meanTL=new float*[nbClusters];float ***covTL=new float**[nbClusters];
		float *divTL=new float[nbClusters];unsigned long int *weightTL=new unsigned long int[nbClusters];
		for (int i=0;i<nbClusters;i++){
			meanTL[i]=new float[nb_dim];covTL[i]=new float*[nb_dim];
			for (int j=0;j<nb_dim;j++)
				covTL[i][j]=new float[nb_dim];
		}
		
		for (int kTL=0;kTL<(int)length;kTL++){
		   if ((float)TmpClass[kMT].DivergenceValue[t][kTL]*weight<seuilDiv||IndexkTLminDiv[t]==kTL){
		      weightTL[auxIndex]=(unsigned long int)TmpClass[kMT].Weight[t][kTL];
		      divTL[auxIndex]=(float)TmpClass[kMT].DivergenceValue[t][kTL]*weightTL[auxIndex];
		      for (int i=0;i<nb_dim;i++){
		   	      meanTL[auxIndex][i]=TmpClass[0].Centroids[t][TmpClass[kMT].AssignedTimelessClass[t][kTL]][i];
		   	      for (int j=0;j<nb_dim;j++){
		       		  covTL[auxIndex][i][j]=TmpClass[0].Covariances[t][TmpClass[kMT].AssignedTimelessClass[t][kTL]][i][j];
		   	      }
		      }
		      auxIndex++;
		   }
			
		}
		if (subGraph==1)
			graph1[0].AddAttTLforMT(t-firstSample,nbClusters,meanTL,covTL,weightTL,divTL);
		if (subGraph==2)
			graph2[0].AddAttTLforMT(t-firstSample,nbClusters,meanTL,covTL,weightTL,divTL);
		
		for (int i=0;i<nbClusters;i++){for (int j=0;j<nb_dim;j++){delete [] covTL[i][j];}delete [] meanTL[i];delete [] covTL[i];}
		delete [] covTL;delete [] meanTL;delete [] weightTL;delete [] divTL;
	}
	
	//Adding related edge between TL clusters
	int length1;int length2;int nbClusters1;int nbClusters2;
	Divergence divObject(nb_dim);
	float *min_bound=new float[nb_dim];float *max_bound=new float[nb_dim];
	float *resolution=new float[nb_dim];
	
	for (int t=firstSample+1;t<nb_samples+firstSample;t++){
	  nbClusters1=0,nbClusters2=0;
	  length1=(int)TmpClass[kMT].AssignedTimelessClass[t-1].size();
	  length2=(int)TmpClass[kMT].AssignedTimelessClass[t].size();
	  for (int kTLpre=0;kTLpre<length1;kTLpre++)
	     if ((float)TmpClass[kMT].DivergenceValue[t-1][kTLpre]*weight<seuilDiv||IndexkTLminDiv[t-1]==kTLpre) 
	          nbClusters1++;
	  for (int kTL=0;kTL<length2;kTL++)
	     if ((float)TmpClass[kMT].DivergenceValue[t][kTL]*weight<seuilDiv||IndexkTLminDiv[t]==kTL)
	  	  nbClusters2++;       
	  unsigned long int **flow=new unsigned long int*[nbClusters1];
	  float **div=new float*[nbClusters1];
	  float **MI=new float*[nbClusters1];
	  for (int i=0;i<nbClusters1;i++){
	  	flow[i]=new unsigned long int[nbClusters2];
		div[i]=new float[nbClusters2];
		MI[i]=new float[nbClusters2];
	  }
	  
	  auxIndex2=0;
	  for (int kTL=0;kTL<length2;kTL++){
	     if ((float)TmpClass[kMT].DivergenceValue[t][kTL]*weight<seuilDiv||IndexkTLminDiv[t]==kTL){
	      	 auxIndex1=0;
	  	 for (int kTLpre=0;kTLpre<length1;kTLpre++){
		     if ((float)TmpClass[kMT].DivergenceValue[t-1][kTLpre]*weight<seuilDiv||IndexkTLminDiv[t-1]==kTLpre){
		     	 flow[auxIndex1][auxIndex2]=(unsigned long int)TmpClass[kMT].flow[t][kTLpre][kTL];	 
			 div[auxIndex1][auxIndex2]=TmpClass[kMT].div[t][kTLpre][kTL];
			 MI[auxIndex1][auxIndex2]=TmpClass[kMT].MutualInfo[t-1];//same MI for all edges at the same time belnging to the same MTk!!!
		         auxIndex1++;
		     }
	  	 } 
		 auxIndex2++;
	     }
	  }
	  if (subGraph==1)
			graph1[0].AddEdgesAttMT(t-firstSample,nbClusters1,nbClusters2,flow,div,MI);
	  if (subGraph==2)
			graph2[0].AddEdgesAttMT(t-firstSample,nbClusters1,nbClusters2,flow,div,MI);
				  
	  for (int i=0;i<nbClusters1;i++){delete [] flow[i];delete [] div[i];delete [] MI[i];}
	  delete [] flow;delete [] div;delete [] MI;
	}
	delete [] min_bound; delete [] max_bound;
	delete [] resolution;
	
				
}
/*****************************************************************************************************/	


