/*---------------------------------------------------------------------------*/
/*!
 * @file readholegraph.cpp
 * @brief  file containing a graph reading method of the GraphManipulator object.
 
  @author P.Has (IRIT / ENSAE / DLR / CNES)
 * @since January 2005
 *
 */
/*---------------------------------------------------------------------------*/
#include <iostream>
#include <string>  
#include <vector>
#include <fstream>

#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cassert>


#include "graphCharac+.hh"


using namespace std;
/*****************************************************************************************************/	
void GraphManipulator::ReadGraphVersion3(char * path,char *pathTime)
{
	
	FILE *ifp;
	ifp=NULL; 
     	assert (ifp=fopen(path,"r"));
	int auxRead;int length;float auxReadF;float weightMT;
	
	fscanf(ifp,"%d",&auxRead);
	nb_TimelessClasses=auxRead;//nb MT clusters
	fscanf(ifp,"%d",&auxRead);
	nb_dim=auxRead;//nb dimension FS
	fscanf(ifp,"%d",&auxRead);
	nb_images=auxRead;//nb images samples
	
	//Time delays between samples
	TimeDelay=new float[nb_images];
	FILE *ifpTime;
	ifpTime=NULL;
	assert (ifpTime=fopen(pathTime,"r"));
	for (int i=0;i<nb_images-1;i++){
		fscanf(ifpTime,"%d",&auxRead);
		TimeDelay[i+1]=auxRead;
	}
	
		
	//Dynamic classe objects allocation
	 TmpClass=new DynaClass[nb_TimelessClasses];
	for (int kMT=0;kMT<nb_TimelessClasses;kMT++){TmpClass[kMT].InitRead(nb_images,nb_dim);}
	
	//for each MT class
	for (int kMT=0;kMT<nb_TimelessClasses;kMT++){
	    fscanf(ifp,"%d",&auxRead);//MT index
	    fscanf(ifp,"%f",&weightMT);//MT weight   
	    //for each TL window
	    for(int t=0;t<nb_images;t++){
	        fscanf(ifp,"%d",&auxRead);//time sample
	        TmpClass[kMT].MTAlloc(t);
		TmpClass[kMT].MTWeight[t][0]=(unsigned long int)weightMT;
		//mean and cov kMT
	        for (int j=0;j<nb_dim;j++){fscanf(ifp,"%f",&auxReadF);TmpClass[kMT].MTCentroids[t][0][j]=auxReadF;}
		for (int j=0;j<nb_dim;j++){
			for (int l=0;l<nb_dim;l++){
				fscanf(ifp,"%f",&auxReadF);
				TmpClass[kMT].MTCovariances[t][0][j][l]=auxReadF;
			}
		} 
		fscanf(ifp,"%d",&auxRead);
	    	length=auxRead;//number of assigned  TLs
		if (kMT==0){TmpClass[kMT].TLAllocCenterCov(t,length);}
		TmpClass[kMT].TLAlloc2(t,length);
		if (t>0){TmpClass[kMT].flowDivAlloc(t,(int)TmpClass[kMT].AssignedTimelessClass[t-1].size(),length);}
		//for each TL_t class
	    	for (int kTL=0;kTL<(int)length;kTL++){ 
			fscanf(ifp,"%d",&auxRead);
			TmpClass[kMT].AssignedTimelessClass[t][kTL]=auxRead;//TL_t cluster index
			fscanf(ifp,"%f",&auxReadF);
			TmpClass[kMT].Weight[t][kTL]=(unsigned long int)auxReadF;//pixel quantity belonging to classes kTL_t & kMT
			fscanf(ifp,"%e",&auxReadF);
			TmpClass[kMT].DivergenceValue[t][kTL]=(float)auxReadF/(float)TmpClass[kMT].MTWeight[t][0];//divergence between classes kTL_t & kMT
			//mean and cov TL_t
			for (int i=0;i<nb_dim;i++){
				fscanf(ifp,"%f",&auxReadF);
				TmpClass[0].Centroids[t][TmpClass[kMT].AssignedTimelessClass[t][kTL]][i]=auxReadF;
			}
			for (int i=0;i<nb_dim;i++){
				for (int j=0;j<nb_dim;j++){
					fscanf(ifp,"%f",&auxReadF);
					TmpClass[0].Covariances[t][TmpClass[kMT].AssignedTimelessClass[t][kTL]][i][j]=auxReadF;
				}
			}
			if (t>0){
 			   //for each assigne TL_(t-1)class
			   for (int kTLpre=0;kTLpre<(int)TmpClass[kMT].AssignedTimelessClass[t-1].size();kTLpre++){
			        fscanf(ifp,"%d",&auxRead);//TL_(t-1) cluster index
				fscanf(ifp,"%f",&auxReadF);
				TmpClass[kMT].flow[t][kTLpre][kTL]=auxReadF;//pixel quantity belonging to classes kTL_t&kTL_(t-1)&kMT : flow
				fscanf(ifp,"%f",&auxReadF);
				TmpClass[kMT].div[t][kTLpre][kTL]=auxReadF;//div TL(t-1) TLt
				if (TmpClass[kMT].div[t][kTLpre][kTL]<0){TmpClass[kMT].div[t][kTLpre][kTL]=0;cout<<"neg div"<<endl;}
				if (TmpClass[kMT].div[t][kTLpre][kTL]>1e5){TmpClass[kMT].div[t][kTLpre][kTL]=1e10;}
			   }
			}
		}

		if (t>0){ fscanf(ifp,"%f",&auxReadF);TmpClass[kMT].MutualInfo[t-1]=auxReadF;}// Mutual information of MT class between t(t-1)
		
	    }
	}
	dim=0;
	for (int kMT=0;kMT<nb_TimelessClasses;kMT++){dim+=TmpClass[kMT].MTWeight[0][0];}
	
     	assert (EOF!=fclose(ifp)); 
	
		
}
