/*---------------------------------------------------------------------------*/
/*!
 * @file ITS_Graph3.java
 * @brief File containing the ITS_Graph3 object used  to create the different components of the Graphical User Interface.\n
  Note that this version is compatible with the graph at format 2 (complete graph with nodes and edges), and with images and classfiles of sizeImxsizeIm pixels. Because this format is not optimized, still requires a lot of memory. Therfore, should be used with a little data set.
 *
 *This visual interface was developped in order to create a first tool to naviguate through the generated image time-series representations, and therfore to validate the results.  As the JAVA programming was a completely new language for me and as not much time was available, the following developpement has been experimental, to avoid saying chaotic. \n
 Nevertheless, this tool enabes the visualisation and the synchronized naviguation through the image time-series representation, the graph of dynamic cluster trajectories and the dynamic classfiles. Futhermore, if the interactive learning C++ program is launch in parallel, the current semantic labelling and the relevance feedback measurements learned by the system can be visualized by this JAVA GUI (The communication between the C and the JAVA program is done using text files).    
 *
 *The frame contains
\li a) the image time-series sub-frame
\li b) the graph sub-frame 
\li c) a TL classfile sub-frame 
\li  d) a MT classfile sub-frame.
\li e) a sub-frame for relevance feedbacks visualisation
\li f)  a sub-frame to naviguate through the current sub-graph a posteriori collection
\li and g) a sub-frame for text values visualisation (sub-graph spatio-temporal location, sub-graph posterior  probabilities, etc...).

User can interact with  almost all these sub-frames :\n
*\n
-> In the sub-frame a) the user can naviguate in time through all the signal representations  by pressing buttons. By clicking on a images location, he can naviguate in space and visualize corresponding MT classes, TL classifications, and graph representations\n
-> In sub-frame b) the user can  visualize the dynamic cluster trajectories associated to the current MT classes (a maximum of 2 MT classes are visualisable at the same time in yellow and red). The mutual information is visualized in the same frame.  The user can update all the other representations by dfining a time coordinate by simple click in the sub-frame. The complexity of the graph can also be fixed interactively, by adjusting the divergence treshold. Note that, consequently the TL classes associated to the MT classes are updated.\n
-> In sub-frame c)  the user visualize the dynamic classifications. They are constituted of TL classes associted to the 2 different MT classes (commun TL classes are displayed in black, and in red and yellow for TL classes associted to only one of the 2 MT classes). Divergence treshold can be tuned to extend or concentrate these dynamic classification.\n
-> In sub-frame d)  the MT classes are visualized. The user can select the MT classes by clicking on the image intersting locations or by pressing buttons. A mximum of 2 MT classses are visualized in this frame : they are visualized in yellow and red in correspondance with the cluster trajectories and the dynamic classifications. \n
-> Sub-frame e)  is dedicated to visualize divergence histograms related to the current state of learning. No interaction exist in this frame \n
In sub-frame f), to naviguate through the current sub-graph a posteriori collection, press buttons are used to read an ./APostlist text file containing relevance feeback measurements and the sub-graphs a posteriori sotred collection. \n
-> Sub-frame g) is dedicated to the visualisation of text parameters. No interaction exist in this frame \n
*\n

PARAMETERS :\n
*\n
The input parameters are :
\li the path of  the images JPG quicklooks
\li the path of the MT classfile
\li the paths of the TL classfiles
\li the  path of the graph at format 2 \n
*\n

@todo The interactive learning procedure has to be incorporated so that  this interface could support the  visualisation and naviguation through the representations, but also so that it could be able to drive the interactive learning alone (without the C++ program communicating with this applet using textfiles).\n
Moreover, the GUI has to be extend to support bigger images than sizeImxsizeIm pixels using visualization tools to naviguate in the full resolution images (zoom, quiklooks generator, etc..).\n
The graph visualisation could be enhanced : visualisation of all the graph attributes, enhancing the clarety of the visualisation (color, shape, etc...), multidimenionnal visualisation (visulaizing the 3D cluster trajectories), etc.\n
As this program is very experimental and very chaotic, DOCUMENTING IT WOULD NOT ENHANCE ITS CLARETY. Therefore,  I would recommend, for future, to RE-CODE THIS APPLET FROM SCRATCH, tacking into account the above remarks.   


 * @author P.Has (IRIT / ENSAE / DLR / CNES)
 * @since January 2005
 *
 */
/*---------------------------------------------------------------------------*/


import java.lang.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.awt.image.*;
import java.text.*;

import javax.swing.*; 
import javax.swing.event.*;
import javax.swing.border.*;


 public class ITS_Graph3 {
    
    // number of MT classes
    private  int nb_TimelessClass;
    //  number of image time samples
    private  int nb_images; 
    //   number of graph nodes at a given time 
    private  int nb_nodes;
    //  dimensionnality of the feature space 
    private int nb_dim; 
    //   number of lines of the images 
    private static int dimx=200; 
    //  number of columns of the images 
    private static int dimy=200; 
    //   array of MT class index currently visualized 
    private int[] displayClass;
    
    
    private ImageIcon icon;
    private ImageIcon icon2MT;
    private ImageIcon iconClass; 
    private ImageIcon iconClass2MT;
    private ImageIcon iconClassTL; 
    private ImageIcon iconClassTL2MT;
    
    private int currentTime = 1;
    private JLabel imageLabel;private JLabel imageLabel2MT;
    private JLabel classLabel; private JLabel classLabel2MT; 
    private JLabel classLabelTL;private JLabel classLabelTL2MT;
    private JLabel graphLabel;
    private JButton button1;
    private JButton button2;
    private JButton button4;
    private JButton button5;
    private JButton buttonSB;
    private JButton buttonSB2;
    private JButton buttonSB3;
    private JButton buttonSB5;
    private JButton buttonClassA;private JButton buttonClassB;
    private JButton button3;
    private JLabel label; 
    private JLabel label3; 
    private JLabel label4;
    private Image im;
    private JLabel mouseCoordinates;
    //private Image imObject;
   
    private CoordinateArea imCoordinateArea;
    private CoordinateArea classfileArea;
    private CoordinateArea imCoordinateArea2MT;
    private CoordinateArea classfileArea2MT;
    private CoordinateArea graphCoordinateArea;
    private CoordinateArea dynaClassCoordinateArea; 
    private CoordinateArea LinksCoordinateArea;
    private JPanel buttonArea;
    private JPanel buttonArea2;
    private JPanel buttonArea3;
    private JPanel buttonArea4;
   
    private TimeNodes[] timeNodes;
    private float[][] MI;
    private float [] cardMT;
    //private MTempClassPath[] mTempClassPath;
    private TimeDelay irregularT;
    private Dates dates;
    private int sizeIm = 200; 
    private int ofx = 5;
    private int ofy = 0;
    private  int dx = 970;
    private  int dy = 285;
    private  int maxValue = 250;
    private  int minValue =50;
    private int displayDim=2;
    private float SeuilDiv=5;
    private float SeuilFlow=50;
    private double[] zoom;
    private int [] barHeigh;
    
      
    private static  DecimalFormat form2;
    private SBList sBList;
    private int indexList=0;
    private double INF=1e10;
   
    private static String labelPrefix = "Sample:";
    // path of the images JPG quicklook directory 
    private static String IMAGEPATH = "../../data/";
   //path of the MT classfile's directory 
    private static String CLASSFILEPATH_MT = "../../data/MTpca.cl";
    // path of the TL classfiles' directory 
    private static String CLASSFILEPATH_TL = "../../data/"; 
   
    private void readGraphDataV2() { 
   //path of the graph at format 2 
   String path= "../../data/grapheV2JAVA";
   String ligne = null;
   String[] value = null;
   int l=0;
   int indexNode;int indexNodePre;
   int divOrFlow=0;  
     
//read data text     
   try{
     RandomAccessFile p= new RandomAccessFile(path,"r");
     ligne  =p.readLine();
     value =ligne.split("  ");
     
     
//fill graph    
     nb_TimelessClass= (int) Float.parseFloat(value[0]);  		 				//Number of multitemporal classes
     System.err.print("NB MT CLASSES ");System.err.println(nb_TimelessClass);
     nb_dim= (int) Float.parseFloat(value[1]);       							//Feature Space Dimension
     System.err.print("NB DIM FS ");System.err.println(nb_dim);
     nb_images= (int) Float.parseFloat(value[2]); 							//Number of images
     System.err.print("NB IMAGE SAMPLES ");System.err.println(nb_images);
     MI=new float[nb_TimelessClass][nb_images-1];    
     cardMT=new float[nb_TimelessClass];   
     timeNodes= new TimeNodes[nb_images];
     for (int i=0;i<nb_images;i++){timeNodes[i]=new TimeNodes();} 			 				
     l=3;
     for (int kMT=0;kMT<nb_TimelessClass;kMT++) {
     	l++; 													//MT Index
	System.err.print("READING MT CLASS ");System.err.println(kMT);
     	cardMT[kMT]=(float) Float.parseFloat(value[l]);l++;  							//MT weight
     	for (int t=0;t<nb_images;t++) {
	   
     	   l++; 												    //time sample
     	   for (int i=0;i<nb_dim;i++){l++;}									    //MT_t mean
     	   for (int i=0;i<nb_dim;i++){for (int j=0;j<nb_dim;j++){l++;}}						    //MT_t cov	   
     	   nb_nodes= (int) Float.parseFloat(value[l]);l++ ;				 			    //nb TL_t Classes
     	   if (kMT==0){
	      timeNodes[t].AllocateTimeNodes(nb_nodes); 
	      for (int kTL=0;kTL<timeNodes[t].totalTimeNb_node;kTL++){
	              timeNodes[t].node[kTL].allocate(100); 
	              timeNodes[t].node[kTL].nbMultitempClass=0; 
	      }
	   }
     	   for (int kTL=0;kTL<timeNodes[t].totalTimeNb_node;kTL++){
	       indexNode=(int) Float.parseFloat(value[l]);l++; 						      		//TL_t cluster index 
	       timeNodes[t].node[indexNode].nbMultitempClass++; 			      
     	       timeNodes[t].node[indexNode].multitempClass[kMT]=kMT;
	       l++;											      		//weight TL_t&MT_t
     	       timeNodes[t].node[indexNode].divKullback[kMT]=(float) Float.parseFloat(value[l]);l++;			//divergence TL_t&MT_t
	       	
     	       for (int i=0;i<nb_dim;i++){timeNodes[t].node[indexNode].mean[i]=(float) Float.parseFloat(value[l]);l++;}	//TL_t measn
     	       for (int i=0;i<nb_dim;i++){										//TL_t cov    
	       		for (int j=0;j<nb_dim;j++){
				timeNodes[t].node[indexNode].cov[i][j]=(float) Float.parseFloat(value[l]);l++;
			} 
	       }
     	       if (t>0){
	           if (kMT==0){timeNodes[t].node[indexNode].allocateEdges(timeNodes[t-1].totalTimeNb_node);}
     	           for (int kTLpre=0;kTLpre<timeNodes[t-1].totalTimeNb_node;kTLpre++){
     	             indexNodePre=(int) Float.parseFloat(value[l]);l++; 							//TL_t-1 cluster index
	             timeNodes[t].node[indexNode].ingoingEdges[indexNodePre].indexNodeA=indexNodePre;
	             timeNodes[t].node[indexNode].ingoingEdges[indexNodePre].indexNodeB=indexNode;
		     if (kMT==0){timeNodes[t].node[indexNode].ingoingEdges[indexNodePre].flowAlloc(nb_TimelessClass);}
     	             timeNodes[t].node[indexNode].ingoingEdges[indexNodePre].flowSave[kMT]=(int) Float.parseFloat(value[l]);l++; 	//flow TL_t-1&TL_t&t&MT_t
		     timeNodes[t].node[indexNode].ingoingEdges[indexNodePre].flowMTindex[kMT]=(char)kMT;
		     timeNodes[t].node[indexNode].ingoingEdges[indexNodePre].flow[kMT]=timeNodes[t].node[indexNode].ingoingEdges[indexNodePre].flowSave[kMT];
		     l++;													//div TL_t-1&TL_t&t&MT_t  
     	           }
     	       }
	       ligne=p.readLine();
	       value =ligne.split("  ");
	       l=0;
     	   } 
	   if (t>0){
     	   	MI[kMT][t-1]=(float) Float.parseFloat(value[l]); l++;							    //MI MT_t-1&MT_t
	   	if (kMT<nb_TimelessClass-1||t<nb_images-1){
			ligne=p.readLine();
	  		value =ligne.split("  ");
	  		l=0;
		} 
	   }  	
     	}
     } 
   } 
   catch (Exception e) {System.err.println("problem reading graphV2JAVA file\n");}     
     
  } 

   
    public Component createButton() {
       
	label = new JLabel(labelPrefix + "1");
	button1 = new JButton("Image+");  	
	button1.setMnemonic(KeyEvent.VK_I);
	button1.addActionListener(new ActionListener() {
	   public void actionPerformed(ActionEvent e) {
                
		currentTime++;
		icon=new ImageIcon(getImage1(currentTime));
		iconClassTL=new ImageIcon(getImage1(currentTime+1));
		classLabelTL.setIcon(iconClassTL);
		imageLabel.setIcon(icon);
		icon2MT=new ImageIcon(getImage1(currentTime));
		iconClassTL2MT=new ImageIcon(getImage1(currentTime));
		classLabelTL2MT.setIcon(iconClassTL2MT);
		imageLabel2MT.setIcon(icon2MT);
                label.setText(labelPrefix + currentTime);
		dynaClassCoordinateArea.repaint();
		graphCoordinateArea.repaint();
		classfileArea2MT.repaint();
		iconClass=new ImageIcon(getImage1(currentTime-1));
		classLabel.setIcon(iconClass);
		
            }
        });
	label.setLabelFor(button1);
	button2 = new JButton("Image-");  	
	button2.setMnemonic(KeyEvent.VK_I);
	button2.addActionListener(new ActionListener() {
	   public void actionPerformed(ActionEvent e) {
                currentTime--;
		icon=new ImageIcon(getImage1(currentTime));
		iconClassTL=new ImageIcon(getImage1(currentTime+1));
		imageLabel.setIcon(icon);
		classLabelTL.setIcon(iconClassTL);
		icon2MT=new ImageIcon(getImage1(currentTime));
		iconClassTL2MT=new ImageIcon(getImage1(currentTime));
		imageLabel2MT.setIcon(icon2MT);
		classLabelTL2MT.setIcon(iconClassTL2MT);
                label.setText(labelPrefix + currentTime);
		dynaClassCoordinateArea.repaint();
		graphCoordinateArea.repaint();
		classfileArea2MT.repaint();
		iconClass=new ImageIcon(getImage1(currentTime-1));
		classLabel.setIcon(iconClass);
            }
        });
	label.setLabelFor(button2);
       	buttonClassB = new JButton("2nd MT+");  	
	buttonClassB.setMnemonic(KeyEvent.VK_I);
	buttonClassB.addActionListener(new ActionListener() {
	   public void actionPerformed(ActionEvent e) {
                displayClass[1]=displayClass[1]+1;  
		if  (displayClass[1]>nb_TimelessClass){displayClass[1]=1;}
		label4.setText("Tresh:" +SeuilDiv+"/MT:"+displayClass[0]+"&"+displayClass[1]+"/");
		iconClassTL2MT=new ImageIcon(getImage1(currentTime));
                iconClass2MT=new ImageIcon(getImage1(currentTime));			 
		classLabelTL2MT.setIcon(iconClassTL2MT);
                label.setText(labelPrefix + currentTime);
		
				 
		graphCoordinateArea.repaint();
		dynaClassCoordinateArea.repaint();
		classfileArea2MT.repaint();
		iconClass=new ImageIcon(getImage1(currentTime-1));
		classLabel.setIcon(iconClass);		 			 
		
            }
        });
	
	buttonArea =new JPanel();
	buttonArea.setLayout(new BorderLayout());
	buttonArea.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
	buttonArea.add(buttonClassB,BorderLayout.WEST);
	buttonArea.add(button1,BorderLayout.EAST);
	buttonArea.add(button2,BorderLayout.CENTER);
	buttonArea.add(mouseCoordinates,BorderLayout.SOUTH);
        buttonArea.add(label,BorderLayout.NORTH);
	return buttonArea;
    } 
    public Component createButton4() {
        displayClass=new int[3];
        displayClass[0]=1;
        displayClass[1]=2;
        displayClass[2]=4;
	
	
	
	label4 = new JLabel("Tresh:" +SeuilDiv+"/MT:"+displayClass[0]+"&"+displayClass[1]+"/");
	button4 = new JButton("DivTresh-");  	
	button4.setMnemonic(KeyEvent.VK_I);
	button4.addActionListener(new ActionListener() {
	   public void actionPerformed(ActionEvent e) {
                
		SeuilDiv=SeuilDiv-10;
		updateSeuil();
                label4.setText("Tresh:" +SeuilDiv+"/MT:"+displayClass[0]+"&"+displayClass[1]+"/");
		graphCoordinateArea.repaint();
		dynaClassCoordinateArea.repaint();
		classfileArea2MT.repaint();
		iconClass=new ImageIcon(getImage1(currentTime-1));
		classLabel.setIcon(iconClass);
		
            }
        });
	label4.setLabelFor(button4);
	
	
	buttonSB3 = new JButton("SG++");  	
	buttonSB3.setMnemonic(KeyEvent.VK_I);	
	buttonSB3.addActionListener(new ActionListener() {
	   public void actionPerformed(ActionEvent e) {
		
		indexList=indexList+(int)((float)sBList.size/(float)10);
		displayClass[0]=sBList.MTa[indexList]+1;
		displayClass[1]=sBList.MTb[indexList]+1;
		currentTime=sBList.time[indexList]+1;
		graphCoordinateArea.repaint();
		LinksCoordinateArea.repaint();
		dynaClassCoordinateArea.repaint();
		classfileArea2MT.repaint();
		iconClass=new ImageIcon(getImage1(currentTime-1));
		classLabel.setIcon(iconClass);
		label4.setText("Tresh:" +SeuilDiv+"/MT:"+displayClass[0]+"&"+displayClass[1]+"/");
		iconClassTL=new ImageIcon(getImage1(currentTime+1));
		iconClass=new ImageIcon(getImage1(currentTime-1));
		classLabelTL.setIcon(iconClassTL);
		classLabel.setIcon(iconClass);
		iconClassTL2MT=new ImageIcon(getImage1(currentTime));
                iconClass2MT=new ImageIcon(getImage1(currentTime));			 
		classLabelTL2MT.setIcon(iconClassTL2MT);
		icon=new ImageIcon(getImage1(currentTime));
		icon2MT=new ImageIcon(getImage1(currentTime));
		imageLabel.setIcon(icon);
		imageLabel2MT.setIcon(icon2MT);
                label.setText(labelPrefix + currentTime+"/Prob:"+(float)((int)(sBList.Prob[indexList]*(float)1000))/(float)1000);
		mouseCoordinates.setText("Post Mean/Var:"+(float)((int)(sBList.PostMean*(float)1000))/(float)1000+"/"+(float)((int)(sBList.PostVar*(float)1000))/(float)1000);
            }
        });
	
	
	
	
	
	
	
	
	buttonArea3 =new JPanel();
	buttonArea3.setLayout(new BorderLayout());
	buttonArea3.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
	buttonArea3.add(button4,BorderLayout.SOUTH);
	buttonArea3.add(buttonSB3,BorderLayout.CENTER);
        buttonArea3.add(label4,BorderLayout.NORTH);
	return buttonArea3;
    }
    
    public Component createButton5() {
        
       
	
	buttonSB = new JButton("Search");  	
	buttonSB.setMnemonic(KeyEvent.VK_I);	
	sBList=new SBList();
        buttonSB.addActionListener(new ActionListener() {
	   public void actionPerformed(ActionEvent e) {
	
		sBList=new SBList();
                indexList=0;
		sBList.InitList();
        	sBList.ReadList();
		for (int i=0;i<7;i++){barHeigh[i]=(int) ((float)sBList.ProbLink[i]*(float)100.);}
		displayClass[0]=sBList.MTa[indexList]+1;
		displayClass[1]=sBList.MTb[indexList]+1;
		currentTime=sBList.time[indexList]+1;
		graphCoordinateArea.repaint();
		dynaClassCoordinateArea.repaint();
		classfileArea2MT.repaint();
		iconClass=new ImageIcon(getImage1(currentTime-1));
		classLabel.setIcon(iconClass);
		LinksCoordinateArea.repaint();
		label4.setText("Tresh:" +SeuilDiv+"/MT:"+displayClass[0]+"&"+displayClass[1]+"/");
		iconClassTL=new ImageIcon(getImage1(currentTime+1));
		iconClass=new ImageIcon(getImage1(currentTime-1));
		classLabelTL.setIcon(iconClassTL);
		classLabel.setIcon(iconClass);
		iconClassTL2MT=new ImageIcon(getImage1(currentTime));
                iconClass2MT=new ImageIcon(getImage1(currentTime));			 
		classLabelTL2MT.setIcon(iconClassTL2MT);
		icon=new ImageIcon(getImage1(currentTime));
		icon2MT=new ImageIcon(getImage1(currentTime));
		imageLabel.setIcon(icon);
		imageLabel2MT.setIcon(icon2MT); 
		label.setText(labelPrefix + currentTime+"/Prob:"+(float)((int)(sBList.Prob[indexList]*(float)1000))/(float)1000);
		mouseCoordinates.setText("Post Mean/Var:"+(float)((int)(sBList.PostMean*(float)1000))/(float)1000+"/"+(float)((int)(sBList.PostVar*(float)1000))/(float)1000);
            }
        });
	
	
	buttonSB5 = new JButton("SGorder-");  	
	buttonSB5.setMnemonic(KeyEvent.VK_I);	
	sBList=new SBList();
        buttonSB5.addActionListener(new ActionListener() {
	   public void actionPerformed(ActionEvent e) {
	
		sBList=new SBList();
                indexList=0;
		sBList.InitList();
        	sBList.ReadInvList();
		for (int i=0;i<7;i++){barHeigh[i]=(int)sBList.ProbLink[i]*100;}
		displayClass[0]=sBList.MTa[indexList]+1;
		displayClass[1]=sBList.MTb[indexList]+1;
		currentTime=sBList.time[indexList]+1;
		graphCoordinateArea.repaint();
		dynaClassCoordinateArea.repaint();
		classfileArea2MT.repaint();
		iconClass=new ImageIcon(getImage1(currentTime-1));
		classLabel.setIcon(iconClass);
		LinksCoordinateArea.repaint();
		label4.setText("Tresh:" +SeuilDiv+"/MT:"+displayClass[0]+"&"+displayClass[1]+"/");
		iconClassTL=new ImageIcon(getImage1(currentTime+1));
		iconClass=new ImageIcon(getImage1(currentTime-1));
		classLabelTL.setIcon(iconClassTL);
		classLabel.setIcon(iconClass);
		iconClassTL2MT=new ImageIcon(getImage1(currentTime));
                iconClass2MT=new ImageIcon(getImage1(currentTime));			 
		classLabelTL2MT.setIcon(iconClassTL2MT);	
		icon=new ImageIcon(getImage1(currentTime));
		icon2MT=new ImageIcon(getImage1(currentTime));
		imageLabel.setIcon(icon);
		imageLabel2MT.setIcon(icon2MT); 
		label.setText(labelPrefix + currentTime+"/Prob:"+(float)((int)(sBList.Prob[indexList]*(float)1000))/(float)1000);
		mouseCoordinates.setText("Post Mean/Var:"+(float)((int)(sBList.PostMean*(float)1000))/(float)1000+"/"+(float)((int)(sBList.PostVar*(float)1000))/(float)1000);
            }
        });
	
	buttonSB2 = new JButton("Next");  	
	buttonSB2.setMnemonic(KeyEvent.VK_I);	
	buttonSB2.addActionListener(new ActionListener() {
	   public void actionPerformed(ActionEvent e) {
		
		indexList++;
		displayClass[0]=sBList.MTa[indexList]+1;
		displayClass[1]=sBList.MTb[indexList]+1;
		currentTime=sBList.time[indexList]+1;
		graphCoordinateArea.repaint();
		dynaClassCoordinateArea.repaint();
		classfileArea2MT.repaint();
		iconClass=new ImageIcon(getImage1(currentTime-1));
		classLabel.setIcon(iconClass);
		LinksCoordinateArea.repaint();
		label4.setText("Tresh:" +SeuilDiv+"/MT:"+displayClass[0]+"&"+displayClass[1]+"/");
		iconClassTL=new ImageIcon(getImage1(currentTime+1));
		iconClass=new ImageIcon(getImage1(currentTime-1));
		classLabelTL.setIcon(iconClassTL);
		classLabel.setIcon(iconClass);
		iconClassTL2MT=new ImageIcon(getImage1(currentTime));
                iconClass2MT=new ImageIcon(getImage1(currentTime));			 
		classLabelTL2MT.setIcon(iconClassTL2MT);
		icon=new ImageIcon(getImage1(currentTime));
		icon2MT=new ImageIcon(getImage1(currentTime));
		imageLabel.setIcon(icon);
		imageLabel2MT.setIcon(icon2MT);
                label.setText(labelPrefix + currentTime+"/Prob:"+(float)((int)(sBList.Prob[indexList]*(float)1000))/(float)1000);
		mouseCoordinates.setText("Post Mean/Var:"+(float)((int)(sBList.PostMean*(float)1000))/(float)1000+"/"+(float)((int)(sBList.PostVar*(float)1000))/(float)1000);
		
		
	    }
        });

	
	buttonArea4 =new JPanel();
	buttonArea4.setLayout(new BorderLayout());
	buttonArea4.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
	buttonArea4.add(buttonSB,BorderLayout.NORTH);
	buttonArea4.add(buttonSB2,BorderLayout.CENTER);
	buttonArea4.add(buttonSB5,BorderLayout.SOUTH);
	
	return buttonArea4;
    }
    public Component createButton2() {
    	label3 = new JLabel(" DimIndex : 1");
	button3 = new JButton(" Dim+");  	
	button3.setMnemonic(KeyEvent.VK_I);
	button3.addActionListener(new ActionListener() {
	   public void actionPerformed(ActionEvent e) {
                
		if (displayDim<nb_dim-1) 
			displayDim++;
		else
			displayDim=0;

		displayDim++;
                label3.setText(" Dim: "+displayDim);
		displayDim--;
		graphCoordinateArea.repaint();
		dynaClassCoordinateArea.repaint();
		classfileArea2MT.repaint();
		iconClass=new ImageIcon(getImage1(currentTime-1));
		classLabel.setIcon(iconClass);
            }
        });
	
	label3.setLabelFor(button3);
	
	
	
        button5 = new JButton("DivTresh+");  	
	button5.setMnemonic(KeyEvent.VK_I);
	button5.addActionListener(new ActionListener() {
	   public void actionPerformed(ActionEvent e) {
                SeuilDiv=SeuilDiv+2;
		updateSeuil();
                label4.setText("Tresh:" +SeuilDiv+"/MT:"+displayClass[0]+"&"+displayClass[1]+"/");
		graphCoordinateArea.repaint();
		dynaClassCoordinateArea.repaint();
		classfileArea2MT.repaint();
		iconClass=new ImageIcon(getImage1(currentTime-1));
		classLabel.setIcon(iconClass);
            }
        });
	
	
		
	buttonArea2 =new JPanel();
	buttonArea2.setLayout(new BorderLayout());
	buttonArea2.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
	buttonArea2.add(button5,BorderLayout.SOUTH);
	buttonArea2.add(button3,BorderLayout.CENTER);
	buttonArea2.add(label3,BorderLayout.NORTH);

	return buttonArea2;
	
    }
    
    
    
    
    
    
    /***************************************************ClassFile & Images*************************************************/
    
    
    
    
    
    
    public Component createComponentsIm() {
	
	
	
	icon=new ImageIcon(getImage1(1));
	iconClassTL=new ImageIcon(getImage1(1));
	imageLabel=new JLabel("",icon,JLabel.CENTER);
	classLabelTL=new JLabel("",iconClassTL,JLabel.CENTER);
        imCoordinateArea.setLayout(new BorderLayout());
        imCoordinateArea.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
        imCoordinateArea.add(imageLabel,BorderLayout.WEST);
        imCoordinateArea.add(classLabelTL,BorderLayout.EAST);
	return imCoordinateArea;
    }
    
    public Component createComponentsClass() throws IOException {
  
	iconClass=new ImageIcon(getImage1(1));
	classLabel=new JLabel("",iconClass,JLabel.CENTER);
	
        classfileArea.setLayout(new BorderLayout());
        classfileArea.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
        classfileArea.add(classLabel,BorderLayout.CENTER); 
	
	return classfileArea;
    } 
   
     public Component createComponentsIm2MT() {
	
	
	icon2MT=new ImageIcon(getImage1(1));
	iconClassTL2MT=new ImageIcon(getImage1(1));
	imageLabel2MT=new JLabel("",icon2MT,JLabel.CENTER);
	classLabelTL2MT=new JLabel("",iconClassTL2MT,JLabel.CENTER);
        imCoordinateArea2MT.setLayout(new BorderLayout());
        imCoordinateArea2MT.setBorder(BorderFactory.createEmptyBorder(0, 0, 0,40));
	barHeigh=new int[7];
	for (int i=0;i<7;i++){barHeigh[i]=0;}
	Component links = createComponentsLinks();	
	buttonClassA = new JButton("1st MT+");  	
	buttonClassA.setMnemonic(KeyEvent.VK_I);
	buttonClassA.addActionListener(new ActionListener() {
	   public void actionPerformed(ActionEvent e) {
                
		displayClass[0]=displayClass[0]+1; 
		if  (displayClass[0]>nb_TimelessClass){displayClass[0]=1;}
		label4.setText("Tresh:" +SeuilDiv+"/MT:"+displayClass[0]+"&"+displayClass[1]+"/");
		iconClassTL=new ImageIcon(getImage1(currentTime+1));
		iconClass=new ImageIcon(getImage1(currentTime-1));
		classLabelTL.setIcon(iconClassTL);
		classLabel.setIcon(iconClass);
                label.setText(labelPrefix + currentTime);
					 
		graphCoordinateArea.repaint();	
		dynaClassCoordinateArea.repaint();
		classfileArea2MT.repaint();
		iconClass=new ImageIcon(getImage1(currentTime-1));
		classLabel.setIcon(iconClass);			 
		
		
		
		
		
            }
        });
	Component dynaClass = createComponentsDynaClass();
	
	imCoordinateArea2MT.add(links,BorderLayout.WEST);
	imCoordinateArea2MT.add(dynaClass,BorderLayout.CENTER);
	imCoordinateArea2MT.add(buttonClassA,BorderLayout.EAST);
	return imCoordinateArea2MT;
    }
    
    public Component createComponentsClass2MT() throws IOException {
  
	classfileArea2MT.setOpaque(false);
        classfileArea2MT.setLayout(new FlowLayout() );
	
	return classfileArea2MT;
    } 
   
    public Component createComponentsDynaClass() {
	
	dynaClassCoordinateArea.setOpaque(false);
        dynaClassCoordinateArea.setLayout(new FlowLayout() );
	
	
	return dynaClassCoordinateArea;
    }   
		
   
    public Component createComponentsGraph() {
	
	graphCoordinateArea.setOpaque(false);
        graphCoordinateArea.setLayout(new FlowLayout() );
	
	
	return graphCoordinateArea;
    }
    public Component createComponentsLinks() {
	
	LinksCoordinateArea.setOpaque(false);
        LinksCoordinateArea.setLayout(new FlowLayout() );
	
	
	return LinksCoordinateArea;
    }
    public void buildUI() { 
		
			
        	imCoordinateArea = new CoordinateArea(this);
		imCoordinateArea.preferredSize= new Dimension(400,sizeIm);
		imCoordinateArea2MT = new CoordinateArea(this);
		imCoordinateArea2MT.preferredSize= new Dimension(400,sizeIm);
		
        	mouseCoordinates= new JLabel("Click within the framed area.");	
        	classfileArea= new CoordinateArea(this);
		classfileArea.preferredSize= new Dimension(sizeIm,sizeIm);
		
		
		classfileArea2MT = new CoordinateArea(this){
              		 public void paintComponent(Graphics g) {
               				Graphics2D gMT= (Graphics2D)g;
               				buildMTClass(gMT);
               				super.paintComponent(gMT);
               		}
       		 };
		classfileArea2MT.preferredSize= new Dimension(sizeIm,sizeIm);
		
		
		irregularT=new TimeDelay();
		dates=new Dates();
		
		readGraphDataV2();
		updateSeuil();
		graphCoordinateArea = new CoordinateArea(this){
                             public void paintComponent(Graphics g) {
                         	Graphics2D g2 = (Graphics2D)g;
                         	buildGraph(g2);
                                super.paintComponent(g2);
                             }
                   };
		graphCoordinateArea.preferredSize= new Dimension(980,290);
		
		dynaClassCoordinateArea = new CoordinateArea(this){
                             public void paintComponent(Graphics gDyna) {
                         	Graphics2D dyna = (Graphics2D)gDyna;
                         	buildDynaClass(dyna);
                                super.paintComponent(dyna);
                             }
                   };
		dynaClassCoordinateArea.preferredSize= new Dimension(sizeIm,sizeIm);
		
		LinksCoordinateArea = new CoordinateArea(this){
                             public void paintComponent(Graphics g) {
                         	Graphics2D g2 = (Graphics2D)g;
                         	buildLinks(g2);
                                super.paintComponent(g2);
                             }
                   };
		LinksCoordinateArea.preferredSize= new Dimension(sizeIm,sizeIm);
    }
    
    
private void buildLinks(Graphics2D g2){

	
      
      g2.setFont(new Font("Arial", Font.PLAIN,10));
      g2.setColor(Color.red);
      g2.drawString("weight",12,barHeigh[0]+10);
      g2.fillRect(12,0,25,barHeigh[0]);
      g2.setColor(Color.cyan);
      g2.drawString("pos",37,barHeigh[1]+20);
      g2.fillRect(37,0,25,barHeigh[1]);
      g2.setColor(Color.green);
      g2.drawString("flow",62,barHeigh[2]+10);
      g2.fillRect(62,0,25,barHeigh[2]);
      g2.setColor(Color.yellow);
      g2.drawString("change",87,barHeigh[3]+20);
      g2.fillRect(87,0,25,barHeigh[3]);
      g2.setColor(Color.blue);
      g2.drawString("MI",112,barHeigh[4]+10);
      g2.fillRect(112,0,25,barHeigh[4]);
      g2.setColor(Color.magenta);
      g2.drawString("2MT",137,barHeigh[5]+20);
      g2.fillRect(137,0,25,barHeigh[5]);
      g2.setColor(Color.pink);
      g2.drawString("time",162,barHeigh[6]+10);
      g2.fillRect(162,0,25,barHeigh[6]);
      
}
private void buildMTClass(Graphics2D g2){
   try{
    	g2.setColor(Color.white);
	g2.fillRect(0,0,sizeIm,sizeIm);

	int[][] classAux;
	classAux=new int[dimx][dimy];
	Reader in;Reader in2;
	String path;String path2;
	path=CLASSFILEPATH_MT;
	in=new FileReader(path);
	for (int i=0;i<dimx;i++){
		for (int j=0;j<dimy;j++){
			classAux[i][j]=in.read();
			
			if (classAux[i][j]==displayClass[0]-1){
				 g2.setColor(Color.red);
				 g2.drawLine(j,i,j,i);	
			}
			if (classAux[i][j]==displayClass[1]-1){
				 g2.setColor(Color.orange);
				 g2.drawLine(j,i,j,i);	
			}
		}
		
	}
    }
    catch (Exception e) {System.err.println("problem reading MTdynaClass file\n");} 
}
private void buildDynaClass(Graphics2D g2)  {
    try{
    	g2.setColor(Color.white);
	g2.fillRect(0,0,sizeIm,sizeIm);
	
	int[][] classAux;
	classAux=new int[dimx][dimy];	
	Reader in;
	String path;
	path=getClassfileTL(currentTime);
	in=new FileReader(path);
      
	double min1=INF;double min2=INF;int IndexMin1=256;int IndexMin2=256;
	if (SeuilDiv<0){
	   for (int indexNode=0;indexNode<timeNodes[currentTime-1].totalTimeNb_node;indexNode++){
		if (timeNodes[currentTime-1].node[indexNode].divKullback[displayClass[1]-1]<min1){
			IndexMin1=indexNode;
			min1=timeNodes[currentTime-1].node[indexNode].divKullback[displayClass[1]-1];
		}
		if (timeNodes[currentTime-1].node[indexNode].divKullback[displayClass[0]-1]<min2){
			IndexMin2=indexNode;
			min2=timeNodes[currentTime-1].node[indexNode].divKullback[displayClass[0]-1];
		}
	   }
	   System.err.println(min1);System.err.println(IndexMin1);System.err.println(timeNodes[currentTime-1].totalTimeNb_node);
	   for (int i=0;i<dimx;i++){
		for (int j=0;j<dimy;j++){
	   		classAux[i][j]=in.read();
			
			if (IndexMin1==classAux[i][j]){
				g2.setColor(Color.orange);
				g2.drawLine(j,i,j,i);	
			}
			if(IndexMin2==classAux[i][j]){
				g2.setColor(Color.red);
				g2.drawLine(j,i,j,i);
			}
			if (IndexMin1==classAux[i][j]&&IndexMin2==classAux[i][j]){
				g2.setColor(Color.black);
				g2.drawLine(j,i,j,i);
			}
		}
		
	  }
			
	}
	else{
	   for (int i=0;i<dimx;i++){
		for (int j=0;j<dimy;j++){
			classAux[i][j]=in.read();
			
			if (timeNodes[currentTime-1].node[classAux[i][j]].divKullback[displayClass[1]-1]<=SeuilDiv){
				
				g2.setColor(Color.orange);
				g2.drawLine(j,i,j,i);	
			}
			
			if (timeNodes[currentTime-1].node[classAux[i][j]].divKullback[displayClass[0]-1]<=SeuilDiv){
				g2.setColor(Color.red);
				g2.drawLine(j,i,j,i);
				if (timeNodes[currentTime-1].node[classAux[i][j]].divKullback[displayClass[1]-1]<=SeuilDiv){	
					g2.setColor(Color.black);
				  	g2.drawLine(j,i,j,i);	
				}
			}
			
		}
	   }
	}
    }
    catch (Exception e) {System.err.println("problem reading TLdynaClass file\n");} 
} 


/***************************************************Graph*************************************************/
    
private void buildGraph(Graphics2D g2){

			
      
/*      		
** Support and graph axis
*/ g2.setColor(Color.black);
      
      g2.setColor(new Color(255,255,255));
      g2.setFont(new Font("Arial", Font.PLAIN,12));
  
      g2.fillRect(ofx,ofy,dx+ofx,dy+ofy);
      g2.setColor(Color.black);
      g2.drawLine(ofx, ofy, ofx, dy+ofy);
      g2.drawLine(ofx, dy+ofy , dx+ofx, dy+ofy);
     
      g2.drawLine(ofx,ofy+dy, ofx+5, ofy+dy);
      g2.drawLine(ofx,ofy+(int)(dy*3/4), ofx+5, ofy+(int)(dy*3/4));
      g2.drawLine(ofx,ofy+(int)(dy/2), ofx+5, ofy+(int)(dy/2));
      g2.drawLine(ofx,ofy+(int)(dy/4), ofx+5, ofy+(int)(dy/4));
      g2.drawLine(ofx,ofy, ofx+5, ofy); 
		
      g2.drawLine(ofx+dx, ofy+dy,ofx+dx, ofy+dy-5);
      g2.drawLine(ofx+(int)(dx*3/4), ofy+dy,ofx+(int)(dx*3/4), ofy+dy-5);
      g2.drawLine(ofx+(int)(dx/2), ofy+dy,ofx+(int)(dx/2), ofy+dy-5);
      g2.drawLine(ofx+(int)(dx/4), ofy+dy,ofx+(int)(dx/4), ofy+dy-5);
      g2.drawLine(ofx, ofy+dy, ofx, ofy+dy-5);	
     
      g2.drawString("Date",ofx+dx-30,ofy+dy-10);
      g2.drawString("Feature ",ofx+15,ofy+20);

      
	
/*      		
** graph painting
*/         
      
      Stroke stroke;
      float thickness=1;
      int eps=1;
      int indexClas;
      int IndexMultitemp=-1,alpha=0; 
      int disp1=0;
      int disp2=1;
      
      zoom=new double[3];
      zoom[0]=1.3;
      zoom[1]=1.3;
      zoom[2]=0.9;
      
      int [] IndexDivMin1=new int[nb_images];
      int [] IndexDivMin2=new int[nb_images];
      
      
      for (int t=0;t<nb_images;t++){

	double min1=1e40;double min2=1e40;
	boolean minEdge=false;
        for (int nodeK=0;nodeK<timeNodes[t].totalTimeNb_node;nodeK++){
		if (timeNodes[t].node[nodeK].divKullback[(displayClass[disp1]-1)]<min1){
			IndexDivMin1[t]=nodeK;
			min1=timeNodes[t].node[nodeK].divKullback[(displayClass[disp1]-1)];
		}
		if (timeNodes[t].node[nodeK].divKullback[(displayClass[disp2]-1)]<min2){
			IndexDivMin2[t]=nodeK;
			min2=timeNodes[t].node[nodeK].divKullback[(displayClass[disp2]-1)];
		}		
	}
	for (int nodeK=0;nodeK<timeNodes[t].totalTimeNb_node;nodeK++){
	
		if (t>0){	
//edge Attributes	
		    for (int edgeK=0;edgeK<timeNodes[t].node[nodeK].nbIngoingEdges;edgeK++){		       
		    	thickness=(float)(Math.log((double)timeNodes[t].node[nodeK].ingoingEdges[edgeK].flow[(displayClass[disp1]-1)]+(double)1)/2.);
			minEdge=(IndexDivMin1[t]==nodeK)&&(IndexDivMin1[t-1]==timeNodes[t].node[nodeK].ingoingEdges[edgeK].indexNodeA);
			if (minEdge&&SeuilDiv<=0){
thickness=(float)(Math.log((double)timeNodes[t].node[nodeK].ingoingEdges[edgeK].flowSave[(displayClass[disp1]-1)]+(double)1)/2.);
			}
		        if (thickness!=0||minEdge){
			            //edge above div treshold but most likely
				    if (minEdge&&thickness==0){ 
					 //edge of interest
				   	 for (int kSB=0;kSB<sBList.size;kSB++) {
					 	if (t>sBList.time[kSB]&&t<(sBList.time[kSB]+sBList.tWindow)){
						    if((displayClass[disp1]-1)==sBList.MTa[kSB]){
							if ((displayClass[disp2]-1)==sBList.MTb[kSB]){
								alpha=(int)(sBList.Prob[kSB]*255.);
								g2.setColor(new Color(10, 255, 10, alpha));
									g2.fillRect(irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy,irregularT.irregularT[t+1]*dx/irregularT.irregularT[nb_images]-irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images], ofy+20);
								if (t==sBList.time[kSB]+1){g2.setColor(Color.black);g2.drawString(""+((float)(int)(sBList.Prob[kSB]*(float)100)/(float)100),irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy+30);g2.drawString(""+irregularT.irregularT[t],irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy+40);g2.setColor(new Color(10, 255, 10, alpha));}
							}
						    }
						    if((displayClass[disp2]-1)==sBList.MTa[kSB]){
							if ((displayClass[disp1]-1)==sBList.MTb[kSB]){
								alpha=(int)(sBList.Prob[kSB]*255.);
								g2.setColor(new Color(10, 255, 10, alpha));
									g2.fillRect(irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy,irregularT.irregularT[t+1]*dx/irregularT.irregularT[nb_images]-irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images], ofy+20);
								if (t==sBList.time[kSB]+1){g2.setColor(Color.black);g2.drawString(""+((float)(int)(sBList.Prob[kSB]*(float)100)/(float)100),irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy+30);g2.drawString(""+irregularT.irregularT[t],irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy+40);g2.setColor(new Color(10, 255, 10, alpha));}
							}
						    }
						}
					 }
					 
					 //all edges
				         g2.setColor(Color.red);
					 float dash1[] = {10.0f};
              			  	 BasicStroke bs = new BasicStroke(1, BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER, 10.0f, dash1, 0.0f);
             			   	 g2.setStroke(bs);
				    }
				    //edge bellow div treshold
              			    else{
				    	 //edge of interest
				   	 for (int kSB=0;kSB<sBList.size;kSB++) {
					 	if (t>sBList.time[kSB]&&t<(sBList.time[kSB]+sBList.tWindow)){
						    if((displayClass[disp1]-1)==sBList.MTa[kSB]){
							if ((displayClass[disp2]-1)==sBList.MTb[kSB]){
								alpha=(int)(sBList.Prob[kSB]*255.);
								g2.setColor(new Color(10, 255, 10, alpha));
							 		g2.fillRect(irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy,irregularT.irregularT[t+1]*dx/irregularT.irregularT[nb_images]-irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images], ofy+20);
									if (t==sBList.time[kSB]+1){g2.setColor(Color.black);g2.drawString(""+((float)(int)(sBList.Prob[kSB]*(float)100)/(float)100),irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy+30);g2.drawString(""+irregularT.irregularT[t],irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy+40);g2.setColor(new Color(10, 255, 10, alpha));}
							}
						    }
						    if((displayClass[disp2]-1)==sBList.MTa[kSB]){
							if ((displayClass[disp1]-1)==sBList.MTb[kSB]){
								alpha=(int)(sBList.Prob[kSB]*255.);
								g2.setColor(new Color(10, 255, 10, alpha));
									g2.fillRect(irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy,irregularT.irregularT[t+1]*dx/irregularT.irregularT[nb_images]-irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images], ofy+20);
									if (t==sBList.time[kSB]+1){g2.setColor(Color.black);g2.drawString(""+((float)(int)(sBList.Prob[kSB]*(float)100)/(float)100),irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy+30);g2.drawString(""+irregularT.irregularT[t],irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy+40);g2.setColor(new Color(10, 255, 10, alpha));}
							}
						    }
						}
					 }
					 //all edges
				         g2.setColor(Color.red);
					 stroke = new BasicStroke(thickness, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
                                    	 g2.setStroke(stroke); 
				    }
				    
g2.drawLine(irregularT.irregularT[t+1]*dx/irregularT.irregularT[nb_images]+ofx,(int)(dy+ofy-(timeNodes[t].node[nodeK].mean[displayDim]-minValue)*zoom[displayDim]), irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx+eps, (int)(dy+ofy-(timeNodes[t-1].node[timeNodes[t].node[nodeK].ingoingEdges[edgeK].indexNodeA].mean[displayDim]-minValue)*zoom[displayDim]));

			}
				    
		 	
		    	
			thickness=(float)(Math.log((double)timeNodes[t].node[nodeK].ingoingEdges[edgeK].flow[(displayClass[disp2]-1)]+(double)1)/2.);
			minEdge=(IndexDivMin2[t]==nodeK)&&(IndexDivMin2[t-1]==timeNodes[t].node[nodeK].ingoingEdges[edgeK].indexNodeA);
		        if (minEdge&&SeuilDiv<=0){
thickness=(float)(Math.log((double)timeNodes[t].node[nodeK].ingoingEdges[edgeK].flowSave[(displayClass[disp2]-1)]+(double)1)/2.);
			}
			if (thickness!=0||minEdge){
			            //edge above div treshold but most likely
			            if (minEdge&&thickness==0){ 
				         //edge of interest
				   	 for (int kSB=0;kSB<sBList.size;kSB++) {
					 	if (t>sBList.time[kSB]&&t<(sBList.time[kSB]+sBList.tWindow)){
						    if((displayClass[disp1]-1)==sBList.MTa[kSB]){
							if ((displayClass[disp2]-1)==sBList.MTb[kSB]){
								alpha=(int)(sBList.Prob[kSB]*255.);
								g2.setColor(new Color(10, 255, 10, alpha));
								g2.fillRect(irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy,irregularT.irregularT[t+1]*dx/irregularT.irregularT[nb_images]-irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images], ofy+20);
								if (t==sBList.time[kSB]+1){g2.setColor(Color.black);g2.drawString(""+((float)(int)(sBList.Prob[kSB]*(float)100)/(float)100),irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy+30);g2.drawString(""+irregularT.irregularT[t],irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy+40);g2.setColor(new Color(10, 255, 10, alpha));}
							}
						    }
						    if((displayClass[disp2]-1)==sBList.MTa[kSB]){
							if ((displayClass[disp1]-1)==sBList.MTb[kSB]){
								alpha=(int)(sBList.Prob[kSB]*255.);
								g2.setColor(new Color(10, 255, 10, alpha));
								g2.fillRect(irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy,irregularT.irregularT[t+1]*dx/irregularT.irregularT[nb_images]-irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images], ofy+20);
								if (t==sBList.time[kSB]+1){g2.setColor(Color.black);g2.drawString(""+((float)(int)(sBList.Prob[kSB]*(float)100)/(float)100),irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy+30);g2.drawString(""+irregularT.irregularT[t],irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy+40);g2.setColor(new Color(10, 255, 10, alpha));}	

							}
						    }
						}
					 }
					 //all edge
				         g2.setColor(Color.orange);
					 float dash1[] = {10.0f};
              			  	 BasicStroke bs = new BasicStroke(1, BasicStroke.CAP_BUTT,BasicStroke.JOIN_MITER, 10.0f, dash1, 0.0f);
             			   	 g2.setStroke(bs);
				    }
				    //edge bellow div treshold 
              			    else{
		 		    	 //edge of interest
				   	 for (int kSB=0;kSB<sBList.size;kSB++) {
					 	if (t>sBList.time[kSB]&&t<(sBList.time[kSB]+sBList.tWindow)){
						    if((displayClass[disp1]-1)==sBList.MTa[kSB]){
							if ((displayClass[disp2]-1)==sBList.MTb[kSB]){
								alpha=(int)(sBList.Prob[kSB]*255.);
								g2.setColor(new Color(10, 255, 10, alpha));
															g2.fillRect(irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy,irregularT.irregularT[t+1]*dx/irregularT.irregularT[nb_images]-irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images], ofy+20);
								if (t==sBList.time[kSB]+1){g2.setColor(Color.black);g2.drawString(""+((float)(int)(sBList.Prob[kSB]*(float)100)/(float)100),irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy+30);g2.drawString(""+irregularT.irregularT[t],irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy+40);g2.setColor(new Color(10, 255, 10, alpha));}
							}
						    }
						    if((displayClass[disp2]-1)==sBList.MTa[kSB]){
							if ((displayClass[disp1]-1)==sBList.MTb[kSB]){
								alpha=(int)(sBList.Prob[kSB]*255.);
								g2.setColor(new Color(10, 255, 10, alpha));
															g2.fillRect(irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy,irregularT.irregularT[t+1]*dx/irregularT.irregularT[nb_images]-irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images], ofy+20);
								if (t==sBList.time[kSB]+1){g2.setColor(Color.black);g2.drawString(""+((float)(int)(sBList.Prob[kSB]*(float)100)/(float)100),irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy+30);g2.drawString(""+irregularT.irregularT[t],irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy+40);g2.setColor(new Color(10, 255, 10, alpha));}
							}
						    }
		
		
						}
					 }
					 //all edge
				         g2.setColor(Color.orange);
					 stroke = new BasicStroke(thickness, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
                                    	 g2.setStroke(stroke); 
				    }
				    
				
g2.drawLine(irregularT.irregularT[t+1]*dx/irregularT.irregularT[nb_images]+ofx,(int)(dy+ofy-(timeNodes[t].node[nodeK].mean[displayDim]-minValue)*zoom[displayDim]), irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx+eps, (int)(dy+ofy-(timeNodes[t-1].node[timeNodes[t].node[nodeK].ingoingEdges[edgeK].indexNodeA].mean[displayDim]-minValue)*zoom[displayDim]));
			}		 				
		    }
		
//current subGraph 
		    if (sBList.size!=0){
		    	   if (t>sBList.time[indexList]&&t<(sBList.time[indexList]+sBList.tWindow)){
			       if((displayClass[disp1]-1)==sBList.MTa[indexList]){
			   	   if ((displayClass[disp2]-1)==sBList.MTb[indexList]){
			   		   alpha=(int)(sBList.Prob[indexList]*255.);
			   		   g2.setColor(new Color(255, 255, 10, alpha));
			   			   g2.fillRect(irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy,irregularT.irregularT[t+1]*dx/irregularT.irregularT[nb_images]-irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images], ofy+20);
					  
			   	   }
			       }
			       if((displayClass[disp2]-1)==sBList.MTa[indexList]){
			   	   if ((displayClass[disp1]-1)==sBList.MTb[indexList]){
			   		   alpha=(int)(sBList.Prob[indexList]*255.);
			   		   g2.setColor(new Color(255, 255, 10, alpha));
			   			   g2.fillRect(irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,ofy,irregularT.irregularT[t+1]*dx/irregularT.irregularT[nb_images]-irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images], ofy+20);
					  
			   	   }
			       }
			   }
		    }	

		
		
		
		}
		
//Node attributes
		
		//System.err.print(t);System.err.print(" ");
		//System.err.print(timeNodes[t].node[nodeK].divKullback[(displayClass[disp1]-1)]);System.err.println("");
		
		if (timeNodes[t].node[nodeK].divKullback[(displayClass[disp1]-1)]<SeuilDiv||IndexDivMin1[t]==nodeK){
			alpha=255-(int)(timeNodes[t].node[nodeK].divKullback[(displayClass[disp1]-1)]*10);
			if (alpha>255){alpha=255;}
			if (alpha<0){alpha=100;}
			g2.setColor(new Color(255, 10, 255, alpha));
			stroke = new BasicStroke(5, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
                	g2.setStroke(stroke);
			g2.drawLine(irregularT.irregularT[t+1]*dx/irregularT.irregularT[nb_images]+ofx,(int)(dy+ofy-((timeNodes[t].node[nodeK].mean[displayDim]-minValue)*zoom[displayDim]-1.5*Math.sqrt(timeNodes[t].node[nodeK].cov[displayDim][displayDim])*zoom[displayDim])),irregularT.irregularT[t+1]*dx/irregularT.irregularT[nb_images]+ofx,(int)(dy+ofy-((timeNodes[t].node[nodeK].mean[displayDim]-minValue)*zoom[displayDim]+1.5*Math.sqrt(timeNodes[t].node[nodeK].cov[displayDim][displayDim])*zoom[displayDim]))); 
		}
		if (timeNodes[t].node[nodeK].divKullback[(displayClass[disp2]-1)]<SeuilDiv||IndexDivMin2[t]==nodeK){
			alpha=255-(int)(timeNodes[t].node[nodeK].divKullback[(displayClass[disp2]-1)]*10);
			if (alpha>255){alpha=255;}
			if (alpha<0){alpha=100;}
			g2.setColor(new Color(10, 10, 255, alpha));
			stroke = new BasicStroke(5, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
                	g2.setStroke(stroke);
			g2.drawLine(irregularT.irregularT[t+1]*dx/irregularT.irregularT[nb_images]+ofx,(int)(dy+ofy-((timeNodes[t].node[nodeK].mean[displayDim]-minValue)*zoom[displayDim]-1.5*Math.sqrt(timeNodes[t].node[nodeK].cov[displayDim][displayDim])*zoom[displayDim])),irregularT.irregularT[t+1]*dx/irregularT.irregularT[nb_images]+ofx,(int)(dy+ofy-((timeNodes[t].node[nodeK].mean[displayDim]-minValue)*zoom[displayDim]+1.5*Math.sqrt(timeNodes[t].node[nodeK].cov[displayDim][displayDim])*zoom[displayDim]))); 
		}
	}
 		
//Mutual Information
	if (t>0){		
	   	    g2.setColor(Color.red);
		    stroke = new BasicStroke(1, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
                    g2.setStroke(stroke);
		    g2.drawLine(irregularT.irregularT[t+1]*dx/irregularT.irregularT[nb_images]+ofx,(int)(dy+ofy-(int)(MI[(displayClass[disp1]-1)][t-1]*10.)),irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,(int)(dy+ofy-(int)(MI[(displayClass[disp1]-1)][t-1]*10.)));
	    
		    g2.setColor(Color.orange);
		    stroke = new BasicStroke(1, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
                    g2.setStroke(stroke);
		    g2.drawLine(irregularT.irregularT[t+1]*dx/irregularT.irregularT[nb_images]+ofx,(int)(dy+ofy-(int)(MI[(displayClass[disp2]-1)][t-1]*10.)),irregularT.irregularT[t]*dx/irregularT.irregularT[nb_images]+ofx,(int)(dy+ofy-(int)(MI[(displayClass[disp2]-1)][t-1]*10.)));
	}      
      
      }
      int vertLine= irregularT.irregularT[currentTime]*dx/irregularT.irregularT[nb_images];
      g2.setColor(Color.black);
      g2.drawLine(vertLine+ofx, ofy,vertLine+ofx, dy+ofy);
      g2.drawString(dates.imageDate[currentTime],vertLine+ofx,ofy+dy-10);
      /*int auxiliaire=(int)((float)(dy+ofy)/(float)zoom[displayDim]);
      g2.drawString(auxiliaire,ofx+15,ofy+dy-10);
      auxiliaire=(int)((float)dy/(float)zoom[displayDim]);
      g2.drawString(auxiliaire,ofx+15,ofy+30);*/
}
		
  
  
    public void updateSeuil(){
    
      for (int kMT=0;kMT<nb_TimelessClass;kMT++) {
    	for (int t=0;t<nb_images;t++) {
    	   for (int indexNode=0;indexNode<timeNodes[t].totalTimeNb_node;indexNode++){
	      if (t>0){
	         for (int indexNodePre=0;indexNodePre<timeNodes[t-1].totalTimeNb_node;indexNodePre++){
			timeNodes[t].node[indexNode].ingoingEdges[indexNodePre].flow[kMT]=timeNodes[t].node[indexNode].ingoingEdges[indexNodePre].flowSave[kMT];
			if (timeNodes[t].node[indexNode].divKullback[kMT]>SeuilDiv||timeNodes[t-1].node[indexNodePre].divKullback[kMT]>SeuilDiv){
				timeNodes[t].node[indexNode].ingoingEdges[indexNodePre].flow[kMT]=0;
			} 
			if (timeNodes[t].node[indexNode].ingoingEdges[indexNodePre].flow[kMT]<SeuilFlow){
				timeNodes[t].node[indexNode].ingoingEdges[indexNodePre].flow[kMT]=0;
			}
		 }
	      }
	   }
	 }
      }
    	    
    }
    public void updateLabel(Point point, CoordinateArea ca) throws IOException {
        
/*	
** Graph interaction
*/
	updateSeuil();
	double aux=1e3;
	double auxCur;
	int auxIndex=0;
	float auxi;
	double scale=(double)dx/(double)irregularT.irregularT[nb_images];
	
	if (ca==graphCoordinateArea){
	   if (point.x<(dx+ofx) && point.x>ofx && point.y<(ofy+dy) && point.y>(ofy)){
		for (int t=0;t<nb_images;t++){
			auxCur=((double)irregularT.irregularT[t+1]*(double)scale+(double)ofx-(double)point.x);
			if ((auxCur*auxCur)<(aux*aux))
			{
				aux=auxCur; 
				auxIndex=t;
			}
			
		} 
		auxIndex++;
		currentTime=auxIndex;
		auxi=(float)(ofy+dy-point.y)/(float)zoom[displayDim];
		label3.setText(" Dim:"+displayDim+"/Day:"+ irregularT.irregularT[auxIndex]+ "/Val:" + (int)auxi);
		icon=new ImageIcon(getImage1(auxIndex));
		iconClassTL=new ImageIcon(getImage1(currentTime+1));
		imageLabel.setIcon(icon);
		classLabelTL.setIcon(iconClassTL);
		icon2MT=new ImageIcon(getImage1(auxIndex));
		iconClassTL2MT=new ImageIcon(getImage1(currentTime));
		imageLabel2MT.setIcon(icon2MT);
		classLabelTL2MT.setIcon(iconClassTL2MT);
                label.setText(labelPrefix + auxIndex);
			
	   }
	   point.x=0; point.y=0;
	   dynaClassCoordinateArea.repaint();
	   graphCoordinateArea.repaint();
	   classfileArea2MT.repaint();
	   iconClass=new ImageIcon(getImage1(currentTime-1));
	   classLabel.setIcon(iconClass);
	}
/*	
** Image interaction
*/
	int[][] classAux;
	Reader in;
	String path;
	int inside=0;
	int x,y;
	boolean continu=true;
	if (ca==imCoordinateArea){
	 
	  x=point.x-0;
	  y=point.y-0;
	  auxIndex=currentTime;
	  if (point.x<(dimx+ofx) && point.x>ofx && point.y>(0) && point.y<(0+dimy)){
	
		 mouseCoordinates.setText("("+x+ ","+y+")"); 
		 classAux=new int[dimx][dimy];
		 
		  
		     
		        path=CLASSFILEPATH_MT;
		        in=new FileReader(path);
		       	for (int i=0;i<dimx;i++)
			  for (int j=0;j<dimy;j++)
			     classAux[j][i]=in.read();
			
				 
				 displayClass[0]=classAux[x][y]+1;
			         //displayClass[1]=k;
        			 //displayClass[2]=k;
				 label4.setText("Tresh:" +SeuilDiv+"/MT:"+displayClass[0]+"&"+displayClass[1]+"/");
                                 iconClassTL=new ImageIcon(getImage1(currentTime+1));
		       		 iconClass=new ImageIcon(getImage1(currentTime-1));
				 classLabelTL.setIcon(iconClassTL);
				 classLabel.setIcon(iconClass);
                		 label.setText(labelPrefix + currentTime);
				 
				 graphCoordinateArea.repaint();
				 dynaClassCoordinateArea.repaint();
				 classfileArea2MT.repaint();
				 iconClass=new ImageIcon(getImage1(currentTime-1));
		                 classLabel.setIcon(iconClass);
				 
				 point.x=0; point.y=0;
				
		 	
				
		    
		  
	  }
	  if (point.x<(2*dimx+ofx) && point.x>(dimx+ofx) && point.y>(0) && point.y<(0+dimy)){
		
		 x=x-dimx;
		 mouseCoordinates.setText("("+x+ ","+y+")");
		 classAux=new int[dimx][dimy];
		 
		   
		     
		        path=CLASSFILEPATH_MT;
		       	in=new FileReader(path);
			for (int i=0;i<dimx;i++)
			  for (int j=0;j<dimy;j++)
			     classAux[j][i]=in.read();
			
				 
				 //displayClass[0]=k;
			         displayClass[1]=classAux[x][y]+1;
        			 //displayClass[2]=k;
				 label4.setText("Tresh:" +SeuilDiv+"/MT:"+displayClass[0]+"&"+displayClass[1]+"/");
                                 iconClassTL=new ImageIcon(getImage1(currentTime+1));
                                 iconClass=new ImageIcon(getImage1(currentTime-1));	 
				 classLabelTL.setIcon(iconClassTL);
				 classLabel.setIcon(iconClass);
                		 label.setText(labelPrefix + currentTime);
				 
				 iconClassTL2MT=new ImageIcon(getImage1(currentTime));
                                 iconClass2MT=new ImageIcon(getImage1(currentTime));	 
				 classLabelTL2MT.setIcon(iconClassTL2MT);
                		 label.setText(labelPrefix + currentTime);
				 
				 graphCoordinateArea.repaint();
				 dynaClassCoordinateArea.repaint();
				 classfileArea2MT.repaint();
				 iconClass=new ImageIcon(getImage1(currentTime-1));
				 classLabel.setIcon(iconClass);
				 
		
				 point.x=0; point.y=0;
				
		 	
				
		    
		  
	  }
	 }
	 if (ca==classfileArea){
	  auxIndex=currentTime;
	  x=point.x-0;
	  y=point.y-0;
	  if (point.x<(dimx+ofx) && point.x>ofx && point.y>(0) && point.y<(0+dimy)){
	
		 mouseCoordinates.setText("   Coordinates : ("+x+ " , "+y+")   ");
		 classAux=new int[dimx][dimy];
		 
		   
		     
		        path=CLASSFILEPATH_MT;
		 	in=new FileReader(path);
			for (int i=0;i<dimx;i++)
			  for (int j=0;j<dimy;j++)
			     classAux[j][i]=in.read();
			
				 
				 displayClass[0]=classAux[x][y]+1;
			         //displayClass[1]=k;
        			 //displayClass[2]=k;
				 label4.setText("Tresh:" +SeuilDiv+"/MT:"+displayClass[0]+"&"+displayClass[1]+"/");
                                 iconClassTL=new ImageIcon(getImage1(currentTime+1));
                                 iconClass=new ImageIcon(getImage1(currentTime-1));
				 classLabelTL.setIcon(iconClassTL);
				 classLabel.setIcon(iconClass);
                		 label.setText(labelPrefix + currentTime);
				 
				 graphCoordinateArea.repaint();
				 dynaClassCoordinateArea.repaint();
				 classfileArea2MT.repaint();
				 iconClass=new ImageIcon(getImage1(currentTime-1));
		                 classLabel.setIcon(iconClass);
				 
				
				 point.x=0; point.y=0;
				
		 	
				
		    
		  
	  }
	 }

		
    }
    public static void main(String[] args) throws Exception{
        
	form2 = new DecimalFormat("00");
	        
        try {
            UIManager.setLookAndFeel(
                UIManager.getCrossPlatformLookAndFeelClassName());
        } catch (Exception e) { }
    	JFrame frame = new JFrame("TimeSeriesDisplay Application");
        ITS_Graph3 controller = new ITS_Graph3();

        frame.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                        System.exit(0);
                }
        });
        controller.buildUI();

        Component buttonContents = controller.createButton();
	Component buttonContents2=controller.createButton2();
	Component buttonContents3=controller.createButton4();	
        Component contents = controller.createComponentsIm();
	Component contents2 = controller.createComponentsClass();
	Component contents2MT = controller.createComponentsIm2MT();
	Component contents2MT2 = controller.createComponentsClass2MT();
        Component graph = controller.createComponentsGraph();
	Component buttonContents4=controller.createButton5();
	
        JPanel subPanel=new JPanel();
        subPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
        subPanel.setLayout(new BorderLayout());
        subPanel.add(contents,BorderLayout.CENTER);
        subPanel.add(buttonContents,BorderLayout.EAST);
	subPanel.add(contents2,BorderLayout.WEST);
	JPanel subPanel2MT=new JPanel();
        subPanel2MT.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
        subPanel2MT.setLayout(new BorderLayout());
        subPanel2MT.add(contents2MT,BorderLayout.CENTER);
	subPanel2MT.add(buttonContents,BorderLayout.EAST);
	subPanel2MT.add(contents2MT2,BorderLayout.WEST);
	JPanel subPanel3=new JPanel();
	subPanel3.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
        subPanel3.setLayout(new BorderLayout());
	subPanel3.add(buttonContents2,BorderLayout.EAST);
	subPanel3.add(buttonContents3,BorderLayout.CENTER);
	subPanel3.add(buttonContents4,BorderLayout.WEST);
	
        frame.getContentPane().add(subPanel,BorderLayout.WEST);
	frame.getContentPane().add(subPanel2MT,BorderLayout.SOUTH);
	frame.getContentPane().add(subPanel3,BorderLayout.EAST);
        frame.getContentPane().add(graph,BorderLayout.NORTH);
        frame.pack();
        frame.setVisible(true);
  }
    
  private   String  getImage1 (int    index) {
      String    result = new String();
      DecimalFormat form = new DecimalFormat("00");
      result = IMAGEPATH + dates.imageDate[index] + ".jpg";
      return(result);
  }
  
  private   String  getClassfileTL (int  index1) {
      String    result = new String();
      result=CLASSFILEPATH_TL+dates.imageDate[index1]+".cl";
      return(result);
  }
  
}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
